/****************************************************************************\
* Base module for all Neo-based keyboard layouts
\****************************************************************************/

#include <windows.h>
#include "kbd.h"
#include "kbd_mod.h"
#include "keysym4utf16.h"

#if defined(_M_IA64)
#pragma section(".data")
#define ALLOC_SECTION_LDATA __declspec(allocate(".data"))
#else
#pragma data_seg(".data")
#define ALLOC_SECTION_LDATA
#endif


/* **************************************************************************************************************\
* ausVK[] - Virtual Scan Code to Virtual Key
* 
*       +---+ +---------------+ +---------------+ +---------------+   +--------------+                       
*       |T01| |F1 ¦F2 ¦F3 ¦F4 | |F5 ¦F6 ¦F7 ¦F8 | |F9 ¦F10¦F11¦F12|   |Druk¦Roll¦Paus|                       
*       +---+ +---------------+ +---------------+ +---------------+   +--------------+                       
*       +---------------------------------------------------------+   +--------------+   +---------------+   
*       |T29¦T02¦T03¦T04¦T05¦T06¦T07¦T08¦T09¦T0A¦T0B¦T0C¦T0D¦ T0E |   |Einf¦Pos1¦PgUp|   ¦Num¦ / ¦ * ¦ - ¦   
*       |---------------------------------------------------------|   |--------------|   +---+---+---+---¦   
*       |T0F¦T10¦T11¦T12¦T13¦T14¦T15¦T16¦T17¦T18¦T19¦T1A¦T1B¦ Ret |   |Entf¦Ende¦PgDn|   ¦ 7 ¦ 8 ¦ 9 ¦   ¦   
*       |-----------------------------------------------------+   |   +--------------+   +---+---+---¦   ¦   
*       | T3A ¦T1E¦T1F¦T20¦T21¦T22¦T23¦T24¦T25¦T26¦T27¦T28¦T2B¦   |                      ¦ 4 ¦ 5 ¦ 6 ¦ + ¦   
*       |---------------------------------------------------------|        +----+        +---+---+---+---¦   
*       |T2A ¦T56¦T2C¦T2D¦T2E¦T2F¦T30¦T31¦T32¦T33¦T34¦T35¦ T36    |        | Up |        ¦ 1 ¦ 2 ¦ 3 ¦   ¦   
*       |---------------------------------------------------------|   +----+----+----+   +-------+---¦   ¦   
*       | Str ¦ Fe ¦ Al ¦     Leerzeichen    ¦X38 ¦ Fe ¦ Me ¦ Str |   |Left¦Down¦ Re.¦   ¦ 0     ¦ , ¦Ent¦   
*       +---------------------------------------------------------+   +--------------+   +---------------+   
* 
\************************************************************************************************************** */


static ALLOC_SECTION_LDATA USHORT ausVK[] = {
    T00, T01, T02, T03, T04, T05, T06, T07,
    T08, T09, T0A, T0B, T0C, T0D, T0E, T0F,
    T10, T11, T12, T13, T14, T15, T16, T17,
    T18, T19, T1A, T1B, T1C, T1D, T1E, T1F,
    T20, T21, T22, T23, T24, T25, T26, T27,
    T28, T29, T2A, T2B, T2C, T2D, T2E, T2F,
    T30, T31, T32, T33, T34, T35,


// Rechtes Shift muss KBDEXT bit haben

    T36 | KBDEXT,

    T37 | KBDMULTIVK,               // numpad_* + Shift/Alt -> SnapShot

    T38, T39, T3A, T3B, T3C, T3D, T3E,
    T3F, T40, T41, T42, T43, T44,


    /* NumLock Key:
     *     KBDEXT     - VK_NUMLOCK ist Extended key
     *     KBDMULTIVK - VK_NUMLOCK oder VK_PAUSE (mit oder ohne STRG) */

    T45 | KBDEXT | KBDMULTIVK,

    T46 | KBDMULTIVK,


    /*
     * Number Pad keys:
     *     KBDNUMPAD  - digits 0-9 and decimal point.
     *     KBDSPECIAL - require special processing by Windows
     */
    
    T47 | KBDNUMPAD | KBDSPECIAL,   // Numpad 7 (Home)
    T48 | KBDNUMPAD | KBDSPECIAL,   // Numpad 8 (Up),
    T49 | KBDNUMPAD | KBDSPECIAL,   // Numpad 9 (PgUp),
    T4A,
    T4B | KBDNUMPAD | KBDSPECIAL,   // Numpad 4 (Left),
    T4C | KBDNUMPAD | KBDSPECIAL,   // Numpad 5 (Clear),
    T4D | KBDNUMPAD | KBDSPECIAL,   // Numpad 6 (Right),
    T4E,
    T4F | KBDNUMPAD | KBDSPECIAL,   // Numpad 1 (End),
    T50 | KBDNUMPAD | KBDSPECIAL,   // Numpad 2 (Down),
    T51 | KBDNUMPAD | KBDSPECIAL,   // Numpad 3 (PgDn),
    T52 | KBDNUMPAD | KBDSPECIAL,   // Numpad 0 (Ins),
    T53 | KBDNUMPAD | KBDSPECIAL,   // Numpad . (Del),

    T54, T55, T56, T57, T58, T59, T5A, T5B,
    T5C, T5D, T5E, T5F, T60, T61, T62, T63,
    T64, T65, T66, T67, T68, T69, T6A, T6B,
    T6C, T6D, T6E, T6F, T70, T71, T72, T73,
    T74, T75, T76, T77, T78, T79, T7A, T7B,
    T7C, T7D, T7E

};


static ALLOC_SECTION_LDATA VSC_VK aE0VscToVk[] = {
        { 0x10, X10 | KBDEXT              },  // Speedracer: Previous Track
        { 0x19, X19 | KBDEXT              },  // Speedracer: Next Track
        { 0x1D, X1D | KBDEXT              },  // RControl
        { 0x20, X20 | KBDEXT              },  // Speedracer: Volume Mute
        { 0x21, X21 | KBDEXT              },  // Speedracer: Launch App 2
        { 0x22, X22 | KBDEXT              },  // Speedracer: Media Play/Pause
        { 0x24, X24 | KBDEXT              },  // Speedracer: Media Stop
        { 0x2E, X2E | KBDEXT              },  // Speedracer: Volume Down
        { 0x30, X30 | KBDEXT              },  // Speedracer: Volume Up
        { 0x32, X32 | KBDEXT              },  // Speedracer: Browser Home
        { 0x35, X35 | KBDEXT              },  // Numpad Divide
        { 0x37, X37 | KBDEXT              },  // Snapshot
        { 0x38, X38 | KBDEXT              },  // RMenu
        { 0x47, X47 | KBDEXT              },  // Home
        { 0x48, X48 | KBDEXT              },  // Up
        { 0x49, X49 | KBDEXT              },  // Prior
        { 0x4B, X4B | KBDEXT              },  // Left
        { 0x4D, X4D | KBDEXT              },  // Right
        { 0x4F, X4F | KBDEXT              },  // End
        { 0x50, X50 | KBDEXT              },  // Down
        { 0x51, X51 | KBDEXT              },  // Next
        { 0x52, X52 | KBDEXT              },  // Insert
        { 0x53, X53 | KBDEXT              },  // Delete
        { 0x5B, X5B | KBDEXT              },  // Left Win
        { 0x5C, X5C | KBDEXT              },  // Right Win
        { 0x5D, X5D | KBDEXT              },  // Application
        { 0x5F, X5F | KBDEXT              },  // Speedracer: Sleep
        { 0x65, X65 | KBDEXT              },  // Speedracer: Browser Search
        { 0x66, X66 | KBDEXT              },  // Speedracer: Browser Favorites
        { 0x67, X67 | KBDEXT              },  // Speedracer: Browser Refresh
        { 0x68, X68 | KBDEXT              },  // Speedracer: Browser Stop
        { 0x69, X69 | KBDEXT              },  // Speedracer: Browser Forward
        { 0x6A, X6A | KBDEXT              },  // Speedracer: Browser Back
        { 0x6B, X6B | KBDEXT              },  // Speedracer: Launch App 1
        { 0x6C, X6C | KBDEXT              },  // Speedracer: Launch Mail
        { 0x6D, X6D | KBDEXT              },  // Speedracer: Launch Media Selector
        { 0x1C, X1C | KBDEXT              },  // Numpad Enter
        { 0x46, X46 | KBDEXT              },  // Break (Ctrl + Pause)
        { 0,      0                       }
};


static ALLOC_SECTION_LDATA VSC_VK aE1VscToVk[] = {
        { 0x1D, Y1D                       },  // Pause
        { 0   ,   0                       }
};


/* **************************************************************************\
* aVkToBits[]  - map Virtual Keys to Modifier Bits
*
* Siehe kbd.h für mehr Infos
\************************************************************************** */

// Es wird nicht zwischen linken und/oder rechtem Modifier unterschieden

static ALLOC_SECTION_LDATA VK_TO_BIT aVkToBits[] = {
    { VK_SHIFT   , KBDSHIFT },
    { VK_CONTROL , KBDCTRL  },
    { VK_MENU    , KBDALT   },
    { VK_OEM_8   , KBDKANA  }, //Mod 4
    { VK_OEM_102 , 16       }, //Mod 3
    { 0          ,  0       }
};


/* **************************************************************************\
* aModification[]  - map character modifier bits to modification number
*
* Siehe kbd.h für mehr Infos
\************************************************************************** */

static ALLOC_SECTION_LDATA MODIFIERS CharModifiers = {
    &aVkToBits[0],
    24, // Maximaler Wert, den die Modifier-Bitmaske annehmen kann
    {
    //  Modifier NEO 
    //  Ebene 0 - nix
    //  Ebene 1 - Shift
    //  Ebene 2 - Neu = Mod 3
    //  Ebene 3 - Kana = Mod 4
    //  Ebene 4 - Neu+Shift 
    //  Ebene 5 - Kasa+Neu
    //  
    //  Modification# //  Keys Pressed
    //  ============= // =======================================
                      // Neu     Kana    Alt     Strg    Shift
        0,            // 0       0       0       0       0
        1,            // 0       0       0       0       1
        6,            // 0       0       0       1       0
        7,            // 0       0       0       1       1
        SHFT_INVALID, // 0       0       1       0       0
        SHFT_INVALID, // 0       0       1       0       1
        SHFT_INVALID, // 0       0       1       1       0
        SHFT_INVALID, // 0       0       1       1       1
        3,            // 0       1       0       0       0
        8,            // 0       1       0       0       1
        SHFT_INVALID, // 0       1       0       1       0
        SHFT_INVALID, // 0       1       0       1       1
        SHFT_INVALID, // 0       1       1       0       0
        SHFT_INVALID, // 0       1       1       0       1
        SHFT_INVALID, // 0       1       1       1       0
        SHFT_INVALID, // 0       1       1       1       1
        2,            // 1       0       0       0       0
        4,            // 1       0       0       0       1
        SHFT_INVALID, // 1       0       0       1       0
        SHFT_INVALID, // 1       0       0       1       1
        SHFT_INVALID, // 1       0       1       0       0
        SHFT_INVALID, // 1       0       1       0       1
        SHFT_INVALID, // 1       0       1       1       0
        SHFT_INVALID, // 1       0       1       1       1
        5,            // 1       1       0       0       0
    }      
};

// Aufgrund der Eigenheit, dass Shift bei aktiviertem Numlock den Cursorblock aktiviert (entspricht Ebene 4)
// wurden für die Tasten 0-9 und Komma die Ebenen 2 und 4 getauscht. Dies gilt nicht für die Tasten
// VK_ADD, VK_DIVIDE, VK_MULTIPLY and VK_SUBTRACT. Ebenen 5 und 6 fehlen, Num ist nicht belegt.
static ALLOC_SECTION_LDATA VK_TO_WCHARS4 aVkToWch4[] = {
//           | CapsLock |         |  SHIFT  |  NEU                |  KANA            |
//           |==========|=========|=========|=====================|==================|
// Originalbelegung 1, 2, 3, 4
{VK_ADD      ,0         ,'+'      ,'+'      ,plusminus            ,minusorplus       },
{VK_DIVIDE   ,0         ,'/'      ,'/'      ,division             ,fractionslash     },
{VK_MULTIPLY ,0         ,'*'      ,'*'      ,dotoperator          ,multiply          },
{VK_SUBTRACT ,0         ,'-'      ,'-'      ,minussign            ,setminus          },
// Belegung: 1, 4, (Windows verwendet bei Shift automatisch den Cursorblock), 3, 2
{VK_DECIMAL  ,0         ,','      ,WCH_NONE ,','                  ,'.'               },
{VK_NUMPAD0  ,0         ,'0'      ,WCH_NONE ,percent              ,signifblank       },
{VK_NUMPAD1  ,0         ,'1'      ,WCH_NONE ,leftrightarrow       ,diamond           },
{VK_NUMPAD2  ,0         ,'2'      ,WCH_NONE ,downarrow            ,heart             },
{VK_NUMPAD3  ,0         ,'3'      ,WCH_NONE ,rightoverleftharpoon ,wordjoiner        },
{VK_NUMPAD4  ,0         ,'4'      ,WCH_NONE ,leftarrow            ,club              },
{VK_NUMPAD5  ,0         ,'5'      ,WCH_NONE ,':'                  ,EuroSign          },
{VK_NUMPAD6  ,0         ,'6'      ,WCH_NONE ,rightarrow           ,triangularbullets },
{VK_NUMPAD7  ,0         ,'7'      ,WCH_NONE ,updownarrow          ,heavycheckmark    },
{VK_NUMPAD8  ,0         ,'8'      ,WCH_NONE ,uparrow              ,heavyballotx      },
{VK_NUMPAD9  ,0         ,'9'      ,WCH_NONE ,circledtimes         ,dagger            },
{0           ,0         ,0        ,0        ,0                    ,0                 }
};


static ALLOC_SECTION_LDATA DEADKEY aDeadKey[] = {
// Schema:
//    Deadtrans( Name oder Unicode der toten Taste,
//               Name oder Unicode der normalen Taste,
//               Name oder Unicode der zu bildenden Taste,
//               0x0000 für  sichtbar, 0x0001 für tot)
//    0, 0    terminiert komplette Liste
//    
//    Bei Doppelbelegungen wird erster Treffer genommen
//
//
// Deadkeys
// Nachfolgend Tafeln für die diakritschen Zeichen
// Kombinationen mit einem Diakritika und Compose mit 2 Zeichen. Der Rest ist im Deutschen selten 
// und lässt sich über das Combiningzeichen (nachgestellt) bilden
// Mehr dazu siehe https://neo-layout.org/Benutzerhandbuch/Tote-Tasten-und-Compose/
//
// =========================================================================
// TASTE 1: ZIRKUMFLEX, HATSCHEK, DREHEN, PUNKT DRÜBER, HAKEN, PUNKT DRUNTER
// Zirkumflex und Superscript ( fertig für en_US.UTF-8 und lang.module)
DEADTRANS( L'^'   , space  , L'^'   , 0x0000), //Zirkumflex
DEADTRANS( L'^'   , L'^'   , 0x0302 , 0x0000), //2x für Combining
DEADTRANS( L'^'   , L'A'   , 0x00c2 , 0x0000),
DEADTRANS( L'^'   , L'a'   , 0x00e2 , 0x0000),
DEADTRANS( L'^'   , L'C'   , 0x0108 , 0x0000),
DEADTRANS( L'^'   , L'c'   , 0x0109 , 0x0000),
DEADTRANS( L'^'   , L'E'   , 0x00ca , 0x0000),
DEADTRANS( L'^'   , L'e'   , 0x00ea , 0x0000),
DEADTRANS( L'^'   , L'G'   , 0x011c , 0x0000),
DEADTRANS( L'^'   , L'g'   , 0x011d , 0x0000),
DEADTRANS( L'^'   , L'H'   , 0x0124 , 0x0000),
DEADTRANS( L'^'   , L'h'   , 0x0125 , 0x0000),
DEADTRANS( L'^'   , L'I'   , 0x00ce , 0x0000),
DEADTRANS( L'^'   , L'i'   , 0x00ee , 0x0000),
DEADTRANS( L'^'   , L'J'   , 0x0134 , 0x0000),
DEADTRANS( L'^'   , L'j'   , 0x0135 , 0x0000),
DEADTRANS( L'^'   , L'O'   , 0x00d4 , 0x0000),
DEADTRANS( L'^'   , L'o'   , 0x00f4 , 0x0000),
DEADTRANS( L'^'   , L'S'   , 0x015c , 0x0000),
DEADTRANS( L'^'   , L's'   , 0x015d , 0x0000),
DEADTRANS( L'^'   , L'U'   , 0x00db , 0x0000),
DEADTRANS( L'^'   , L'u'   , 0x00fb , 0x0000),
DEADTRANS( L'^'   , L'W'   , 0x0174 , 0x0000),
DEADTRANS( L'^'   , L'w'   , 0x0175 , 0x0000),
DEADTRANS( L'^'   , L'Y'   , 0x0176 , 0x0000),
DEADTRANS( L'^'   , L'y'   , 0x0177 , 0x0000),
DEADTRANS( L'^'   , L'Z'   , 0x1e90 , 0x0000),
DEADTRANS( L'^'   , L'z'   , 0x1e91 , 0x0000),
DEADTRANS( L'^'   , L'?'   , 0x02c0 , 0x0000), // ab hier lang.module 
DEADTRANS( L'^'   , multiply , 0x02c0 , 0x0000),
DEADTRANS( L'^'   , Greek_alpha , 0x1d45 , 0x0000), //Greek_alpha
DEADTRANS( L'^'   , Greek_epsilon , 0x1d4b , 0x0000), //Greek_epsilon
DEADTRANS( L'^'   , Greek_upsilon , 0x1db7 , 0x0000), //Greek_upsilon
DEADTRANS( L'^'   , scriptphi , 0x1db2 , 0x0000), // Ende lang.module
DEADTRANS( L'^'   , L'1'   , onesuperior , 0x0000), //ab hier hochgestelltes
DEADTRANS( L'^'   , L'2'   , twosuperior , 0x0000),
DEADTRANS( L'^'   , L'3'   , threesuperior , 0x0000),
DEADTRANS( L'^'   , L'4'   , 0x2074 , 0x0000),
DEADTRANS( L'^'   , L'5'   , 0x2075 , 0x0000),
DEADTRANS( L'^'   , L'6'   , 0x2076 , 0x0000),
DEADTRANS( L'^'   , L'7'   , 0x2077 , 0x0000),
DEADTRANS( L'^'   , L'8'   , 0x2078 , 0x0000),
DEADTRANS( L'^'   , L'9'   , 0x2079 , 0x0000),
DEADTRANS( L'^'   , L'0'   , 0x2070 , 0x0000),
DEADTRANS( L'^'   , L'+'   , 0x207a , 0x0000),
DEADTRANS( L'^'   , L'-'   , 0x207b , 0x0000),
DEADTRANS( L'^'   , L'='   , 0x207c , 0x0000),
DEADTRANS( L'^'   , L'('   , 0x207d , 0x0000),
DEADTRANS( L'^'   , L')'   , 0x207e , 0x0000),
DEADTRANS( L'^'   , L'n'   , 0x207f , 0x0000),

// Caron ( fertig für en_US.UTF-8 und lang.module)
DEADTRANS( caron , space  , caron  , 0x0000), //Caron 
DEADTRANS( caron , caron  , 0x030C , 0x0000), //2x für Combining
DEADTRANS( caron , L'A'   , 0x01CD , 0x0000),
DEADTRANS( caron , L'a'   , 0x01CE , 0x0000),
DEADTRANS( caron , L'C'   , 0x010c , 0x0000),
DEADTRANS( caron , L'c'   , 0x010d , 0x0000),
DEADTRANS( caron , L'D'   , 0x010e , 0x0000),
DEADTRANS( caron , L'd'   , 0x010f , 0x0000),
DEADTRANS( caron , L'E'   , 0x011a , 0x0000),
DEADTRANS( caron , L'e'   , 0x011b , 0x0000),
DEADTRANS( caron , L'G'   , 0x01e6 , 0x0000),
DEADTRANS( caron , L'g'   , 0x01e7 , 0x0000),
DEADTRANS( caron , L'H'   , 0x021e , 0x0000),
DEADTRANS( caron , L'h'   , 0x021f , 0x0000),
DEADTRANS( caron , L'I'   , 0x01cf , 0x0000),
DEADTRANS( caron , L'i'   , 0x01d0 , 0x0000),
DEADTRANS( caron , L'j'   , 0x01f0 , 0x0000),
DEADTRANS( caron , L'K'   , 0x01e8 , 0x0000),
DEADTRANS( caron , L'k'   , 0x01e9 , 0x0000),
DEADTRANS( caron , L'L'   , 0x013d , 0x0000),
DEADTRANS( caron , L'l'   , 0x013e , 0x0000),
DEADTRANS( caron , L'N'   , 0x0147 , 0x0000),
DEADTRANS( caron , L'n'   , 0x0148 , 0x0000),
DEADTRANS( caron , L'O'   , 0x01d1 , 0x0000),
DEADTRANS( caron , L'o'   , 0x01d2 , 0x0000),
DEADTRANS( caron , L'R'   , 0x0158 , 0x0000),
DEADTRANS( caron , L'r'   , 0x0159 , 0x0000),
DEADTRANS( caron , L'S'   , 0x0160 , 0x0000),
DEADTRANS( caron , L's'   , 0x0161 , 0x0000),
DEADTRANS( caron , L'T'   , 0x0164 , 0x0000),
DEADTRANS( caron , L't'   , 0x0165 , 0x0000),
DEADTRANS( caron , L'U'   , 0x01d3 , 0x0000),
DEADTRANS( caron , L'u'   , 0x01d4 , 0x0000),
DEADTRANS( caron , udiaeresis , 0x01da , 0x0000), // Ü
DEADTRANS( caron , Udiaeresis , 0x01d9 , 0x0000), // ü
DEADTRANS( caron , L'Z'   , 0x017d , 0x0000),
DEADTRANS( caron , L'z'   , 0x017e , 0x0000),
DEADTRANS( caron , L'1'   , onesubscript , 0x0000), // tiefgestellt
DEADTRANS( caron , L'2'   , twosubscript , 0x0000),
DEADTRANS( caron , L'3'   , threesubscript , 0x0000),
DEADTRANS( caron , L'4'   , 0x2084 , 0x0000),
DEADTRANS( caron , L'5'   , 0x2085 , 0x0000),
DEADTRANS( caron , L'6'   , 0x2086 , 0x0000),
DEADTRANS( caron , L'7'   , 0x2087 , 0x0000),
DEADTRANS( caron , L'8'   , 0x2088 , 0x0000),
DEADTRANS( caron , L'9'   , 0x2089 , 0x0000),
DEADTRANS( caron , L'0'   , zerosubscript , 0x0000),
DEADTRANS( caron , L'+'   , 0x208a , 0x0000),
DEADTRANS( caron , L'-'   , 0x208b , 0x0000),
DEADTRANS( caron , L'='   , 0x208c , 0x0000),
DEADTRANS( caron , L'('   , 0x208d , 0x0000),
DEADTRANS( caron , L')'   , 0x208e , 0x0000),
DEADTRANS( caron , L'x'   , 0x2093 , 0x0000), // Ende tiefgestellt

//Drehen ( fertig für en_US.UTF-8 und lang.module)
DEADTRANS( totdreh , space  , totdreh , 0x0000), //Drehen-Symbol
DEADTRANS( totdreh , L'3'   , 0x0190 , 0x0000), //ab hier lang.module
DEADTRANS( totdreh , L'a'   , 0x0250 , 0x0000),
DEADTRANS( totdreh , L'C'   , 0x0186 , 0x0000),
DEADTRANS( totdreh , L'c'   , 0x0254 , 0x0000),
DEADTRANS( totdreh , L'e'   , 0x0258 , 0x0000),
DEADTRANS( totdreh , L'g'   , 0x1d77 , 0x0000),
DEADTRANS( totdreh , L'h'   , 0x0265 , 0x0000),
DEADTRANS( totdreh , L'm'   , 0x026F , 0x0000),
DEADTRANS( totdreh , L'R'   , 0x0281 , 0x0000),
DEADTRANS( totdreh , L'r'   , 0x0279 , 0x0000),
DEADTRANS( totdreh , L'S'   , 0x01A7 , 0x0000),
DEADTRANS( totdreh , L'V'   , 0x0245 , 0x0000),
DEADTRANS( totdreh , L'v'   , 0x028C , 0x0000),
DEADTRANS( totdreh , L'w'   , 0x028D , 0x0000),
DEADTRANS( totdreh , L'y'   , 0x028E , 0x0000),
DEADTRANS( totdreh , Greek_alpha , 0x0252 , 0x0000),
DEADTRANS( totdreh , Greek_lamda , 0x028E , 0x0000),
DEADTRANS( totdreh , Greek_OMEGA , 0x01b1 , 0x0000),
DEADTRANS( totdreh , Greek_tau , 0x027e , 0x0000),
DEADTRANS( totdreh , Greek_delta , 0x018d , 0x0000),
DEADTRANS( totdreh , L')'   , 0x203f , 0x0000),
DEADTRANS( totdreh , L'('   , 0x0361 , 0x0000),
DEADTRANS( totdreh , variation , 0x0223 , 0x0000), //Ende lang.module

//Punkt Drüber & Mittenpunkt  ( fertig für en_US.UTF-8 und lang.module)
DEADTRANS( abovedot , space  , abovedot , 0x0000), //Punkt Drüber
DEADTRANS( abovedot , abovedot , 0x0307 , 0x0000), //2x für Combining 
DEADTRANS( abovedot , L'L'   , 0x013F , 0x0000), //Mittenpunkt
DEADTRANS( abovedot , L'l'   , 0x0140 , 0x0000), //Mittenpunkt
DEADTRANS( abovedot , slong  , 0x1E9B , 0x0000), // Lang-s mit Punkt drüber
DEADTRANS( abovedot , L'A'   , 0x0226 , 0x0000),
DEADTRANS( abovedot , L'a'   , 0x0227 , 0x0000),
DEADTRANS( abovedot , L'B'   , 0x1e02 , 0x0000),
DEADTRANS( abovedot , L'b'   , 0x1e03 , 0x0000),
DEADTRANS( abovedot , L'C'   , 0x010a , 0x0000),
DEADTRANS( abovedot , L'c'   , 0x010b , 0x0000),
DEADTRANS( abovedot , L'D'   , 0x1e0a , 0x0000),
DEADTRANS( abovedot , L'd'   , 0x1e0b , 0x0000),
DEADTRANS( abovedot , L'E'   , 0x0116 , 0x0000),
DEADTRANS( abovedot , L'e'   , 0x0117 , 0x0000),
DEADTRANS( abovedot , L'f'   , 0x1e1e , 0x0000),
DEADTRANS( abovedot , L'F'   , 0x1e1f , 0x0000),
DEADTRANS( abovedot , L'G'   , 0x0120 , 0x0000),
DEADTRANS( abovedot , L'g'   , 0x0121 , 0x0000),
DEADTRANS( abovedot , L'H'   , 0x1e22 , 0x0000),
DEADTRANS( abovedot , L'h'   , 0x1e23 , 0x0000),
DEADTRANS( abovedot , L'I'   , 0x0130 , 0x0000),
DEADTRANS( abovedot , L'i'   , 0x0131 , 0x0000), // i ohne Punkt
DEADTRANS( abovedot , L'M'   , 0x1e40 , 0x0000),
DEADTRANS( abovedot , L'm'   , 0x1e41 , 0x0000),
DEADTRANS( abovedot , L'N'   , 0x1e44 , 0x0000),
DEADTRANS( abovedot , L'n'   , 0x1e45 , 0x0000),
DEADTRANS( abovedot , L'O'   , 0x022e , 0x0000),
DEADTRANS( abovedot , L'o'   , 0x022f , 0x0000),
DEADTRANS( abovedot , L'P'   , 0x1e56 , 0x0000),
DEADTRANS( abovedot , L'p'   , 0x1e57 , 0x0000),
DEADTRANS( abovedot , L'R'   , 0x1e58 , 0x0000),
DEADTRANS( abovedot , L'r'   , 0x1e59 , 0x0000),
DEADTRANS( abovedot , L'S'   , 0x1e60 , 0x0000),
DEADTRANS( abovedot , L's'   , 0x1e61 , 0x0000),
DEADTRANS( abovedot , L'T'   , 0x1e6a , 0x0000),
DEADTRANS( abovedot , L't'   , 0x1e6b , 0x0000),
DEADTRANS( abovedot , L'W'   , 0x1e86 , 0x0000),
DEADTRANS( abovedot , L'w'   , 0x1e87 , 0x0000),
DEADTRANS( abovedot , L'X'   , 0x1e8a , 0x0000),
DEADTRANS( abovedot , L'x'   , 0x1e8b , 0x0000),
DEADTRANS( abovedot , L'Y'   , 0x1e8e , 0x0000),
DEADTRANS( abovedot , L'y'   , 0x1e8f , 0x0000),
DEADTRANS( abovedot , L'Z'   , 0x017b , 0x0000),
DEADTRANS( abovedot , L'z'   , 0x017c , 0x0000),
DEADTRANS( abovedot , L'0'   , 0x0298 , 0x0000), //lang.module

//Rhotic hook ( fertig für en_US.UTF-8 und lang.module)
DEADTRANS( rhotichook , space  , rhotichook , 0x0000), // Rhotic Hook
DEADTRANS( rhotichook , L'B'   , 0x0181 , 0x0000), // ab hier lang.module
DEADTRANS( rhotichook , L'b'   , 0x0253 , 0x0000),
DEADTRANS( rhotichook , L'C'   , 0x0187 , 0x0000),
DEADTRANS( rhotichook , L'c'   , 0x0188 , 0x0000),
DEADTRANS( rhotichook , L'D'   , 0x018A , 0x0000),
DEADTRANS( rhotichook , L'd'   , 0x0257 , 0x0000),
DEADTRANS( rhotichook , L'F'   , 0x0191 , 0x0000),
DEADTRANS( rhotichook , L'f'   , 0x0192 , 0x0000),
DEADTRANS( rhotichook , L'G'   , 0x0193 , 0x0000),
DEADTRANS( rhotichook , L'g'   , 0x0260 , 0x0000),
DEADTRANS( rhotichook , L'h'   , 0x0266 , 0x0000),
DEADTRANS( rhotichook , L'I'   , 0x0196 , 0x0000),
DEADTRANS( rhotichook , L'i'   , 0x0269 , 0x0000),
DEADTRANS( rhotichook , L'j'   , 0x029D , 0x0000),
DEADTRANS( rhotichook , L'K'   , 0x0198 , 0x0000),
DEADTRANS( rhotichook , L'k'   , 0x0199 , 0x0000),
DEADTRANS( rhotichook , L'l'   , 0x026C , 0x0000),
DEADTRANS( rhotichook , L'M'   , 0x2C6E , 0x0000),
DEADTRANS( rhotichook , L'm'   , 0x0271 , 0x0000),
DEADTRANS( rhotichook , L'N'   , 0x014A , 0x0000),
DEADTRANS( rhotichook , L'n'   , 0x014B , 0x0000),
DEADTRANS( rhotichook , L'O'   , 0x01A0 , 0x0000),
DEADTRANS( rhotichook , L'o'   , 0x01A1 , 0x0000),
DEADTRANS( rhotichook , L'P'   , 0x01A4 , 0x0000),
DEADTRANS( rhotichook , L'p'   , 0x01A5 , 0x0000),
DEADTRANS( rhotichook , L'R'   , 0x2C64 , 0x0000),
DEADTRANS( rhotichook , L'r'   , 0x027D , 0x0000),
DEADTRANS( rhotichook , L's'   , 0x0282 , 0x0000),
DEADTRANS( rhotichook , L'T'   , 0x01AC , 0x0000),
DEADTRANS( rhotichook , L't'   , 0x01AD , 0x0000),
DEADTRANS( rhotichook , L'U'   , 0x01Af , 0x0000),
DEADTRANS( rhotichook , L'u'   , 0x01b0 , 0x0000),
DEADTRANS( rhotichook , L'v'   , 0x2C71 , 0x0000),
DEADTRANS( rhotichook , L'W'   , 0x2C72 , 0x0000),
DEADTRANS( rhotichook , L'w'   , 0x2C73 , 0x0000),
DEADTRANS( rhotichook , L'X'   , 0x0194 , 0x0000),
DEADTRANS( rhotichook , L'x'   , 0x0263 , 0x0000),
DEADTRANS( rhotichook , L'Y'   , 0x01b3 , 0x0000),
DEADTRANS( rhotichook , L'y'   , 0x01b4 , 0x0000),
DEADTRANS( rhotichook , L'z'   , 0x0290 , 0x0000),
DEADTRANS( rhotichook , L'3'   , 0x025d , 0x0000), // bis hier lang.module

//Punkt Darunter ( fertig für en_US.UTF-8 und lang.module)
DEADTRANS( L'.' , space  , L'.'     , 0x0000), //Punkt darunter
DEADTRANS( L'.' , L'.'   , 0x0323   , 0x0000), //2x für Combining
DEADTRANS( L'.' , L'A'   , 0x1ea0   , 0x0000),
DEADTRANS( L'.' , L'a'   , 0x1ea1   , 0x0000),
DEADTRANS( L'.' , L'B'   , 0x1e04   , 0x0000),
DEADTRANS( L'.' , L'b'   , 0x1e05   , 0x0000),
DEADTRANS( L'.' , L'D'   , 0x1e0c   , 0x0000),
DEADTRANS( L'.' , L'd'   , 0x1e0d   , 0x0000),
DEADTRANS( L'.' , L'E'   , 0x1eb8   , 0x0000),
DEADTRANS( L'.' , L'e'   , 0x1eb9   , 0x0000),
DEADTRANS( L'.' , L'H'   , 0x1e24   , 0x0000),
DEADTRANS( L'.' , L'h'   , 0x1e25   , 0x0000),
DEADTRANS( L'.' , L'I'   , 0x1eca   , 0x0000),
DEADTRANS( L'.' , L'i'   , 0x1ecb   , 0x0000),
DEADTRANS( L'.' , L'K'   , 0x1e32   , 0x0000),
DEADTRANS( L'.' , L'k'   , 0x1e33   , 0x0000),
DEADTRANS( L'.' , L'L'   , 0x1e36   , 0x0000),
DEADTRANS( L'.' , L'l'   , 0x1e37   , 0x0000),
DEADTRANS( L'.' , L'M'   , 0x1e42   , 0x0000),
DEADTRANS( L'.' , L'm'   , 0x1e43   , 0x0000),
DEADTRANS( L'.' , L'N'   , 0x1e46   , 0x0000),
DEADTRANS( L'.' , L'n'   , 0x1e47   , 0x0000),
DEADTRANS( L'.' , L'O'   , 0x1ecc   , 0x0000),
DEADTRANS( L'.' , L'o'   , 0x1ecd   , 0x0000),
DEADTRANS( L'.' , L'R'   , 0x1e5a   , 0x0000),
DEADTRANS( L'.' , L'r'   , 0x1e5b   , 0x0000),
DEADTRANS( L'.' , L'S'   , 0x1e62   , 0x0000),
DEADTRANS( L'.' , L's'   , 0x1e63   , 0x0000),
DEADTRANS( L'.' , L'T'   , 0x1e6c   , 0x0000),
DEADTRANS( L'.' , L't'   , 0x1e6d   , 0x0000),
DEADTRANS( L'.' , L'U'   , 0x1ee4   , 0x0000),
DEADTRANS( L'.' , L'u'   , 0x1ee5   , 0x0000),
DEADTRANS( L'.' , L'V'   , 0x1e7e   , 0x0000),
DEADTRANS( L'.' , L'v'   , 0x1e7f   , 0x0000),
DEADTRANS( L'.' , L'W'   , 0x1e88   , 0x0000),
DEADTRANS( L'.' , L'w'   , 0x1e89   , 0x0000),
DEADTRANS( L'.' , L'Y'   , 0x1ef4   , 0x0000),
DEADTRANS( L'.' , L'y'   , 0x1ef5   , 0x0000),
DEADTRANS( L'.' , L'Z'   , 0x1e92   , 0x0000),
DEADTRANS( L'.' , L'z'   , 0x1e93   , 0x0000),
// ENDE TASTE 1
// =====================================================================

// =====================================================================
// TASTE 2: GRAVIS, CEDILLE, RING, TREMA, OGONEK, MAKRON
// Gravis ( fertig für en_US.UTF-8 und lang.module)
DEADTRANS( grave , space  , grave  , 0x0000),	//Gravis
DEADTRANS( grave , grave  , 0x0300 , 0x0000),	//2x für Combining
DEADTRANS( grave , L'a'   , 0x00e0 , 0x0000),			
DEADTRANS( grave , L'A'   , 0x00c0 , 0x0000),			
DEADTRANS( grave , L'E'   , 0x00c8 , 0x0000),			
DEADTRANS( grave , L'e'   , 0x00e8 , 0x0000),			
DEADTRANS( grave , L'I'   , 0x00cc , 0x0000),			
DEADTRANS( grave , L'i'   , 0x00ec , 0x0000),			
DEADTRANS( grave , L'N'   , 0x01f8 , 0x0000),			
DEADTRANS( grave , L'n'   , 0x01f9 , 0x0000),			
DEADTRANS( grave , L'O'   , 0x00d2 , 0x0000),			
DEADTRANS( grave , L'o'   , 0x00f2 , 0x0000),			
DEADTRANS( grave , L'U'   , 0x00d9 , 0x0000),			
DEADTRANS( grave , L'u'   , 0x00f9 , 0x0000),			
DEADTRANS( grave , L'W'   , 0x1e80 , 0x0000),			
DEADTRANS( grave , L'w'   , 0x1e81 , 0x0000),			
DEADTRANS( grave , L'Y'   , 0x1ef2 , 0x0000),			
DEADTRANS( grave , L'y'   , 0x1ef3 , 0x0000),			
DEADTRANS( grave , Udiaeresis , 0x01db , 0x0000),	//Ü
DEADTRANS( grave , udiaeresis , 0x01dc , 0x0000),	//ü
DEADTRANS( grave , 0x0391 , 0x1FBA , 0x0000), //Ab hier griechische Kombis
DEADTRANS( grave , Greek_alpha , 0x1F70 , 0x0000),
DEADTRANS( grave , 0x0395 , 0x1FC8 , 0x0000),
DEADTRANS( grave , Greek_epsilon , 0x1F72 , 0x0000),
DEADTRANS( grave , 0x0397 , 0x1FCA , 0x0000),
DEADTRANS( grave , Greek_eta , 0x1F74 , 0x0000),
DEADTRANS( grave , 0x0399 , 0x1FDA , 0x0000),
DEADTRANS( grave , Greek_iota , 0x1F76 , 0x0000),
DEADTRANS( grave , Greek_OMEGA , 0x1FFA , 0x0000),
DEADTRANS( grave , Greek_omega , 0x1F7C , 0x0000),
DEADTRANS( grave , 0x039f , 0x1FF8 , 0x0000),
DEADTRANS( grave , Greek_omicron , 0x1F78 , 0x0000),
DEADTRANS( grave , 0x03a5 , 0x1FEA , 0x0000),
DEADTRANS( grave , Greek_upsilon , 0x1F7A , 0x0000),
 
//Cedille ( fertig für en_US.UTF-8 und lang.module)
DEADTRANS( cedilla , space  , cedilla, 0x0000),	//Cedille
DEADTRANS( cedilla , cedilla, 0x0327 , 0x0000),	 //2x für Combining
DEADTRANS( cedilla , L'C'   , 0x00c7 , 0x0000),
DEADTRANS( cedilla , L'c'   , 0x00e7 , 0x0000),
DEADTRANS( cedilla , L'D'   , 0x1e10 , 0x0000),
DEADTRANS( cedilla , L'd'   , 0x1e11 , 0x0000),
DEADTRANS( cedilla , L'G'   , 0x0122 , 0x0000),
DEADTRANS( cedilla , L'g'   , 0x0123 , 0x0000),
DEADTRANS( cedilla , L'H'   , 0x1e28 , 0x0000),
DEADTRANS( cedilla , L'h'   , 0x1e29 , 0x0000),
DEADTRANS( cedilla , L'K'   , 0x0136 , 0x0000),
DEADTRANS( cedilla , L'k'   , 0x0137 , 0x0000),
DEADTRANS( cedilla , L'L'   , 0x013b , 0x0000),
DEADTRANS( cedilla , L'l'   , 0x013c , 0x0000),
DEADTRANS( cedilla , L'N'   , 0x0145 , 0x0000),
DEADTRANS( cedilla , L'n'   , 0x0146 , 0x0000),
DEADTRANS( cedilla , L'R'   , 0x0156 , 0x0000),
DEADTRANS( cedilla , L'r'   , 0x0157 , 0x0000),
DEADTRANS( cedilla , L'S'   , 0x015e , 0x0000),
DEADTRANS( cedilla , L's'   , 0x015f , 0x0000),
DEADTRANS( cedilla , L'T'   , 0x0162 , 0x0000),
DEADTRANS( cedilla , L't'   , 0x0163 , 0x0000),
DEADTRANS( cedilla , L'A'   , 0x0104 , 0x0000),	//Ogonek
DEADTRANS( cedilla , L'a'   , 0x0105 , 0x0000),
DEADTRANS( cedilla , L'E'   , 0x0118 , 0x0000),
DEADTRANS( cedilla , L'e'   , 0x0119 , 0x0000),
DEADTRANS( cedilla , L'I'   , 0x012e , 0x0000),
DEADTRANS( cedilla , L'i'   , 0x012f , 0x0000),
DEADTRANS( cedilla , L'O'   , 0x01ea , 0x0000),
DEADTRANS( cedilla , L'o'   , 0x01eb , 0x0000),
DEADTRANS( cedilla , L'U'   , 0x0172 , 0x0000),
DEADTRANS( cedilla , L'u'   , 0x0173 , 0x0000),

//Ring ( fertig für en_US.UTF-8 und lang.module)
DEADTRANS( ringabove , space  , ringabove , 0x0000),	//Ring
DEADTRANS( ringabove , ringabove , 0x030A , 0x0000),	//2x für Combining
DEADTRANS( ringabove , L'A'   , 0x00c5 , 0x0000),
DEADTRANS( ringabove , L'a'   , 0x00e5 , 0x0000),
DEADTRANS( ringabove , L'U'   , 0x016e , 0x0000),
DEADTRANS( ringabove , L'u'   , 0x016f , 0x0000),
DEADTRANS( ringabove , L'w'   , 0x1e98 , 0x0000),
DEADTRANS( ringabove , L'y'   , 0x1e99 , 0x0000),
DEADTRANS( ringabove , L'~'   , 0x0120 , 0x0000),
DEADTRANS( ringabove , L'c'   , 0x0255 , 0x0000), //ab hier lang.module
DEADTRANS( ringabove , L'd'   , 0x0221 , 0x0000),
DEADTRANS( ringabove , L'l'   , 0x0234 , 0x0000),
DEADTRANS( ringabove , L'n'   , 0x0235 , 0x0000),
DEADTRANS( ringabove , L't'   , 0x0236 , 0x0000),
DEADTRANS( ringabove , L'z'   , 0x0291 , 0x0000),

//Trema ( fertig für en_US.UTF-8 und lang.module)
DEADTRANS( diaeresis , space  , diaeresis , 0x0000),	//Trema
DEADTRANS( diaeresis , diaeresis , 0x0308 , 0x0000),	//2x für Combining
DEADTRANS( diaeresis , acute  , 0x0344 , 0x0001),  //Trema und Akut: COMBINING GREEK DIALYTIKA TONOS
DEADTRANS( diaeresis , L'A'   , Adiaeresis , 0x0000),
DEADTRANS( diaeresis , L'a'   , adiaeresis , 0x0000),
DEADTRANS( diaeresis , L'E'   , 0x00cb , 0x0000),
DEADTRANS( diaeresis , L'e'   , 0x00eb , 0x0000),
DEADTRANS( diaeresis , L'H'   , 0x1e26 , 0x0000),
DEADTRANS( diaeresis , L'h'   , 0x1e27 , 0x0000),
DEADTRANS( diaeresis , L'I'   , 0x00cf , 0x0000),
DEADTRANS( diaeresis , L'i'   , 0x00ef , 0x0000),
DEADTRANS( diaeresis , L'O'   , Odiaeresis , 0x0000),
DEADTRANS( diaeresis , L'o'   , 0x00f6 , 0x0000),
DEADTRANS( diaeresis , L't'   , 0x1e97 , 0x0000),
DEADTRANS( diaeresis , L'U'   , Udiaeresis , 0x0000),
DEADTRANS( diaeresis , L'u'   , udiaeresis , 0x0000),
DEADTRANS( diaeresis , L'W'   , 0x1e84 , 0x0000),
DEADTRANS( diaeresis , L'w'   , 0x1e85 , 0x0000),
DEADTRANS( diaeresis , L'X'   , 0x1e8c , 0x0000),
DEADTRANS( diaeresis , L'x'   , 0x1e8d , 0x0000),
DEADTRANS( diaeresis , L'Y'   , 0x0178 , 0x0000),
DEADTRANS( diaeresis , L'y'   , 0x00ff , 0x0000),
DEADTRANS( diaeresis , 0x0399 , 0x03aa , 0x0000), //ab hier griechisch
DEADTRANS( diaeresis , Greek_iota , 0x03ca , 0x0000),
DEADTRANS( diaeresis , 0x03a5 , 0x03ab , 0x0000),
DEADTRANS( diaeresis , Greek_upsilon , 0x03cb , 0x0000),

//Spiritus asper ( fertig für en_US.UTF-8 und lang.module)
DEADTRANS( greek_dasia , space   , greek_dasia , 0x0000),	//Spiritus asper
DEADTRANS( greek_dasia , greek_dasia , 0x0314 , 0x0000),	//2x für Combining
DEADTRANS( greek_dasia , Greek_alpha , 0x1f01 , 0x0000),//Greek_alpha
DEADTRANS( greek_dasia , 0x0391 , 0x1f09 , 0x0000),//Greek_ALPHA
DEADTRANS( greek_dasia , Greek_epsilon , 0x1f11 , 0x0000),//Greek_epsilon
DEADTRANS( greek_dasia , 0x0395 , 0x1f19 , 0x0000),//Greek_EPSILON
DEADTRANS( greek_dasia , Greek_eta , 0x1f21 , 0x0000),//Greek_eta
DEADTRANS( greek_dasia , 0x0397 , 0x1f29 , 0x0000),//Greek_ETA
DEADTRANS( greek_dasia , Greek_iota , 0x1f31 , 0x0000),//Greek_iot
DEADTRANS( greek_dasia , 0x0399 , 0x1f39 , 0x0000),//Greek_IOT
DEADTRANS( greek_dasia , Greek_omega , 0x1f61 , 0x0000),//Greek_omega
DEADTRANS( greek_dasia , Greek_OMEGA , 0x1f69 , 0x0000),//Greek_OMEGA
DEADTRANS( greek_dasia , Greek_omicron , 0x1f41 , 0x0000),//Greek_omicron
DEADTRANS( greek_dasia , 0x039f , 0x1f49 , 0x0000),//Greek_OMICRON
DEADTRANS( greek_dasia , Greek_rho , 0x1fe5 , 0x0000),//Greek_rho
DEADTRANS( greek_dasia , 0x03a1 , 0x1fec , 0x0000),//Greek_RHO
DEADTRANS( greek_dasia , Greek_upsilon , 0x1f51 , 0x0000),//Greek_upsilon
DEADTRANS( greek_dasia , 0x03a5 , 0x1f59 , 0x0000),//Greek_UPSILON

// Makron ( fertig für en_US.UTF-8 und lang.module)
DEADTRANS( macron , space  , macron , 0x0000),	//Makron
DEADTRANS( macron , macron , 0x0304 , 0x0000),	//2x für Combining
DEADTRANS( macron , L'A'   , 0x0100 , 0x0000),
DEADTRANS( macron , L'a'   , 0x0101 , 0x0000),
DEADTRANS( macron , L'E'   , 0x0112 , 0x0000),
DEADTRANS( macron , L'e'   , 0x0113 , 0x0000),
DEADTRANS( macron , L'G'   , 0x1e20 , 0x0000),
DEADTRANS( macron , L'g'   , 0x1e21 , 0x0000),
DEADTRANS( macron , L'I'   , 0x012a , 0x0000),
DEADTRANS( macron , L'i'   , 0x012b , 0x0000),
DEADTRANS( macron , L'O'   , 0x014c , 0x0000),
DEADTRANS( macron , L'o'   , 0x014d , 0x0000),
DEADTRANS( macron , L'U'   , 0x016a , 0x0000),
DEADTRANS( macron , L'u'   , 0x016b , 0x0000),
DEADTRANS( macron , L'Y'   , 0x0232 , 0x0000),
DEADTRANS( macron , L'y'   , 0x0233 , 0x0000),
DEADTRANS( macron , Adiaeresis , 0x01de , 0x0000),	//Ä
DEADTRANS( macron , adiaeresis , 0x01df , 0x0000),  //ä
DEADTRANS( macron , Udiaeresis , 0x01d5 , 0x0000),	//Ü
DEADTRANS( macron , udiaeresis , 0x01d6 , 0x0000),	//ü
DEADTRANS( macron , Odiaeresis , 0x022a , 0x0000),	//Ö
DEADTRANS( macron , 0x00f6 , 0x022b , 0x0000),	//ö
DEADTRANS( macron , 0x00c6 , 0x01E2 , 0x0000),	//Æ
DEADTRANS( macron , 0x00e6 , 0x01E3 , 0x0000),	//æ
DEADTRANS( macron , 0x0391 , 0x1FB9 , 0x0000),  //ab hier griechisch
DEADTRANS( macron , 0x0399 , 0x1FD9 , 0x0000),
DEADTRANS( macron , 0x03a5 , 0x1FE9 , 0x0000),
DEADTRANS( macron , Greek_alpha , 0x1FB1 , 0x0000),
DEADTRANS( macron , Greek_iota , 0x1FD1 , 0x0000),
DEADTRANS( macron , Greek_upsilon , 0x1FE1 , 0x0000),
DEADTRANS( macron , L'b'   , 0x1e07 , 0x0000),  // ab hier Makron darunter
DEADTRANS( macron , L'B'   , 0x1e06 , 0x0000),
DEADTRANS( macron , L'd'   , 0x1e0f , 0x0000),
DEADTRANS( macron , L'D'   , 0x1e0e , 0x0000),
DEADTRANS( macron , L'h'   , 0x1e96 , 0x0000),
DEADTRANS( macron , L'k'   , 0x1e35 , 0x0000),
DEADTRANS( macron , L'K'   , 0x1e34 , 0x0000),
DEADTRANS( macron , L'l'   , 0x1e3b , 0x0000),
DEADTRANS( macron , L'L'   , 0x1e3a , 0x0000),
DEADTRANS( macron , L'n'   , 0x1e49 , 0x0000),
DEADTRANS( macron , L'N'   , 0x1e48 , 0x0000),
DEADTRANS( macron , L'r'   , 0x1e5f , 0x0000),
DEADTRANS( macron , L'R'   , 0x1e5e , 0x0000),
DEADTRANS( macron , L't'   , 0x1e6f , 0x0000),
DEADTRANS( macron , L'T'   , 0x1e6e , 0x0000),
DEADTRANS( macron , L'z'   , 0x1e95 , 0x0000),
DEADTRANS( macron , L'Z'   , 0x1e94 , 0x0000), // Ende Makron darunter
DEADTRANS( macron , L'1'   , onesubscript , 0x0000), // tiefgestellt
DEADTRANS( macron , L'2'   , twosubscript , 0x0000),
DEADTRANS( macron , L'3'   , threesubscript , 0x0000),
DEADTRANS( macron , L'4'   , 0x2084 , 0x0000),
DEADTRANS( macron , L'5'   , 0x2085 , 0x0000),
DEADTRANS( macron , L'6'   , 0x2086 , 0x0000),
DEADTRANS( macron , L'7'   , 0x2087 , 0x0000),
DEADTRANS( macron , L'8'   , 0x2088 , 0x0000),
DEADTRANS( macron , L'9'   , 0x2089 , 0x0000),
DEADTRANS( macron , L'0'   , zerosubscript , 0x0000),
DEADTRANS( macron , L'+'   , 0x208a , 0x0000),
DEADTRANS( macron , L'-'   , 0x208b , 0x0000),
DEADTRANS( macron , L'='   , 0x208c , 0x0000),
DEADTRANS( macron , L'('   , 0x208d , 0x0000),
DEADTRANS( macron , L')'   , 0x208e , 0x0000),
DEADTRANS( macron , L'x'   , 0x2093 , 0x0000), // Ende tiefgestellt
// ENDE TASTE 2
// =====================================================================

// =====================================================================
// TASTE 3: AKUT, TILDE, QUERSTRICH, DOPPELAKUT, OGONEK, BREVE
// Akut ( fertig für en_US.UTF-8 und lang.module)
DEADTRANS( acute , space  , acute  , 0x0000),	//Akut
DEADTRANS( acute , acute  , 0x0301 , 0x0000),	//2x für Combining
DEADTRANS( acute , diaeresis , 0x0344 , 0x0001),  //Trema und Akut: COMBINING GREEK DIALYTIKA TONOS
DEADTRANS( acute , L'A'   , 0x00c1 , 0x0000),
DEADTRANS( acute , L'a'   , 0x00e1 , 0x0000),
DEADTRANS( acute , L'C'   , 0x0106 , 0x0000),
DEADTRANS( acute , L'c'   , 0x0107 , 0x0000),
DEADTRANS( acute , L'E'   , 0x00c9 , 0x0000),
DEADTRANS( acute , L'e'   , 0x00e9 , 0x0000),
DEADTRANS( acute , L'G'   , 0x01f4 , 0x0000),
DEADTRANS( acute , L'g'   , 0x01f5 , 0x0000),
DEADTRANS( acute , L'I'   , 0x00cd , 0x0000),
DEADTRANS( acute , L'i'   , 0x00ed , 0x0000),
DEADTRANS( acute , L'K'   , 0x1e30 , 0x0000),
DEADTRANS( acute , L'k'   , 0x1e31 , 0x0000),
DEADTRANS( acute , L'L'   , 0x0139 , 0x0000),
DEADTRANS( acute , L'l'   , 0x013a , 0x0000),
DEADTRANS( acute , L'M'   , 0x1e3e , 0x0000),
DEADTRANS( acute , L'm'   , 0x1e3f , 0x0000),
DEADTRANS( acute , L'N'   , 0x0143 , 0x0000),
DEADTRANS( acute , L'n'   , 0x0144 , 0x0000),
DEADTRANS( acute , L'O'   , 0x00d3 , 0x0000),
DEADTRANS( acute , L'o'   , 0x00f3 , 0x0000),
DEADTRANS( acute , L'P'   , 0x1e54 , 0x0000),
DEADTRANS( acute , L'p'   , 0x1e55 , 0x0000),
DEADTRANS( acute , L'R'   , 0x0154 , 0x0000),
DEADTRANS( acute , L'r'   , 0x0155 , 0x0000),
DEADTRANS( acute , L'S'   , 0x015a , 0x0000),
DEADTRANS( acute , L's'   , 0x015b , 0x0000),
DEADTRANS( acute , L'U'   , 0x00da , 0x0000),
DEADTRANS( acute , L'u'   , 0x00fa , 0x0000),
DEADTRANS( acute , L'W'   , 0x1e82 , 0x0000),
DEADTRANS( acute , L'w'   , 0x1e83 , 0x0000),
DEADTRANS( acute , L'Y'   , 0x00dd , 0x0000),
DEADTRANS( acute , L'y'   , 0x00fd , 0x0000),
DEADTRANS( acute , L'Z'   , 0x0179 , 0x0000),
DEADTRANS( acute , L'z'   , 0x017a , 0x0000),
DEADTRANS( acute , Udiaeresis , 0x01d7 , 0x0000),	//Ü
DEADTRANS( acute , udiaeresis , 0x01d8 , 0x0000),	//ü
DEADTRANS( acute , 0x00c6 , 0x01fc , 0x0000),	//Æ
DEADTRANS( acute , 0x00e6 , 0x01fd , 0x0000),	//æ
DEADTRANS( acute , 0x0391 , 0x0386 , 0x0000),	//ab hier griechisch
DEADTRANS( acute , 0x0395 , 0x0388 , 0x0000),
DEADTRANS( acute , 0x0397 , 0x0389 , 0x0000),
DEADTRANS( acute , 0x0399 , 0x038a , 0x0000),
DEADTRANS( acute , 0x039f , 0x038c , 0x0000),
DEADTRANS( acute , 0x03a5 , 0x038e , 0x0000),
DEADTRANS( acute , Greek_OMEGA , 0x038f , 0x0000),
DEADTRANS( acute , Greek_alpha , 0x03ac , 0x0000),
DEADTRANS( acute , Greek_epsilon , 0x03ad , 0x0000),
DEADTRANS( acute , Greek_eta , Greek_eta , 0x0000),
DEADTRANS( acute , Greek_iota , 0x03af , 0x0000),
DEADTRANS( acute , Greek_omicron , 0x03cc , 0x0000),
DEADTRANS( acute , Greek_upsilon , 0x03cd , 0x0000),
DEADTRANS( acute , Greek_omega , 0x03ce , 0x0000),

// Tilde  ( fertig für en_US.UTF-8 und lang.module)
DEADTRANS( L'~'   , space  , L'~'   , 0x0000), //Tilde
DEADTRANS( L'~'   , L'~'   , 0x0303 , 0x0000), //2x für Combining
DEADTRANS( L'~'   , L'A'   , 0x00c3 , 0x0000),
DEADTRANS( L'~'   , L'a'   , 0x00e3 , 0x0000),
DEADTRANS( L'~'   , L'E'   , 0x1ebc , 0x0000),
DEADTRANS( L'~'   , L'e'   , 0x1ebd , 0x0000),
DEADTRANS( L'~'   , L'I'   , 0x0128 , 0x0000),
DEADTRANS( L'~'   , L'i'   , 0x0129 , 0x0000),
DEADTRANS( L'~'   , L'N'   , 0x00d1 , 0x0000),
DEADTRANS( L'~'   , L'n'   , 0x00f1 , 0x0000),
DEADTRANS( L'~'   , L'O'   , 0x00d5 , 0x0000),
DEADTRANS( L'~'   , L'o'   , 0x00f5 , 0x0000),
DEADTRANS( L'~'   , L'U'   , 0x0168 , 0x0000),
DEADTRANS( L'~'   , L'u'   , 0x0169 , 0x0000),
DEADTRANS( L'~'   , L'V'   , 0x1e7c , 0x0000),
DEADTRANS( L'~'   , L'v'   , 0x1e7d , 0x0000),
DEADTRANS( L'~'   , L'Y'   , 0x1ef8 , 0x0000),
DEADTRANS( L'~'   , L'y'   , 0x1ef9 , 0x0000),
DEADTRANS( L'~'   , Greek_alpha , 0x1FB6 , 0x0000),	//ab hier griechisch
DEADTRANS( L'~'   , Greek_eta , 0x1FC6 , 0x0000),
DEADTRANS( L'~'   , Greek_iota , 0x1FD6 , 0x0000),
DEADTRANS( L'~'   , Greek_omega , 0x1FF6 , 0x0000),
DEADTRANS( L'~'   , Greek_upsilon , 0x1FE6 , 0x0000),

//Quer-/Schrägstrich ( fertig für en_US.UTF-8 und lang.module)
DEADTRANS( L'-'   , space  , nobreakhyphen , 0x0000),	//Quer-/Schrägstrich; mit Space wird echter Hyphen erzeugt
DEADTRANS( L'-'   , L'-'   , 0x0335 , 0x0000),	//2x für Combining (Combining Short Stroke)
DEADTRANS( L'-'   , L'A'   , 0x023A , 0x0000),  //lang.module
DEADTRANS( L'-'   , L'a'   , 0x2C65 , 0x0000),
DEADTRANS( L'-'   , L'B'   , 0x0243 , 0x0000),
DEADTRANS( L'-'   , L'b'   , 0x0180 , 0x0000),
DEADTRANS( L'-'   , L'C'   , 0x023B , 0x0000),
DEADTRANS( L'-'   , L'c'   , 0x023C , 0x0000),
DEADTRANS( L'-'   , L'E'   , 0x0246 , 0x0000),
DEADTRANS( L'-'   , L'e'   , 0x0247 , 0x0000),
DEADTRANS( L'-'   , L'I'   , 0x0197 , 0x0000),
DEADTRANS( L'-'   , L'J'   , 0x025F , 0x0000),
DEADTRANS( L'-'   , L'j'   , 0x025F , 0x0000),
DEADTRANS( L'-'   , L'P'   , 0x2C63 , 0x0000),
DEADTRANS( L'-'   , L'p'   , 0x1D7D , 0x0000),
DEADTRANS( L'-'   , L'R'   , 0x024C , 0x0000),
DEADTRANS( L'-'   , L'r'   , 0x024D , 0x0000),
DEADTRANS( L'-'   , L'u'   , 0x0289 , 0x0000),
DEADTRANS( L'-'   , L'U'   , 0x0244 , 0x0000),
DEADTRANS( L'-'   , L'Y'   , 0x024E , 0x0000),
DEADTRANS( L'-'   , L'y'   , 0x024F , 0x0000),
DEADTRANS( L'-'   , Greek_iota , 0x1d7c , 0x0000),  //Greek_iota
DEADTRANS( L'-'   , Greek_lamda , 0x019b , 0x0000),  //Greek_lambda
DEADTRANS( L'-'   , Greek_upsilon , 0x1d7f , 0x0000),  //Greek_upsilon
DEADTRANS( L'-'   , slong , 0x1e9c , 0x0000),  //lang-s
DEADTRANS( L'-'   , L'2'   , 0x01bb , 0x0000),  //Ende lang.module
DEADTRANS( L'-'   , L'O'   , 0x00d8 , 0x0000),	//Schrägstrich
DEADTRANS( L'-'   , L'o'   , 0x00f8 , 0x0000),
DEADTRANS( L'-'   , L'L'   , 0x0141 , 0x0000),
DEADTRANS( L'-'   , L'l'   , 0x0142 , 0x0000),

//Doppelakut ( fertig für en_US.UTF-8 und lang.module)
DEADTRANS( doubleacute , space  , doubleacute , 0x0000),	//Doppelakut
DEADTRANS( doubleacute , doubleacute , 0x030B , 0x0000),	//2x für Combining
DEADTRANS( doubleacute , L'O'   , 0x0150 , 0x0000),
DEADTRANS( doubleacute , L'o'   , 0x0151 , 0x0000),
DEADTRANS( doubleacute , L'U'   , 0x0170 , 0x0000),
DEADTRANS( doubleacute , L'u'   , 0x0171 , 0x0000),

//Spiritus Lenis ( fertig für en_US.UTF-8 und lang.module)
DEADTRANS( greek_psili , space   , greek_psili , 0x0000),  //Spiritus Lenis
DEADTRANS( greek_psili , greek_psili , 0x0313 , 0x0000),
DEADTRANS( greek_psili , Greek_alpha , 0x1F00 , 0x0000),//Greek_alpha
DEADTRANS( greek_psili , 0x0391 , 0x1F08 , 0x0000),//Greek_ALPHA
DEADTRANS( greek_psili , Greek_epsilon , 0x1F10 , 0x0000),//Greek_epsilon
DEADTRANS( greek_psili , 0x0395 , 0x1F18 , 0x0000),//Greek_EPSILON
DEADTRANS( greek_psili , Greek_eta , 0x1F20 , 0x0000),//Greek_eta
DEADTRANS( greek_psili , 0x0397 , 0x1F28 , 0x0000),//Greek_ETA
DEADTRANS( greek_psili , Greek_iota , 0x1F30 , 0x0000),//Greek_iota
DEADTRANS( greek_psili , 0x0399 , 0x1F38 , 0x0000),//Greek_IOTA
DEADTRANS( greek_psili , Greek_omega , 0x1F60 , 0x0000),//Greek_omega
DEADTRANS( greek_psili , Greek_OMEGA , 0x1F68 , 0x0000),//Greek_OMEGA
DEADTRANS( greek_psili , Greek_omicron , 0x1F40 , 0x0000),//Greek_omicron
DEADTRANS( greek_psili , 0x039f , 0x1F48 , 0x0000),//Greek_OMICRON
DEADTRANS( greek_psili , Greek_rho , 0x1FE4 , 0x0000),//Greek_rho
DEADTRANS( greek_psili , Greek_upsilon , 0x1F50 , 0x0000),//Greek_upsilon
DEADTRANS( greek_psili , L'A'   , 0x1EA2 , 0x0000),  //lang.module
DEADTRANS( greek_psili , L'a'   , 0x1EA3 , 0x0000),
DEADTRANS( greek_psili , L'E'   , 0x1EBA , 0x0000),
DEADTRANS( greek_psili , L'e'   , 0x1EBB , 0x0000),
DEADTRANS( greek_psili , L'I'   , 0x1EC8 , 0x0000),
DEADTRANS( greek_psili , L'i'   , 0x1EC9 , 0x0000),
DEADTRANS( greek_psili , L'O'   , 0x1ECE , 0x0000),
DEADTRANS( greek_psili , L'o'   , 0x1ECF , 0x0000),
DEADTRANS( greek_psili , L'U'   , 0x1EE6 , 0x0000),
DEADTRANS( greek_psili , L'u'   , 0x1EE7 , 0x0000),
DEADTRANS( greek_psili , L'Y'   , 0x1EF6 , 0x0000),
DEADTRANS( greek_psili , L'y'   , 0x1EF7 , 0x0000),

// Breve ( fertig für en_US.UTF-8 und lang.module)
DEADTRANS( breve , space  , breve  , 0x0000),	//Breve
DEADTRANS( breve , breve  , 0x0306 , 0x0000),	//2x für Combining
DEADTRANS( breve , L'A'   , 0x0102 , 0x0000),
DEADTRANS( breve , L'a'   , 0x0103 , 0x0000),
DEADTRANS( breve , L'E'   , 0x0114 , 0x0000),
DEADTRANS( breve , L'e'   , 0x0115 , 0x0000),
DEADTRANS( breve , L'G'   , 0x011e , 0x0000),
DEADTRANS( breve , L'g'   , 0x011f , 0x0000),
DEADTRANS( breve , L'I'   , 0x012c , 0x0000),
DEADTRANS( breve , L'i'   , 0x012d , 0x0000),
DEADTRANS( breve , L'O'   , 0x014e , 0x0000),
DEADTRANS( breve , L'o'   , 0x014f , 0x0000),
DEADTRANS( breve , L'U'   , 0x016c , 0x0000),
DEADTRANS( breve , L'u'   , 0x016d , 0x0000),
DEADTRANS( breve , Greek_alpha , 0x1FB0 , 0x0000),//Greek_alpha
DEADTRANS( breve , 0x0391 , 0x1FB8 , 0x0000),//Greek_ALPHA
DEADTRANS( breve , Greek_iota , 0x1FD0 , 0x0000),//Greek_iota
DEADTRANS( breve , 0x0399 , 0x1FD8 , 0x0000),//Greek_IOTA
DEADTRANS( breve , Greek_upsilon , 0x1FE0 , 0x0000),//Greek_upsilon
DEADTRANS( breve , 0x03a5 , 0x1FE8 , 0x0000),//Greek_UPSILON
DEADTRANS( breve , L'H'   , 0x1e2a , 0x0000), //Breve darunter
DEADTRANS( breve , L'h'   , 0x1e2b , 0x0000), //Breve darunter
// ENDE TASTE 3
// =====================================================================

// =====================================================================
//Kombinierte Diakritika, die es im Unicode gibt
//Trema und Akut: COMBINING GREEK DIALYTIKA TONOS
DEADTRANS( 0x0344 , space   , 0x0344 , 0x0000),
DEADTRANS( 0x0344 , Greek_upsilon , 0x03b0 , 0x0000),
DEADTRANS( 0x0344 , Greek_iota , 0x0390 , 0x0000),
// =====================================================================


// Der Vollständigkeit halber:
// Doppelgravis
DEADTRANS( 0x030f , space  , 0x02F5 , 0x0000),	//Doppelgravis
DEADTRANS( 0x030f , 0x030f , 0x030f , 0x0000),	//2x für Combining
DEADTRANS( 0x030f , L'A'   , 0x0200 , 0x0000),
DEADTRANS( 0x030f , L'E'   , 0x0204 , 0x0000),
DEADTRANS( 0x030f , L'I'   , 0x0208 , 0x0000),
DEADTRANS( 0x030f , L'O'   , 0x020c , 0x0000),
DEADTRANS( 0x030f , L'R'   , 0x0210 , 0x0000),
DEADTRANS( 0x030f , L'U'   , 0x0214 , 0x0000),
DEADTRANS( 0x030f , L'a'   , 0x0201 , 0x0000),
DEADTRANS( 0x030f , L'e'   , 0x0205 , 0x0000),
DEADTRANS( 0x030f , L'i'   , 0x0209 , 0x0000),
DEADTRANS( 0x030f , L'o'   , 0x020d , 0x0000),
DEADTRANS( 0x030f , L'r'   , 0x0211 , 0x0000),
DEADTRANS( 0x030f , L'u'   , 0x0215 , 0x0000),
//nicht belegt
// =====================================================================

// =====================================================================
//Compose
//Compose-Taste
DEADTRANS(	Multi_key	,	space	,	Multi_key	,	0x0001),
DEADTRANS(	Multi_key	,	Multi_key	,	Multi_key	,	0x0000),
//Vorabdefinitionen
DEADTRANS(	Multi_key	,	0x006F	,	0x006F	,	0x0001),
DEADTRANS(	Multi_key	,	0x0032	,	0x0032	,	0x0001),
DEADTRANS(	Multi_key	,	0x003F	,	0x003F	,	0x0001),
DEADTRANS(	Multi_key	,	0x003A	,	0x003A	,	0x0001),
DEADTRANS(	Multi_key	,	0x0053	,	0x0053	,	0x0001),
DEADTRANS(	Multi_key	,	0x0073	,	0x0073	,	0x0001),
DEADTRANS(	Multi_key	,	0x0054	,	0x0054	,	0x0001),
DEADTRANS(	Multi_key	,	0x0074	,	0x0074	,	0x0001),
DEADTRANS(	Multi_key	,	0x004C	,	0x004C	,	0x0001),
DEADTRANS(	Multi_key	,	0x006C	,	0x006C	,	0x0001),
DEADTRANS(	Multi_key	,	0x0031	,	0x0031	,	0x0001),
DEADTRANS(	Multi_key	,	0x0021	,	0x0021	,	0x0001),
DEADTRANS(	Multi_key	,	0x0052	,	0x0052	,	0x0001),
DEADTRANS(	Multi_key	,	0x0072	,	0x0072	,	0x0001),
DEADTRANS(	Multi_key	,	0x003C	,	0x003C	,	0x0001),
DEADTRANS(	Multi_key	,	0x0033	,	0x0033	,	0x0001),
DEADTRANS(	Multi_key	,	0x0034	,	0x0034	,	0x0001),
DEADTRANS(	Multi_key	,	0x0035	,	0x0035	,	0x0001),
DEADTRANS(	Multi_key	,	0x0037	,	0x0037	,	0x0001),
DEADTRANS(	Multi_key	,	0x0045	,	0x0045	,	0x0001),
DEADTRANS(	Multi_key	,	0x002D	,	0x002D	,	0x0001),
DEADTRANS(	Multi_key	,	0x0049	,	0x0049	,	0x0001),
DEADTRANS(	Multi_key	,	0x004E	,	0x004E	,	0x0001),
DEADTRANS(	Multi_key	,	0x0044	,	0x0044	,	0x0001),
DEADTRANS(	Multi_key	,	0x005B	,	0x005B	,	0x0001),
DEADTRANS(	Multi_key	,	0x005F	,	0x005F	,	0x0001),
DEADTRANS(	Multi_key	,	0x0070	,	0x0070	,	0x0001),
DEADTRANS(	Multi_key	,	0x0066	,	0x0066	,	0x0001),
DEADTRANS(	Multi_key	,	0x007A	,	0x007A	,	0x0001),
DEADTRANS(	Multi_key	,	0x0069	,	0x0069	,	0x0001),
DEADTRANS(	Multi_key	,	0x0046	,	0x0046	,	0x0001),
DEADTRANS(	Multi_key	,	0x006E	,	0x006E	,	0x0001),
DEADTRANS(	Multi_key	,	slong	,	slong	,	0x0001),
DEADTRANS(	Multi_key	,	0x0064	,	0x0064	,	0x0001),
DEADTRANS(	Multi_key	,	0x2026	,	0x2026	,	0x0001),
DEADTRANS(	Multi_key	,	nobreakspace	,	nobreakspace	,	0x0001),
DEADTRANS(	Multi_key	,	rightsinglequotemark	,	rightsinglequotemark	,	0x0001),
DEADTRANS(	Multi_key	,	dagger	,	dagger	,	0x0001),
DEADTRANS(	Multi_key	,	signifblank	,	signifblank	,	0x0001),
DEADTRANS(	Multi_key	,	heavycheckmark	,	heavycheckmark	,	0x0001),
DEADTRANS(	Multi_key	,	heavyballotx	,	heavyballotx	,	0x0001),
DEADTRANS(	Multi_key	,	femalesymbol	,	femalesymbol	,	0x0001),
DEADTRANS(	Multi_key	,	malesymbol	,	malesymbol	,	0x0001),
DEADTRANS(	Multi_key	,	0x25A1	,	0x25A1	,	0x0001),
DEADTRANS(	Multi_key	,	0x0020	,	0x0020	,	0x0001),
DEADTRANS(	Multi_key	,	quotedbl	,	quotedbl	,	0x0001),
DEADTRANS(	Multi_key	,	0x0023	,	0x0023	,	0x0001),
DEADTRANS(	Multi_key	,	percent	,	percent	,	0x0001),
DEADTRANS(	Multi_key	,	apostrophe	,	apostrophe	,	0x0001),
DEADTRANS(	Multi_key	,	0x0028	,	0x0028	,	0x0001),
DEADTRANS(	Multi_key	,	0x0029	,	0x0029	,	0x0001),
DEADTRANS(	Multi_key	,	0x002B	,	0x002B	,	0x0001),
DEADTRANS(	Multi_key	,	0x002C	,	0x002C	,	0x0001),
DEADTRANS(	Multi_key	,	0x002E	,	0x002E	,	0x0001),
DEADTRANS(	Multi_key	,	0x002F	,	0x002F	,	0x0001),
DEADTRANS(	Multi_key	,	0x003B	,	0x003B	,	0x0001),
DEADTRANS(	Multi_key	,	0x003D	,	0x003D	,	0x0001),
DEADTRANS(	Multi_key	,	0x003E	,	0x003E	,	0x0001),
DEADTRANS(	Multi_key	,	0x0041	,	0x0041	,	0x0001),
DEADTRANS(	Multi_key	,	0x0043	,	0x0043	,	0x0001),
DEADTRANS(	Multi_key	,	0x004F	,	0x004F	,	0x0001),
DEADTRANS(	Multi_key	,	0x0050	,	0x0050	,	0x0001),
DEADTRANS(	Multi_key	,	0x0055	,	0x0055	,	0x0001),
DEADTRANS(	Multi_key	,	0x0056	,	0x0056	,	0x0001),
DEADTRANS(	Multi_key	,	0x0057	,	0x0057	,	0x0001),
DEADTRANS(	Multi_key	,	0x0059	,	0x0059	,	0x0001),
DEADTRANS(	Multi_key	,	asciicircum	,	asciicircum	,	0x0001),
DEADTRANS(	Multi_key	,	grave	,	grave	,	0x0001),
DEADTRANS(	Multi_key	,	0x0061	,	0x0061	,	0x0001),
DEADTRANS(	Multi_key	,	0x0062	,	0x0062	,	0x0001),
DEADTRANS(	Multi_key	,	0x0063	,	0x0063	,	0x0001),
DEADTRANS(	Multi_key	,	0x0065	,	0x0065	,	0x0001),
DEADTRANS(	Multi_key	,	0x006B	,	0x006B	,	0x0001),
DEADTRANS(	Multi_key	,	0x006D	,	0x006D	,	0x0001),
DEADTRANS(	Multi_key	,	0x006F	,	0x006F	,	0x0001),
DEADTRANS(	Multi_key	,	0x0076	,	0x0076	,	0x0001),
DEADTRANS(	Multi_key	,	0x0078	,	0x0078	,	0x0001),
DEADTRANS(	Multi_key	,	0x007C	,	0x007C	,	0x0001),
DEADTRANS(	Multi_key	,	0x007E	,	0x007E	,	0x0001),
DEADTRANS(	Multi_key	,	diaeresis	,	diaeresis	,	0x0001),
DEADTRANS(	Multi_key	,	macron	,	macron	,	0x0001),
DEADTRANS(	Multi_key	,	acute	,	acute	,	0x0001),
DEADTRANS(	Multi_key	,	Greek_iota	,	Greek_iota	,	0x0001),
DEADTRANS(	Multi_key	,	0x05B4	,	0x05B4	,	0x0001),
DEADTRANS(	Multi_key	,	0x05B7	,	0x05B7	,	0x0001),
DEADTRANS(	Multi_key	,	0x05B8	,	0x05B8	,	0x0001),
DEADTRANS(	Multi_key	,	0x05B9	,	0x05B9	,	0x0001),
DEADTRANS(	Multi_key	,	0x05BC	,	0x05BC	,	0x0001),
DEADTRANS(	Multi_key	,	0x05BF	,	0x05BF	,	0x0001),
DEADTRANS(	Multi_key	,	0x05C1	,	0x05C1	,	0x0001),
DEADTRANS(	Multi_key	,	0x05C2	,	0x05C2	,	0x0001),
DEADTRANS(	Multi_key	,	0x0653	,	0x0653	,	0x0001),
DEADTRANS(	Multi_key	,	0x0654	,	0x0654	,	0x0001),
DEADTRANS(	Multi_key	,	0x0655	,	0x0655	,	0x0001),
DEADTRANS(	Multi_key	,	0x093C	,	0x093C	,	0x0001),
DEADTRANS(	Multi_key	,	0x09BC	,	0x09BC	,	0x0001),
DEADTRANS(	Multi_key	,	0x09C7	,	0x09C7	,	0x0001),
DEADTRANS(	Multi_key	,	0x0A3C	,	0x0A3C	,	0x0001),
DEADTRANS(	Multi_key	,	0x0B3C	,	0x0B3C	,	0x0001),
DEADTRANS(	Multi_key	,	0x0B47	,	0x0B47	,	0x0001),
DEADTRANS(	Multi_key	,	0x0BC6	,	0x0BC6	,	0x0001),
DEADTRANS(	Multi_key	,	0x0BC7	,	0x0BC7	,	0x0001),
DEADTRANS(	Multi_key	,	0x0BD7	,	0x0BD7	,	0x0001),
DEADTRANS(	Multi_key	,	0x0C46	,	0x0C46	,	0x0001),
DEADTRANS(	Multi_key	,	0x0CBF	,	0x0CBF	,	0x0001),
DEADTRANS(	Multi_key	,	0x0CC6	,	0x0CC6	,	0x0001),
DEADTRANS(	Multi_key	,	0x0CCA	,	0x0CCA	,	0x0001),
DEADTRANS(	Multi_key	,	0x0D46	,	0x0D46	,	0x0001),
DEADTRANS(	Multi_key	,	0x0D47	,	0x0D47	,	0x0001),
DEADTRANS(	Multi_key	,	0x0DD9	,	0x0DD9	,	0x0001),
DEADTRANS(	Multi_key	,	0x0DDC	,	0x0DDC	,	0x0001),
DEADTRANS(	Multi_key	,	0x0F71	,	0x0F71	,	0x0001),
DEADTRANS(	Multi_key	,	0x0F90	,	0x0F90	,	0x0001),
DEADTRANS(	Multi_key	,	0x0F92	,	0x0F92	,	0x0001),
DEADTRANS(	Multi_key	,	0x0F9C	,	0x0F9C	,	0x0001),
DEADTRANS(	Multi_key	,	0x0FA1	,	0x0FA1	,	0x0001),
DEADTRANS(	Multi_key	,	0x0FA6	,	0x0FA6	,	0x0001),
DEADTRANS(	Multi_key	,	0x0FAB	,	0x0FAB	,	0x0001),
DEADTRANS(	Multi_key	,	0x0FB2	,	0x0FB2	,	0x0001),
DEADTRANS(	Multi_key	,	0x0FB3	,	0x0FB3	,	0x0001),
DEADTRANS(	Multi_key	,	0x0FB5	,	0x0FB5	,	0x0001),
DEADTRANS(	Multi_key	,	0x0FB7	,	0x0FB7	,	0x0001),
DEADTRANS(	Multi_key	,	0x102E	,	0x102E	,	0x0001),
DEADTRANS(	Multi_key	,	0x1100	,	0x1100	,	0x0001),
DEADTRANS(	Multi_key	,	0x1102	,	0x1102	,	0x0001),
DEADTRANS(	Multi_key	,	0x1103	,	0x1103	,	0x0001),
DEADTRANS(	Multi_key	,	0x1105	,	0x1105	,	0x0001),
DEADTRANS(	Multi_key	,	0x1106	,	0x1106	,	0x0001),
DEADTRANS(	Multi_key	,	0x1107	,	0x1107	,	0x0001),
DEADTRANS(	Multi_key	,	0x1108	,	0x1108	,	0x0001),
DEADTRANS(	Multi_key	,	0x1109	,	0x1109	,	0x0001),
DEADTRANS(	Multi_key	,	0x110A	,	0x110A	,	0x0001),
DEADTRANS(	Multi_key	,	0x110B	,	0x110B	,	0x0001),
DEADTRANS(	Multi_key	,	0x110C	,	0x110C	,	0x0001),
DEADTRANS(	Multi_key	,	0x110E	,	0x110E	,	0x0001),
DEADTRANS(	Multi_key	,	0x1111	,	0x1111	,	0x0001),
DEADTRANS(	Multi_key	,	0x1112	,	0x1112	,	0x0001),
DEADTRANS(	Multi_key	,	0x1121	,	0x1121	,	0x0001),
DEADTRANS(	Multi_key	,	0x1132	,	0x1132	,	0x0001),
DEADTRANS(	Multi_key	,	0x113C	,	0x113C	,	0x0001),
DEADTRANS(	Multi_key	,	0x113E	,	0x113E	,	0x0001),
DEADTRANS(	Multi_key	,	0x114E	,	0x114E	,	0x0001),
DEADTRANS(	Multi_key	,	0x1150	,	0x1150	,	0x0001),
DEADTRANS(	Multi_key	,	0x1161	,	0x1161	,	0x0001),
DEADTRANS(	Multi_key	,	0x1163	,	0x1163	,	0x0001),
DEADTRANS(	Multi_key	,	0x1165	,	0x1165	,	0x0001),
DEADTRANS(	Multi_key	,	0x1167	,	0x1167	,	0x0001),
DEADTRANS(	Multi_key	,	0x1169	,	0x1169	,	0x0001),
DEADTRANS(	Multi_key	,	0x116A	,	0x116A	,	0x0001),
DEADTRANS(	Multi_key	,	0x116D	,	0x116D	,	0x0001),
DEADTRANS(	Multi_key	,	0x116E	,	0x116E	,	0x0001),
DEADTRANS(	Multi_key	,	0x116F	,	0x116F	,	0x0001),
DEADTRANS(	Multi_key	,	0x1172	,	0x1172	,	0x0001),
DEADTRANS(	Multi_key	,	0x1173	,	0x1173	,	0x0001),
DEADTRANS(	Multi_key	,	0x1174	,	0x1174	,	0x0001),
DEADTRANS(	Multi_key	,	0x1175	,	0x1175	,	0x0001),
DEADTRANS(	Multi_key	,	0x119E	,	0x119E	,	0x0001),
DEADTRANS(	Multi_key	,	0x11A8	,	0x11A8	,	0x0001),
DEADTRANS(	Multi_key	,	0x11AA	,	0x11AA	,	0x0001),
DEADTRANS(	Multi_key	,	0x11AB	,	0x11AB	,	0x0001),
DEADTRANS(	Multi_key	,	0x11AE	,	0x11AE	,	0x0001),
DEADTRANS(	Multi_key	,	0x11AF	,	0x11AF	,	0x0001),
DEADTRANS(	Multi_key	,	0x11B0	,	0x11B0	,	0x0001),
DEADTRANS(	Multi_key	,	0x11B1	,	0x11B1	,	0x0001),
DEADTRANS(	Multi_key	,	0x11B2	,	0x11B2	,	0x0001),
DEADTRANS(	Multi_key	,	0x11B3	,	0x11B3	,	0x0001),
DEADTRANS(	Multi_key	,	0x11B7	,	0x11B7	,	0x0001),
DEADTRANS(	Multi_key	,	0x11B8	,	0x11B8	,	0x0001),
DEADTRANS(	Multi_key	,	0x11BA	,	0x11BA	,	0x0001),
DEADTRANS(	Multi_key	,	0x11BC	,	0x11BC	,	0x0001),
DEADTRANS(	Multi_key	,	0x11C1	,	0x11C1	,	0x0001),
DEADTRANS(	Multi_key	,	0x11C2	,	0x11C2	,	0x0001),
DEADTRANS(	Multi_key	,	0x11CE	,	0x11CE	,	0x0001),
DEADTRANS(	Multi_key	,	0x11DD	,	0x11DD	,	0x0001),
DEADTRANS(	Multi_key	,	0x11EC	,	0x11EC	,	0x0001),
DEADTRANS(	Multi_key	,	0x11F0	,	0x11F0	,	0x0001),
DEADTRANS(	Multi_key	,	greek_psili	,	greek_psili	,	0x0001),
DEADTRANS(	Multi_key	,	greek_dasia	,	greek_dasia	,	0x0001),
DEADTRANS(	Multi_key	,	thereexists	,	thereexists	,	0x0001),
DEADTRANS(	Multi_key	,	elementof	,	elementof	,	0x0001),
DEADTRANS(	Multi_key	,	0x220B	,	0x220B	,	0x0001),
DEADTRANS(	Multi_key	,	0x2223	,	0x2223	,	0x0001),
DEADTRANS(	Multi_key	,	parallelto	,	parallelto	,	0x0001),
DEADTRANS(	Multi_key	,	0x223C	,	0x223C	,	0x0001),
DEADTRANS(	Multi_key	,	0x2243	,	0x2243	,	0x0001),
DEADTRANS(	Multi_key	,	0x2248	,	0x2248	,	0x0001),
DEADTRANS(	Multi_key	,	0x224D	,	0x224D	,	0x0001),
DEADTRANS(	Multi_key	,	0x2261	,	0x2261	,	0x0001),
DEADTRANS(	Multi_key	,	0x2264	,	0x2264	,	0x0001),
DEADTRANS(	Multi_key	,	0x2265	,	0x2265	,	0x0001),
DEADTRANS(	Multi_key	,	0x2272	,	0x2272	,	0x0001),
DEADTRANS(	Multi_key	,	0x2273	,	0x2273	,	0x0001),
DEADTRANS(	Multi_key	,	0x2276	,	0x2276	,	0x0001),
DEADTRANS(	Multi_key	,	0x2277	,	0x2277	,	0x0001),
DEADTRANS(	Multi_key	,	0x227A	,	0x227A	,	0x0001),
DEADTRANS(	Multi_key	,	0x227B	,	0x227B	,	0x0001),
DEADTRANS(	Multi_key	,	0x227C	,	0x227C	,	0x0001),
DEADTRANS(	Multi_key	,	0x227D	,	0x227D	,	0x0001),
DEADTRANS(	Multi_key	,	includedin	,	includedin	,	0x0001),
DEADTRANS(	Multi_key	,	0x2283	,	0x2283	,	0x0001),
DEADTRANS(	Multi_key	,	0x2286	,	0x2286	,	0x0001),
DEADTRANS(	Multi_key	,	0x2287	,	0x2287	,	0x0001),
DEADTRANS(	Multi_key	,	0x2291	,	0x2291	,	0x0001),
DEADTRANS(	Multi_key	,	0x2292	,	0x2292	,	0x0001),
DEADTRANS(	Multi_key	,	0x22A3	,	0x22A3	,	0x0001),
DEADTRANS(	Multi_key	,	0x22A8	,	0x22A8	,	0x0001),
DEADTRANS(	Multi_key	,	0x22A9	,	0x22A9	,	0x0001),
DEADTRANS(	Multi_key	,	0x22AB	,	0x22AB	,	0x0001),
DEADTRANS(	Multi_key	,	0x22B2	,	0x22B2	,	0x0001),
DEADTRANS(	Multi_key	,	0x22B3	,	0x22B3	,	0x0001),
DEADTRANS(	Multi_key	,	0x22B4	,	0x22B4	,	0x0001),
DEADTRANS(	Multi_key	,	0x22B5	,	0x22B5	,	0x0001),
DEADTRANS(	Multi_key	,	0x2ADD	,	0x2ADD	,	0x0001),
DEADTRANS(	Multi_key	,	0x007C	,	0x007C	,	0x0001),
DEADTRANS(	Multi_key	,	0x0069	,	0x0069	,	0x0001),
DEADTRANS(	Multi_key	,	0x0077	,	0x0077	,	0x0001),
DEADTRANS(	Multi_key	,	apostrophe	,	apostrophe	,	0x0001),
DEADTRANS(	Multi_key	,	0x005F	,	0x005F	,	0x0001),
DEADTRANS(	Multi_key	,	0x0028	,	0x0028	,	0x0001),
DEADTRANS(	Multi_key	,	0x0029	,	0x0029	,	0x0001),
DEADTRANS(	Multi_key	,	0x0056	,	0x0056	,	0x0001),
DEADTRANS(	Multi_key	,	0x0076	,	0x0076	,	0x0001),
DEADTRANS(	Multi_key	,	0x002A	,	0x002A	,	0x0001),
DEADTRANS(	Multi_key	,	0x006F	,	0x006F	,	0x0001),
DEADTRANS(	Multi_key	,	0x002D	,	0x002D	,	0x0001),
DEADTRANS(	Multi_key	,	0x003C	,	0x003C	,	0x0001),
DEADTRANS(	Multi_key	,	0x003E	,	0x003E	,	0x0001),
DEADTRANS(	Multi_key	,	asciicircum	,	asciicircum	,	0x0001),
DEADTRANS(	Multi_key	,	guillemotleft	,	guillemotleft	,	0x0001),
DEADTRANS(	Multi_key	,	guillemotright	,	guillemotright	,	0x0001),
DEADTRANS(	Multi_key	,	leftarrow	,	leftarrow	,	0x0001),
DEADTRANS(	Multi_key	,	0x0030	,	0x0030	,	0x0001),
DEADTRANS(	Multi_key	,	0x2026	,	0x2026	,	0x0001),
DEADTRANS(	Multi_key	,	0x002E	,	0x002E	,	0x0001),
DEADTRANS(	Multi_key	,	0x002F	,	0x002F	,	0x0001),
DEADTRANS(	Multi_key	,	0x0061	,	0x0061	,	0x0001),
DEADTRANS(	Multi_key	,	0x006D	,	0x006D	,	0x0001),
DEADTRANS(	Multi_key	,	0x0074	,	0x0074	,	0x0001),
DEADTRANS(	Multi_key	,	integral	,	integral	,	0x0001),
DEADTRANS(	Multi_key	,	0x0033	,	0x0033	,	0x0001),
DEADTRANS(	Multi_key	,	radical	,	radical	,	0x0001),
DEADTRANS(	Multi_key	,	0x003A	,	0x003A	,	0x0001),
DEADTRANS(	Multi_key	,	0x003D	,	0x003D	,	0x0001),
DEADTRANS(	Multi_key	,	0x007E	,	0x007E	,	0x0001),
DEADTRANS(	Multi_key	,	degree	,	degree	,	0x0001),
DEADTRANS(	Multi_key	,	doublearrowleft	,	doublearrowleft	,	0x0001),
DEADTRANS(	Multi_key	,	0x0042	,	0x0042	,	0x0001),
DEADTRANS(	Multi_key	,	0x0047	,	0x0047	,	0x0001),
DEADTRANS(	Multi_key	,	0x0059	,	0x0059	,	0x0001),
DEADTRANS(	Multi_key	,	0x0048	,	0x0048	,	0x0001),
DEADTRANS(	Multi_key	,	0x0053	,	0x0053	,	0x0001),
DEADTRANS(	Multi_key	,	0x005A	,	0x005A	,	0x0001),
DEADTRANS(	Multi_key	,	0x0049	,	0x0049	,	0x0001),
DEADTRANS(	Multi_key	,	0x004C	,	0x004C	,	0x0001),
DEADTRANS(	Multi_key	,	0x004E	,	0x004E	,	0x0001),
DEADTRANS(	Multi_key	,	0x006A	,	0x006A	,	0x0001),
DEADTRANS(	Multi_key	,	0x0052	,	0x0052	,	0x0001),
DEADTRANS(	Multi_key	,	0x004F	,	0x004F	,	0x0001),
DEADTRANS(	Multi_key	,	0x0067	,	0x0067	,	0x0001),
DEADTRANS(	Multi_key	,	0x005B	,	0x005B	,	0x0001),
DEADTRANS(	Multi_key	,	backslash	,	backslash	,	0x0001),
DEADTRANS(	Multi_key	,	0x005D	,	0x005D	,	0x0001),
DEADTRANS(	Multi_key	,	0x0072	,	0x0072	,	0x0001),
DEADTRANS(	Multi_key	,	0x0078	,	0x0078	,	0x0001),
DEADTRANS(	Multi_key	,	macron	,	macron	,	0x0001),
DEADTRANS(	Multi_key	,	0x0068	,	0x0068	,	0x0001),
DEADTRANS(	Multi_key	,	0x0079	,	0x0079	,	0x0001),
DEADTRANS(	Multi_key	,	0x0073	,	0x0073	,	0x0001),
DEADTRANS(	Multi_key	,	0x0044	,	0x0044	,	0x0001),
DEADTRANS(	Multi_key	,	0x0054	,	0x0054	,	0x0001),
DEADTRANS(	Multi_key	,	0x0064	,	0x0064	,	0x0001),
DEADTRANS(	Multi_key	,	0x006C	,	0x006C	,	0x0001),
DEADTRANS(	Multi_key	,	0x006E	,	0x006E	,	0x0001),
DEADTRANS(	Multi_key	,	0x0057	,	0x0057	,	0x0001),
DEADTRANS(	Multi_key	,	0x007B	,	0x007B	,	0x0001),
DEADTRANS(	Multi_key	,	0x0043	,	0x0043	,	0x0001),
DEADTRANS(	Multi_key	,	multiply	,	multiply	,	0x0001),
DEADTRANS(	Multi_key	,	uparrow	,	uparrow	,	0x0001),
DEADTRANS(	Multi_key	,	rightarrow	,	rightarrow	,	0x0001),
DEADTRANS(	Multi_key	,	downarrow	,	downarrow	,	0x0001),
DEADTRANS(	Multi_key	,	updownarrow	,	updownarrow	,	0x0001),
DEADTRANS(	Multi_key	,	0x007D	,	0x007D	,	0x0001),
DEADTRANS(	Multi_key	,	grave	,	grave	,	0x0001),
DEADTRANS(	Multi_key	,	acute	,	acute	,	0x0001),
DEADTRANS(	Multi_key	,	periodcentered	,	periodcentered	,	0x0001),
DEADTRANS(	Multi_key	,	Greek_PI	,	Greek_PI	,	0x0001),
DEADTRANS(	Multi_key	,	Greek_SIGMA	,	Greek_SIGMA	,	0x0001),
DEADTRANS(	Multi_key	,	Greek_PHI	,	Greek_PHI	,	0x0001),
DEADTRANS(	Multi_key	,	Greek_alpha	,	Greek_alpha	,	0x0001),
DEADTRANS(	Multi_key	,	Greek_gamma	,	Greek_gamma	,	0x0001),
DEADTRANS(	Multi_key	,	Greek_epsilon	,	Greek_epsilon	,	0x0001),
DEADTRANS(	Multi_key	,	0x03BC	,	0x03BC	,	0x0001),
DEADTRANS(	Multi_key	,	Greek_omicron	,	Greek_omicron	,	0x0001),
DEADTRANS(	Multi_key	,	Greek_upsilon	,	Greek_upsilon	,	0x0001),
DEADTRANS(	Multi_key	,	Greek_phi	,	Greek_phi	,	0x0001),
DEADTRANS(	Multi_key	,	scriptphi	,	scriptphi	,	0x0001),
DEADTRANS(	Multi_key	,	0x007A	,	0x007A	,	0x0001),
DEADTRANS(	Multi_key	,	elementof	,	elementof	,	0x0001),
DEADTRANS(	Multi_key	,	0x0034	,	0x0034	,	0x0001),
DEADTRANS(	Multi_key	,	logicaland	,	logicaland	,	0x0001),
DEADTRANS(	Multi_key	,	logicalor	,	logicalor	,	0x0001),
DEADTRANS(	Multi_key	,	intersection	,	intersection	,	0x0001),
DEADTRANS(	Multi_key	,	union	,	union	,	0x0001),
DEADTRANS(	Multi_key	,	0x0032	,	0x0032	,	0x0001),
DEADTRANS(	Multi_key	,	jot	,	jot	,	0x0001),
DEADTRANS(	Multi_key	,	0x2295	,	0x2295	,	0x0001),
DEADTRANS(	Multi_key	,	circledtimes	,	circledtimes	,	0x0001),
DEADTRANS(	Multi_key	,	0x2299	,	0x2299	,	0x0001),
DEADTRANS(	Multi_key	,	uptack	,	uptack	,	0x0001),


//base.module
DEADTRANS(	0x006F	,	0x0020	,	0x25CC	,	0x0000),
DEADTRANS(	0x0032	,	0x0021	,	0x203C	,	0x0000),
DEADTRANS(	0x003F	,	0x0021	,	0x2048	,	0x0000),
DEADTRANS(	0x003A	,	0x0028	,	0x2639	,	0x0000),
DEADTRANS(	0x003A	,	0x0029	,	0x263A	,	0x0000),
DEADTRANS(	0x0053	,	0x002C	,	0x0218	,	0x0000),
DEADTRANS(	0x0073	,	0x002C	,	0x0219	,	0x0000),
DEADTRANS(	0x0054	,	0x002C	,	0x021A	,	0x0000),
DEADTRANS(	0x0074	,	0x002C	,	0x021B	,	0x0000),
DEADTRANS(	0x004C	,	0x002E	,	0x013F	,	0x0000),
DEADTRANS(	0x006C	,	0x002E	,	0x0140	,	0x0000),
DEADTRANS(	0x0031	,	0x002F	,	0x215F	,	0x0000),
DEADTRANS(	0x0021	,	0x0032	,	0x203C	,	0x0000),
DEADTRANS(	0x003F	,	0x0032	,	0x2047	,	0x0000),
DEADTRANS(	0x0052	,	0x0032	,	0x2161	,	0x0000),
DEADTRANS(	0x0072	,	0x0032	,	0x2171	,	0x0000),
DEADTRANS(	0x0031	,	0x0033	,	0x2153	,	0x0000),
DEADTRANS(	0x0032	,	0x0033	,	0x2154	,	0x0000),
DEADTRANS(	0x0052	,	0x0033	,	0x2162	,	0x0000),
DEADTRANS(	0x0072	,	0x0033	,	0x2172	,	0x0000),
DEADTRANS(	0x003C	,	0x0033	,	heart	,	0x0000),
DEADTRANS(	0x0052	,	0x0034	,	0x2163	,	0x0000),
DEADTRANS(	0x0072	,	0x0034	,	0x2173	,	0x0000),
DEADTRANS(	0x0031	,	0x0035	,	0x2155	,	0x0000),
DEADTRANS(	0x0032	,	0x0035	,	0x2156	,	0x0000),
DEADTRANS(	0x0033	,	0x0035	,	0x2157	,	0x0000),
DEADTRANS(	0x0034	,	0x0035	,	0x2158	,	0x0000),
DEADTRANS(	0x0052	,	0x0035	,	0x2164	,	0x0000),
DEADTRANS(	0x0072	,	0x0035	,	0x2174	,	0x0000),
DEADTRANS(	0x0031	,	0x0036	,	0x2159	,	0x0000),
DEADTRANS(	0x0035	,	0x0036	,	0x215A	,	0x0000),
DEADTRANS(	0x0052	,	0x0036	,	0x2165	,	0x0000),
DEADTRANS(	0x0072	,	0x0036	,	0x2175	,	0x0000),
DEADTRANS(	0x0037	,	0x0037	,	0x204A	,	0x0000),
DEADTRANS(	0x0052	,	0x0037	,	0x2166	,	0x0000),
DEADTRANS(	0x0072	,	0x0037	,	0x2176	,	0x0000),
DEADTRANS(	0x0031	,	0x0038	,	0x215B	,	0x0000),
DEADTRANS(	0x0033	,	0x0038	,	0x215C	,	0x0000),
DEADTRANS(	0x0035	,	0x0038	,	0x215D	,	0x0000),
DEADTRANS(	0x0037	,	0x0038	,	0x215E	,	0x0000),
DEADTRANS(	0x0052	,	0x0038	,	0x2167	,	0x0000),
DEADTRANS(	0x0072	,	0x0038	,	0x2177	,	0x0000),
DEADTRANS(	0x0052	,	0x0039	,	0x2168	,	0x0000),
DEADTRANS(	0x0072	,	0x0039	,	0x2178	,	0x0000),
DEADTRANS(	0x0032	,	0x003F	,	0x2047	,	0x0000),
DEADTRANS(	0x0021	,	0x003F	,	0x2049	,	0x0000),
DEADTRANS(	0x0045	,	0x0045	,	0x018F	,	0x0000),
DEADTRANS(	0x002D	,	0x0048	,	0x0126	,	0x0000),
DEADTRANS(	0x0049	,	0x0049	,	0x0130	,	0x0000),
DEADTRANS(	0x0049	,	0x004A	,	0x0132	,	0x0000),
DEADTRANS(	0x004C	,	0x004A	,	0x01C7	,	0x0000),
DEADTRANS(	0x004E	,	0x004A	,	0x01CA	,	0x0000),
DEADTRANS(	0x0044	,	0x005A	,	0x01F1	,	0x0000),
DEADTRANS(	0x005B	,	0x005D	,	0x25A1	,	0x0000),
DEADTRANS(	0x005F	,	0x005F	,	0x0332	,	0x0000),
DEADTRANS(	0x0070	,	0x0066	,	0x20B0	,	0x0000),
DEADTRANS(	0x0066	,	0x0066	,	0xFB00	,	0x0000),
DEADTRANS(	0x002D	,	0x0068	,	0x0127	,	0x0000),
DEADTRANS(	0x007A	,	0x0068	,	0x0292	,	0x0000),
DEADTRANS(	0x0069	,	0x0069	,	0x0131	,	0x0000),
DEADTRANS(	0x0066	,	0x0069	,	0xFB01	,	0x0000),
DEADTRANS(	0x0046	,	0x0069	,	0xFB03	,	0x0000),
DEADTRANS(	0x0069	,	0x006A	,	0x0133	,	0x0000),
DEADTRANS(	0x004C	,	0x006A	,	0x01C8	,	0x0000),
DEADTRANS(	0x006C	,	0x006A	,	0x01C9	,	0x0000),
DEADTRANS(	0x004E	,	0x006A	,	0x01CB	,	0x0000),
DEADTRANS(	0x006E	,	0x006A	,	0x01CC	,	0x0000),
DEADTRANS(	0x0066	,	0x006C	,	0xFB02	,	0x0000),
DEADTRANS(	0x0046	,	0x006C	,	0xFB04	,	0x0000),
DEADTRANS(	0x0074	,	0x006D	,	0x2122	,	0x0000),
DEADTRANS(	slong	,	0x0074	,	0xFB05	,	0x0000),
DEADTRANS(	0x0073	,	0x0074	,	0xFB06	,	0x0000),
DEADTRANS(	0x0044	,	0x007A	,	0x01F2	,	0x0000),
DEADTRANS(	0x0064	,	0x007A	,	0x01F3	,	0x0000),
DEADTRANS(	0x007A	,	0x007A	,	0x0292	,	0x0000),
DEADTRANS(	0x2026	,	0x007C	,	0x22EE	,	0x0000),
DEADTRANS(	nobreakspace	,	nobreakspace	,	0x002D	,	0x0000),
DEADTRANS(	0x002D	,	sterling	,	0x20A4	,	0x0000),
DEADTRANS(	rightsinglequotemark	,	rightsinglequotemark	,	0x02BC	,	0x0000),
DEADTRANS(	dagger	,	dagger	,	0x2021	,	0x0000),
DEADTRANS(	signifblank	,	signifblank	,	0x237D	,	0x0000),
DEADTRANS(	heavycheckmark	,	0x25A1	,	0x2611	,	0x0000),
DEADTRANS(	heavyballotx	,	0x25A1	,	0x2612	,	0x0000),
DEADTRANS(	femalesymbol	,	femalesymbol	,	0x26A2	,	0x0000),
DEADTRANS(	malesymbol	,	femalesymbol	,	0x26A4	,	0x0000),
DEADTRANS(	malesymbol	,	malesymbol	,	0x26A3	,	0x0000),
DEADTRANS(	femalesymbol	,	malesymbol	,	0x26A4	,	0x0000),
DEADTRANS(	0x25A1	,	heavycheckmark	,	0x2611	,	0x0000),
DEADTRANS(	0x25A1	,	heavyballotx	,	0x2612	,	0x0000),

//math.base und lang.base
DEADTRANS(	0x007C	,	0x0020	,	0x2223	,	0x0000),
DEADTRANS(	0x0069	,	0x0021	,	0x01C3	,	0x0000),
DEADTRANS(	0x0077	,	quotedbl	,	0x2033	,	0x0000),
DEADTRANS(	apostrophe	,	apostrophe	,	0x02C8	,	0x0000),
DEADTRANS(	0x005F	,	apostrophe	,	0x02CC	,	0x0000),
DEADTRANS(	0x0077	,	apostrophe	,	0x2032	,	0x0000),
DEADTRANS(	0x0028	,	0x0028	,	0x2985	,	0x0000),
DEADTRANS(	0x007C	,	0x0028	,	0x2985	,	0x0000),
DEADTRANS(	0x0029	,	0x0029	,	0x2986	,	0x0000),
DEADTRANS(	0x0056	,	0x0029	,	0x01B2	,	0x0000),
DEADTRANS(	0x0076	,	0x0029	,	0x028B	,	0x0000),
DEADTRANS(	0x007C	,	0x0029	,	0x2986	,	0x0000),
DEADTRANS(	0x002A	,	0x002A	,	0x2217	,	0x0000),
DEADTRANS(	0x006F	,	0x002A	,	0x229B	,	0x0000),
DEADTRANS(	0x002D	,	0x002B	,	minusorplus	,	0x0000),
DEADTRANS(	0x006F	,	0x002B	,	0x2295	,	0x0000),
DEADTRANS(	0x003C	,	0x002D	,	leftarrow	,	0x0000),
DEADTRANS(	0x003E	,	0x002D	,	0x291A	,	0x0000),
DEADTRANS(	asciicircum	,	0x002D	,	0x207B	,	0x0000),
DEADTRANS(	0x005F	,	0x002D	,	0x208B	,	0x0000),
DEADTRANS(	0x005F	,	0x002D	,	0x208B	,	0x0000),
DEADTRANS(	0x006F	,	0x002D	,	0x2296	,	0x0000),
DEADTRANS(	0x007C	,	0x002D	,	0x22A2	,	0x0000),
DEADTRANS(	guillemotleft	,	0x002D	,	0x219E	,	0x0000),
DEADTRANS(	guillemotright	,	0x002D	,	0x291C	,	0x0000),
DEADTRANS(	leftarrow	,	0x002D	,	0x27F5	,	0x0000),
DEADTRANS(	0x0030	,	0x002E	,	0x0298	,	0x0000),
DEADTRANS(	0x003C	,	0x002E	,	0x2235	,	0x0000),
DEADTRANS(	0x003E	,	0x002E	,	0x2234	,	0x0000),
DEADTRANS(	0x006F	,	0x002E	,	0x2299	,	0x0000),
DEADTRANS(	0x006F	,	0x002F	,	0x2298	,	0x0000),
DEADTRANS(	0x2026	,	0x002F	,	0x22F0	,	0x0000),
DEADTRANS(	0x002E	,	0x0030	,	0x0298	,	0x0000),
DEADTRANS(	0x002F	,	0x0030	,	emptyset	,	0x0000),
DEADTRANS(	0x0061	,	0x0030	,	0x0660	,	0x0000),
DEADTRANS(	0x0061	,	0x0031	,	0x0661	,	0x0000),
DEADTRANS(	0x006D	,	0x0031	,	0x02E5	,	0x0000),
DEADTRANS(	0x0074	,	0x0031	,	0x02E9	,	0x0000),
DEADTRANS(	0x0061	,	0x0032	,	0x0662	,	0x0000),
DEADTRANS(	0x006D	,	0x0032	,	0x02E7	,	0x0000),
DEADTRANS(	0x0074	,	0x0032	,	0x02E8	,	0x0000),
DEADTRANS(	integral	,	0x0032	,	0x222C	,	0x0000),
DEADTRANS(	0x0033	,	0x0033	,	0x025C	,	0x0000),
DEADTRANS(	0x0061	,	0x0033	,	0x0663	,	0x0000),
DEADTRANS(	0x006D	,	0x0033	,	0x02E8	,	0x0000),
DEADTRANS(	0x0074	,	0x0033	,	0x02E7	,	0x0000),
DEADTRANS(	0x0077	,	0x0033	,	0x2034	,	0x0000),
DEADTRANS(	radical	,	0x0033	,	0x221B	,	0x0000),
DEADTRANS(	integral	,	0x0033	,	0x222D	,	0x0000),
DEADTRANS(	0x0061	,	0x0034	,	0x0664	,	0x0000),
DEADTRANS(	0x006D	,	0x0034	,	0x02E5	,	0x0000),
DEADTRANS(	0x0074	,	0x0034	,	0x02E6	,	0x0000),
DEADTRANS(	0x0077	,	0x0034	,	0x2057	,	0x0000),
DEADTRANS(	radical	,	0x0034	,	0x221C	,	0x0000),
DEADTRANS(	integral	,	0x0034	,	0x2A0C	,	0x0000),
DEADTRANS(	0x0061	,	0x0035	,	0x0665	,	0x0000),
DEADTRANS(	0x0074	,	0x0035	,	0x02E5	,	0x0000),
DEADTRANS(	0x0061	,	0x0036	,	0x0666	,	0x0000),
DEADTRANS(	0x0061	,	0x0037	,	0x0667	,	0x0000),
DEADTRANS(	0x0061	,	0x0038	,	0x0668	,	0x0000),
DEADTRANS(	0x0061	,	0x0039	,	0x0669	,	0x0000),
DEADTRANS(	0x002D	,	0x003A	,	0x2239	,	0x0000),
DEADTRANS(	0x003A	,	0x003A	,	0x02D0	,	0x0000),
DEADTRANS(	0x003D	,	0x003A	,	0x2255	,	0x0000),
DEADTRANS(	0x002D	,	0x003C	,	0x2919	,	0x0000),
DEADTRANS(	0x002F	,	0x003C	,	0x226E	,	0x0000),
DEADTRANS(	0x003C	,	0x003C	,	0x226A	,	0x0000),
DEADTRANS(	0x003D	,	0x003C	,	0x2264	,	0x0000),
DEADTRANS(	asciicircum	,	0x003C	,	0x20D6	,	0x0000),
DEADTRANS(	0x007C	,	0x003C	,	0x22B2	,	0x0000),
DEADTRANS(	leftarrow	,	0x003C	,	0x21A2	,	0x0000),
DEADTRANS(	0x002F	,	0x003D	,	0x2260	,	0x0000),
DEADTRANS(	0x003A	,	0x003D	,	0x2254	,	0x0000),
DEADTRANS(	0x003C	,	0x003D	,	0x2264	,	0x0000),
DEADTRANS(	0x003D	,	0x003D	,	0x2261	,	0x0000),
DEADTRANS(	0x003E	,	0x003D	,	0x2265	,	0x0000),
DEADTRANS(	asciicircum	,	0x003D	,	0x207C	,	0x0000),
DEADTRANS(	0x006F	,	0x003D	,	0x229C	,	0x0000),
DEADTRANS(	0x007C	,	0x003D	,	0x22A8	,	0x0000),
DEADTRANS(	0x007E	,	0x003D	,	0x2245	,	0x0000),
DEADTRANS(	degree	,	0x003D	,	0x2257	,	0x0000),
DEADTRANS(	doublearrowleft	,	0x003D	,	0x27F8	,	0x0000),
DEADTRANS(	0x002D	,	0x003E	,	rightarrow	,	0x0000),
DEADTRANS(	0x002F	,	0x003E	,	0x226F	,	0x0000),
DEADTRANS(	0x003C	,	0x003E	,	0x2260	,	0x0000),
DEADTRANS(	0x003D	,	0x003E	,	0x2265	,	0x0000),
DEADTRANS(	0x003E	,	0x003E	,	0x226B	,	0x0000),
DEADTRANS(	asciicircum	,	0x003E	,	0x20D7	,	0x0000),
DEADTRANS(	0x007C	,	0x003E	,	0x22B3	,	0x0000),
DEADTRANS(	leftarrow	,	0x003E	,	leftrightarrow	,	0x0000),
DEADTRANS(	0x0069	,	0x003F	,	0x0294	,	0x0000),
DEADTRANS(	0x002F	,	0x0041	,	0x023A	,	0x0000),
DEADTRANS(	asciicircum	,	0x0041	,	0x1D2C	,	0x0000),
DEADTRANS(	0x002D	,	0x0042	,	0x0243	,	0x0000),
DEADTRANS(	0x0042	,	0x0042	,	0x0299	,	0x0000),
DEADTRANS(	asciicircum	,	0x0042	,	0x1D2E	,	0x0000),
DEADTRANS(	0x002F	,	0x0043	,	0x023B	,	0x0000),
DEADTRANS(	0x007C	,	0x0043	,	doublestruck_c	,	0x0000),
DEADTRANS(	degree	,	0x0043	,	0x2103	,	0x0000),
DEADTRANS(	asciicircum	,	0x0044	,	0x1D30	,	0x0000),
DEADTRANS(	0x002F	,	0x0045	,	0x0246	,	0x0000),
DEADTRANS(	asciicircum	,	0x0045	,	0x1D31	,	0x0000),
DEADTRANS(	degree	,	0x0046	,	0x2109	,	0x0000),
DEADTRANS(	0x0047	,	0x0047	,	0x0262	,	0x0000),
DEADTRANS(	0x0059	,	0x0047	,	0x021C	,	0x0000),
DEADTRANS(	asciicircum	,	0x0047	,	0x1D33	,	0x0000),
DEADTRANS(	0x0048	,	0x0048	,	0x029C	,	0x0000),
DEADTRANS(	0x0053	,	0x0048	,	0x01A9	,	0x0000),
DEADTRANS(	0x005A	,	0x0048	,	0x01B7	,	0x0000),
DEADTRANS(	asciicircum	,	0x0048	,	0x1D34	,	0x0000),
DEADTRANS(	0x007C	,	0x0048	,	0x210D	,	0x0000),
DEADTRANS(	0x002D	,	0x0049	,	0x0197	,	0x0000),
DEADTRANS(	0x0049	,	0x0049	,	0x026A	,	0x0000),
DEADTRANS(	asciicircum	,	0x0049	,	0x1D35	,	0x0000),
DEADTRANS(	0x002D	,	0x004A	,	0x0248	,	0x0000),
DEADTRANS(	asciicircum	,	0x004A	,	0x1D36	,	0x0000),
DEADTRANS(	asciicircum	,	0x004B	,	0x1D37	,	0x0000),
DEADTRANS(	0x002D	,	0x004C	,	0x023D	,	0x0000),
DEADTRANS(	0x004C	,	0x004C	,	0x029F	,	0x0000),
DEADTRANS(	asciicircum	,	0x004C	,	0x1D38	,	0x0000),
DEADTRANS(	asciicircum	,	0x004D	,	0x1D39	,	0x0000),
DEADTRANS(	0x004E	,	0x004E	,	0x0274	,	0x0000),
DEADTRANS(	asciicircum	,	0x004E	,	0x1D3A	,	0x0000),
DEADTRANS(	0x006A	,	0x004E	,	0x019D	,	0x0000),
DEADTRANS(	0x007C	,	0x004E	,	doublestruck_n	,	0x0000),
DEADTRANS(	0x002D	,	0x004F	,	0x019F	,	0x0000),
DEADTRANS(	asciicircum	,	0x004F	,	0x1D3C	,	0x0000),
DEADTRANS(	0x002D	,	0x0050	,	0x2C63	,	0x0000),
DEADTRANS(	asciicircum	,	0x0050	,	0x1D3E	,	0x0000),
DEADTRANS(	0x007C	,	0x0050	,	0x2119	,	0x0000),
DEADTRANS(	0x007C	,	0x0051	,	doublestruck_q	,	0x0000),
DEADTRANS(	0x002D	,	0x0052	,	0x024C	,	0x0000),
DEADTRANS(	0x0052	,	0x0052	,	0x0280	,	0x0000),
DEADTRANS(	asciicircum	,	0x0052	,	0x1D3F	,	0x0000),
DEADTRANS(	0x007C	,	0x0052	,	doublestruck_r	,	0x0000),
DEADTRANS(	0x002F	,	0x0054	,	0x023E	,	0x0000),
DEADTRANS(	asciicircum	,	0x0054	,	0x1D40	,	0x0000),
DEADTRANS(	0x002D	,	0x0055	,	0x0244	,	0x0000),
DEADTRANS(	0x004F	,	0x0055	,	0x0222	,	0x0000),
DEADTRANS(	asciicircum	,	0x0055	,	0x1D41	,	0x0000),
DEADTRANS(	0x0067	,	0x0055	,	0x0222	,	0x0000),
DEADTRANS(	asciicircum	,	0x0056	,	0x2C7D	,	0x0000),
DEADTRANS(	asciicircum	,	0x0057	,	0x1D42	,	0x0000),
DEADTRANS(	0x002D	,	0x0059	,	0x024E	,	0x0000),
DEADTRANS(	0x0059	,	0x0059	,	0x028F	,	0x0000),
DEADTRANS(	0x005A	,	0x005A	,	0x01B7	,	0x0000),
DEADTRANS(	0x007C	,	0x005A	,	doublestruck_z	,	0x0000),
DEADTRANS(	0x005B	,	0x005B	,	0x27E6	,	0x0000),
DEADTRANS(	asciicircum	,	0x005B	,	0x2308	,	0x0000),
DEADTRANS(	0x005F	,	0x005B	,	0x230A	,	0x0000),
DEADTRANS(	0x007C	,	0x005B	,	0x27E6	,	0x0000),
DEADTRANS(	backslash	,	backslash	,	setminus	,	0x0000),
DEADTRANS(	0x2026	,	backslash	,	0x22F1	,	0x0000),
DEADTRANS(	0x005D	,	0x005D	,	0x27E7	,	0x0000),
DEADTRANS(	asciicircum	,	0x005D	,	0x2309	,	0x0000),
DEADTRANS(	0x005F	,	0x005D	,	0x230B	,	0x0000),
DEADTRANS(	0x007C	,	0x005D	,	0x27E7	,	0x0000),
DEADTRANS(	0x002F	,	asciicircum	,	0x2197	,	0x0000),
DEADTRANS(	0x003D	,	asciicircum	,	0x2259	,	0x0000),
DEADTRANS(	backslash	,	asciicircum	,	0x2196	,	0x0000),
DEADTRANS(	0x007C	,	asciicircum	,	uparrow	,	0x0000),
DEADTRANS(	0x0072	,	0x005F	,	0x027E	,	0x0000),
DEADTRANS(	0x0078	,	0x005F	,	0x0264	,	0x0000),
DEADTRANS(	macron	,	grave	,	0x1DC6	,	0x0000),
DEADTRANS(	0x002F	,	0x0061	,	0x2C65	,	0x0000),
DEADTRANS(	asciicircum	,	0x0061	,	0x1D43	,	0x0000),
DEADTRANS(	0x005F	,	0x0061	,	0x2090	,	0x0000),
DEADTRANS(	0x005F	,	0x0061	,	0x2090	,	0x0000),
DEADTRANS(	0x0061	,	0x0061	,	0x0251	,	0x0000),
DEADTRANS(	0x0068	,	0x0061	,	alefsymbol	,	0x0000),
DEADTRANS(	asciicircum	,	0x0062	,	0x1D47	,	0x0000),
DEADTRANS(	0x0068	,	0x0062	,	0x2136	,	0x0000),
DEADTRANS(	0x002F	,	0x0063	,	0x023C	,	0x0000),
DEADTRANS(	asciicircum	,	0x0063	,	0x1D9C	,	0x0000),
DEADTRANS(	asciicircum	,	0x0064	,	0x1D48	,	0x0000),
DEADTRANS(	0x0068	,	0x0064	,	0x2138	,	0x0000),
DEADTRANS(	0x002F	,	0x0065	,	0x0247	,	0x0000),
DEADTRANS(	0x0052	,	0x0065	,	0x211C	,	0x0000),
DEADTRANS(	asciicircum	,	0x0065	,	0x1D49	,	0x0000),
DEADTRANS(	0x005F	,	0x0065	,	0x2091	,	0x0000),
DEADTRANS(	0x005F	,	0x0065	,	0x2091	,	0x0000),
DEADTRANS(	0x0072	,	0x0065	,	0x211C	,	0x0000),
DEADTRANS(	asciicircum	,	0x0066	,	0x1DA0	,	0x0000),
DEADTRANS(	0x006A	,	0x0066	,	0x0284	,	0x0000),
DEADTRANS(	0x0059	,	0x0067	,	0x021C	,	0x0000),
DEADTRANS(	asciicircum	,	0x0067	,	0x1D4D	,	0x0000),
DEADTRANS(	0x0067	,	0x0067	,	0x0261	,	0x0000),
DEADTRANS(	0x0068	,	0x0067	,	0x2137	,	0x0000),
DEADTRANS(	0x0079	,	0x0067	,	0x021D	,	0x0000),
DEADTRANS(	asciicircum	,	0x0068	,	0x02B0	,	0x0000),
DEADTRANS(	0x0073	,	0x0068	,	0x0283	,	0x0000),
DEADTRANS(	asciicircum	,	0x0069	,	0x2071	,	0x0000),
DEADTRANS(	0x005F	,	0x0069	,	0x1D62	,	0x0000),
DEADTRANS(	0x005F	,	0x0069	,	0x1D62	,	0x0000),
DEADTRANS(	0x002D	,	0x006A	,	0x0249	,	0x0000),
DEADTRANS(	asciicircum	,	0x006A	,	0x02B2	,	0x0000),
DEADTRANS(	0x005F	,	0x006A	,	0x2C7C	,	0x0000),
DEADTRANS(	0x005F	,	0x006A	,	0x2C7C	,	0x0000),
DEADTRANS(	asciicircum	,	0x006B	,	0x1D4F	,	0x0000),
DEADTRANS(	0x002D	,	0x006C	,	0x019A	,	0x0000),
DEADTRANS(	0x0044	,	0x006C	,	0x0189	,	0x0000),
DEADTRANS(	0x0054	,	0x006C	,	0x01AE	,	0x0000),
DEADTRANS(	asciicircum	,	0x006C	,	0x02E1	,	0x0000),
DEADTRANS(	0x0064	,	0x006C	,	0x0256	,	0x0000),
DEADTRANS(	0x006C	,	0x006C	,	0x026D	,	0x0000),
DEADTRANS(	0x006E	,	0x006C	,	0x0273	,	0x0000),
DEADTRANS(	0x0074	,	0x006C	,	0x0288	,	0x0000),
DEADTRANS(	0x0049	,	0x006D	,	0x2111	,	0x0000),
DEADTRANS(	asciicircum	,	0x006D	,	0x1D50	,	0x0000),
DEADTRANS(	0x0069	,	0x006D	,	0x2111	,	0x0000),
DEADTRANS(	asciicircum	,	0x006E	,	0x207F	,	0x0000),
DEADTRANS(	0x006A	,	0x006E	,	0x0272	,	0x0000),
DEADTRANS(	0x002D	,	0x006F	,	0x0275	,	0x0000),
DEADTRANS(	asciicircum	,	0x006F	,	0x1D52	,	0x0000),
DEADTRANS(	0x005F	,	0x006F	,	0x2092	,	0x0000),
DEADTRANS(	0x005F	,	0x006F	,	0x2092	,	0x0000),
DEADTRANS(	0x002D	,	0x0070	,	0x1D7D	,	0x0000),
DEADTRANS(	asciicircum	,	0x0070	,	0x1D56	,	0x0000),
DEADTRANS(	0x002D	,	0x0072	,	0x024D	,	0x0000),
DEADTRANS(	asciicircum	,	0x0072	,	0x02B3	,	0x0000),
DEADTRANS(	0x005F	,	0x0072	,	0x1D63	,	0x0000),
DEADTRANS(	0x005F	,	0x0072	,	0x1D63	,	0x0000),
DEADTRANS(	0x0072	,	0x0072	,	0x027C	,	0x0000),
DEADTRANS(	asciicircum	,	0x0073	,	0x02E2	,	0x0000),
DEADTRANS(	0x002F	,	0x0074	,	0x2C66	,	0x0000),
DEADTRANS(	asciicircum	,	0x0074	,	0x1D57	,	0x0000),
DEADTRANS(	0x002D	,	0x0075	,	0x0289	,	0x0000),
DEADTRANS(	asciicircum	,	0x0075	,	0x1D58	,	0x0000),
DEADTRANS(	0x005F	,	0x0075	,	0x1D64	,	0x0000),
DEADTRANS(	0x005F	,	0x0075	,	0x1D64	,	0x0000),
DEADTRANS(	0x002F	,	0x0076	,	0x2199	,	0x0000),
DEADTRANS(	0x0048	,	0x0076	,	0x01F6	,	0x0000),
DEADTRANS(	backslash	,	0x0076	,	0x2198	,	0x0000),
DEADTRANS(	asciicircum	,	0x0076	,	0x1D5B	,	0x0000),
DEADTRANS(	0x005F	,	0x0076	,	0x1D65	,	0x0000),
DEADTRANS(	0x005F	,	0x0076	,	0x1D65	,	0x0000),
DEADTRANS(	0x0068	,	0x0076	,	0x0195	,	0x0000),
DEADTRANS(	asciicircum	,	0x0077	,	0x02B7	,	0x0000),
DEADTRANS(	asciicircum	,	0x0078	,	0x02E3	,	0x0000),
DEADTRANS(	0x005F	,	0x0078	,	0x2093	,	0x0000),
DEADTRANS(	0x005F	,	0x0078	,	0x2093	,	0x0000),
DEADTRANS(	0x0061	,	0x0078	,	variation	,	0x0000),
DEADTRANS(	0x002D	,	0x0079	,	0x024F	,	0x0000),
DEADTRANS(	0x0057	,	0x0079	,	0x01F7	,	0x0000),
DEADTRANS(	asciicircum	,	0x0079	,	0x02B8	,	0x0000),
DEADTRANS(	0x0077	,	0x0079	,	0x01BF	,	0x0000),
DEADTRANS(	asciicircum	,	0x007A	,	0x1DBB	,	0x0000),
DEADTRANS(	0x006C	,	0x007A	,	0x026E	,	0x0000),
DEADTRANS(	0x007B	,	0x007B	,	0x2983	,	0x0000),
DEADTRANS(	0x007C	,	0x007B	,	0x2983	,	0x0000),
DEADTRANS(	0x002D	,	0x007C	,	0x22A3	,	0x0000),
DEADTRANS(	0x002F	,	0x007C	,	0x2224	,	0x0000),
DEADTRANS(	0x003C	,	0x007C	,	0x22B2	,	0x0000),
DEADTRANS(	0x0043	,	0x007C	,	0x2C6D	,	0x0000),
DEADTRANS(	0x004E	,	0x007C	,	0x0220	,	0x0000),
DEADTRANS(	0x006E	,	0x007C	,	0x019E	,	0x0000),
DEADTRANS(	0x0076	,	0x007C	,	downarrow	,	0x0000),
DEADTRANS(	0x007C	,	0x007C	,	0x2016	,	0x0000),
DEADTRANS(	guillemotleft	,	0x007C	,	0x219F	,	0x0000),
DEADTRANS(	multiply	,	0x007C	,	0x22CA	,	0x0000),
DEADTRANS(	0x2026	,	0x007C	,	0x22EE	,	0x0000),
DEADTRANS(	leftarrow	,	0x007C	,	0x21A4	,	0x0000),
DEADTRANS(	uparrow	,	0x007C	,	0x21A5	,	0x0000),
DEADTRANS(	rightarrow	,	0x007C	,	0x21E5	,	0x0000),
DEADTRANS(	downarrow	,	0x007C	,	0x21A7	,	0x0000),
DEADTRANS(	updownarrow	,	0x007C	,	0x21A8	,	0x0000),
DEADTRANS(	doublearrowleft	,	0x007C	,	0x2906	,	0x0000),
DEADTRANS(	0x007C	,	0x007D	,	0x2984	,	0x0000),
DEADTRANS(	0x007D	,	0x007D	,	0x2984	,	0x0000),
DEADTRANS(	0x002F	,	0x007E	,	0x2241	,	0x0000),
DEADTRANS(	0x003D	,	0x007E	,	0x2245	,	0x0000),
DEADTRANS(	0x005F	,	0x007E	,	0x0330	,	0x0000),
DEADTRANS(	0x007E	,	0x007E	,	0x2248	,	0x0000),
DEADTRANS(	0x002D	,	guillemotleft	,	0x291B	,	0x0000),
DEADTRANS(	0x007C	,	notsign	,	0x2AEC	,	0x0000),
DEADTRANS(	grave	,	macron	,	0x1DC5	,	0x0000),
DEADTRANS(	acute	,	macron	,	0x1DC7	,	0x0000),
DEADTRANS(	0x003D	,	degree	,	0x2257	,	0x0000),
DEADTRANS(	0x005F	,	degree	,	0x0325	,	0x0000),
DEADTRANS(	0x006F	,	degree	,	0x229A	,	0x0000),
DEADTRANS(	0x0077	,	degree	,	degree	,	0x0000),
DEADTRANS(	macron	,	acute	,	0x1DC4	,	0x0000),
DEADTRANS(	periodcentered	,	periodcentered	,	0x02D1	,	0x0000),
DEADTRANS(	0x002D	,	guillemotright	,	0x21A0	,	0x0000),
DEADTRANS(	0x007C	,	guillemotright	,	0x21A1	,	0x0000),
DEADTRANS(	0x006F	,	multiply	,	circledtimes	,	0x0000),
DEADTRANS(	0x002F	,	slong	,	0x1E9C	,	0x0000),
DEADTRANS(	0x007C	,	Greek_GAMMA	,	0x213E	,	0x0000),
DEADTRANS(	0x007C	,	Greek_PI	,	0x213F	,	0x0000),
DEADTRANS(	Greek_PI	,	Greek_PI	,	0x220F	,	0x0000),
DEADTRANS(	Greek_SIGMA	,	Greek_SIGMA	,	0x2211	,	0x0000),
DEADTRANS(	Greek_PHI	,	Greek_PHI	,	0x0278	,	0x0000),
DEADTRANS(	asciicircum	,	Greek_alpha	,	0x1D45	,	0x0000),
DEADTRANS(	Greek_alpha	,	Greek_alpha	,	0x0251	,	0x0000),
DEADTRANS(	asciicircum	,	Greek_beta	,	0x1D5D	,	0x0000),
DEADTRANS(	0x005F	,	Greek_beta	,	0x1D66	,	0x0000),
DEADTRANS(	0x005F	,	Greek_beta	,	0x1D66	,	0x0000),
DEADTRANS(	asciicircum	,	Greek_gamma	,	0x1D5E	,	0x0000),
DEADTRANS(	0x005F	,	Greek_gamma	,	0x1D67	,	0x0000),
DEADTRANS(	0x005F	,	Greek_gamma	,	0x1D67	,	0x0000),
DEADTRANS(	0x007C	,	Greek_gamma	,	0x213D	,	0x0000),
DEADTRANS(	Greek_gamma	,	Greek_gamma	,	0x0263	,	0x0000),
DEADTRANS(	asciicircum	,	Greek_delta	,	0x1D5F	,	0x0000),
DEADTRANS(	asciicircum	,	Greek_epsilon	,	0x1D4B	,	0x0000),
DEADTRANS(	Greek_epsilon	,	Greek_epsilon	,	0x025B	,	0x0000),
DEADTRANS(	asciicircum	,	Greek_theta	,	0x1DBF	,	0x0000),
DEADTRANS(	0x002F	,	Greek_lamda	,	0x019B	,	0x0000),
DEADTRANS(	0x03BC	,	0x03BC	,	mu	,	0x0000),
DEADTRANS(	0x007C	,	Greek_pi	,	0x213C	,	0x0000),
DEADTRANS(	0x005F	,	Greek_rho	,	0x1D68	,	0x0000),
DEADTRANS(	0x005F	,	Greek_rho	,	0x1D68	,	0x0000),
DEADTRANS(	asciicircum	,	Greek_upsilon	,	0x1DB7	,	0x0000),
DEADTRANS(	Greek_omicron	,	Greek_upsilon	,	0x0223	,	0x0000),
DEADTRANS(	Greek_upsilon	,	Greek_upsilon	,	0x028A	,	0x0000),
DEADTRANS(	asciicircum	,	Greek_phi	,	0x1D60	,	0x0000),
DEADTRANS(	0x005F	,	Greek_phi	,	0x1D69	,	0x0000),
DEADTRANS(	0x005F	,	Greek_phi	,	0x1D69	,	0x0000),
DEADTRANS(	Greek_phi	,	Greek_phi	,	0x0278	,	0x0000),
DEADTRANS(	asciicircum	,	Greek_chi	,	0x1D61	,	0x0000),
DEADTRANS(	0x005F	,	Greek_chi	,	0x1D6A	,	0x0000),
DEADTRANS(	0x005F	,	Greek_chi	,	0x1D6A	,	0x0000),
DEADTRANS(	asciicircum	,	scriptphi	,	0x1DB2	,	0x0000),
DEADTRANS(	scriptphi	,	scriptphi	,	0x0278	,	0x0000),
DEADTRANS(	0x2026	,	endash	,	0x22EF	,	0x0000),
DEADTRANS(	0x2026	,	emdash	,	0x22EF	,	0x0000),
DEADTRANS(	0x002F	,	leftarrow	,	0x219A	,	0x0000),
DEADTRANS(	asciicircum	,	leftarrow	,	0x20D6	,	0x0000),
DEADTRANS(	0x007C	,	leftarrow	,	0x21E4	,	0x0000),
DEADTRANS(	0x2026	,	leftarrow	,	0x21E0	,	0x0000),
DEADTRANS(	leftarrow	,	leftarrow	,	0x21C7	,	0x0000),
DEADTRANS(	rightarrow	,	leftarrow	,	0x21C4	,	0x0000),
DEADTRANS(	0x003D	,	uparrow	,	0x21DE	,	0x0000),
DEADTRANS(	0x007C	,	uparrow	,	0x21D1	,	0x0000),
DEADTRANS(	0x2026	,	uparrow	,	0x21E1	,	0x0000),
DEADTRANS(	uparrow	,	uparrow	,	0x21C8	,	0x0000),
DEADTRANS(	downarrow	,	uparrow	,	0x21F5	,	0x0000),
DEADTRANS(	0x002D	,	rightarrow	,	0x27F6	,	0x0000),
DEADTRANS(	0x003C	,	rightarrow	,	leftrightarrow	,	0x0000),
DEADTRANS(	0x003E	,	rightarrow	,	0x21A3	,	0x0000),
DEADTRANS(	asciicircum	,	rightarrow	,	0x20D7	,	0x0000),
DEADTRANS(	0x007A	,	rightarrow	,	0x21AF	,	0x0000),
DEADTRANS(	0x007C	,	rightarrow	,	arrowfrombarright	,	0x0000),
DEADTRANS(	0x2026	,	rightarrow	,	0x21E2	,	0x0000),
DEADTRANS(	leftarrow	,	rightarrow	,	0x21C6	,	0x0000),
DEADTRANS(	rightarrow	,	rightarrow	,	0x21C9	,	0x0000),
DEADTRANS(	0x003D	,	downarrow	,	0x21DF	,	0x0000),
DEADTRANS(	0x007C	,	downarrow	,	0x21D3	,	0x0000),
DEADTRANS(	0x2026	,	downarrow	,	0x21E3	,	0x0000),
DEADTRANS(	uparrow	,	downarrow	,	0x21C5	,	0x0000),
DEADTRANS(	downarrow	,	downarrow	,	0x21CA	,	0x0000),
DEADTRANS(	0x002F	,	leftrightarrow	,	0x21AE	,	0x0000),
DEADTRANS(	0x002F	,	doublearrowleft	,	0x21CD	,	0x0000),
DEADTRANS(	0x002F	,	implies	,	0x21CF	,	0x0000),
DEADTRANS(	0x003D	,	implies	,	0x27F9	,	0x0000),
DEADTRANS(	0x007C	,	implies	,	0x2907	,	0x0000),
DEADTRANS(	doublearrowleft	,	implies	,	0x27FA	,	0x0000),
DEADTRANS(	0x002F	,	ifonlyif	,	0x21CE	,	0x0000),
DEADTRANS(	0x002F	,	thereexists	,	0x2204	,	0x0000),
DEADTRANS(	0x002F	,	elementof	,	0x2209	,	0x0000),
DEADTRANS(	elementof	,	elementof	,	0x220B	,	0x0000),
DEADTRANS(	0x006F	,	jot	,	0x229A	,	0x0000),
DEADTRANS(	0x0033	,	radical	,	0x221B	,	0x0000),
DEADTRANS(	0x0034	,	radical	,	0x221C	,	0x0000),
DEADTRANS(	0x002F	,	0x2223	,	0x2224	,	0x0000),
DEADTRANS(	0x002F	,	parallelto	,	0x2226	,	0x0000),
DEADTRANS(	logicaland	,	logicaland	,	0x22C0	,	0x0000),
DEADTRANS(	logicalor	,	logicalor	,	0x22C1	,	0x0000),
DEADTRANS(	intersection	,	intersection	,	0x22C2	,	0x0000),
DEADTRANS(	union	,	union	,	0x22C3	,	0x0000),
DEADTRANS(	0x0032	,	integral	,	0x222C	,	0x0000),
DEADTRANS(	0x0033	,	integral	,	0x222D	,	0x0000),
DEADTRANS(	0x0034	,	integral	,	0x2A0C	,	0x0000),
DEADTRANS(	0x006F	,	integral	,	0x222E	,	0x0000),
DEADTRANS(	jot	,	integral	,	0x222E	,	0x0000),
DEADTRANS(	integral	,	integral	,	0x222C	,	0x0000),
DEADTRANS(	0x002F	,	0x2264	,	0x2270	,	0x0000),
DEADTRANS(	0x002F	,	0x2265	,	0x2271	,	0x0000),
DEADTRANS(	0x002F	,	includedin	,	0x2284	,	0x0000),
DEADTRANS(	0x002F	,	0x2283	,	0x2285	,	0x0000),
DEADTRANS(	0x2295	,	0x2295	,	0x2A01	,	0x0000),
DEADTRANS(	circledtimes	,	circledtimes	,	0x2A02	,	0x0000),
DEADTRANS(	0x2299	,	0x2299	,	0x2A00	,	0x0000),
DEADTRANS(	uptack	,	uptack	,	0x22A4	,	0x0000),


//en_US.UTF8

DEADTRANS(	0x0020	,	0x0020	,	nobreakspace	,	0x0000),
DEADTRANS(	0x0020	,	apostrophe	,	apostrophe	,	0x0000),
DEADTRANS(	0x0020	,	0x002C	,	cedilla	,	0x0000),
DEADTRANS(	0x0020	,	0x002D	,	0x007E	,	0x0000),
DEADTRANS(	0x0020	,	0x002E	,	0x2008	,	0x0000),
DEADTRANS(	0x0020	,	0x003E	,	asciicircum	,	0x0000),
DEADTRANS(	0x0020	,	asciicircum	,	asciicircum	,	0x0000),
DEADTRANS(	0x0020	,	grave	,	grave	,	0x0000),
DEADTRANS(	0x0020	,	0x007E	,	0x007E	,	0x0000),
DEADTRANS(	0x0021	,	0x0021	,	exclamdown	,	0x0000),
DEADTRANS(	0x0021	,	0x0041	,	0x1EA0	,	0x0000),
DEADTRANS(	0x0021	,	0x0042	,	0x1E04	,	0x0000),
DEADTRANS(	0x0021	,	0x0044	,	0x1E0C	,	0x0000),
DEADTRANS(	0x0021	,	0x0045	,	0x1EB8	,	0x0000),
DEADTRANS(	0x0021	,	0x0048	,	0x1E24	,	0x0000),
DEADTRANS(	0x0021	,	0x0049	,	0x1ECA	,	0x0000),
DEADTRANS(	0x0021	,	0x004B	,	0x1E32	,	0x0000),
DEADTRANS(	0x0021	,	0x004C	,	0x1E36	,	0x0000),
DEADTRANS(	0x0021	,	0x004D	,	0x1E42	,	0x0000),
DEADTRANS(	0x0021	,	0x004E	,	0x1E46	,	0x0000),
DEADTRANS(	0x0021	,	0x004F	,	0x1ECC	,	0x0000),
DEADTRANS(	0x0021	,	0x0052	,	0x1E5A	,	0x0000),
DEADTRANS(	0x0021	,	0x0053	,	0x1E62	,	0x0000),
DEADTRANS(	0x0021	,	0x0054	,	0x1E6C	,	0x0000),
DEADTRANS(	0x0021	,	0x0055	,	0x1EE4	,	0x0000),
DEADTRANS(	0x0021	,	0x0056	,	0x1E7E	,	0x0000),
DEADTRANS(	0x0021	,	0x0057	,	0x1E88	,	0x0000),
DEADTRANS(	0x0021	,	0x0059	,	0x1EF4	,	0x0000),
DEADTRANS(	0x0021	,	0x005A	,	0x1E92	,	0x0000),
DEADTRANS(	0x0021	,	asciicircum	,	0x00A6	,	0x0000),
DEADTRANS(	0x0021	,	0x0061	,	0x1EA1	,	0x0000),
DEADTRANS(	0x0021	,	0x0062	,	0x1E05	,	0x0000),
DEADTRANS(	0x0021	,	0x0064	,	0x1E0D	,	0x0000),
DEADTRANS(	0x0021	,	0x0065	,	0x1EB9	,	0x0000),
DEADTRANS(	0x0021	,	0x0068	,	0x1E25	,	0x0000),
DEADTRANS(	0x0021	,	0x0069	,	0x1ECB	,	0x0000),
DEADTRANS(	0x0021	,	0x006B	,	0x1E33	,	0x0000),
DEADTRANS(	0x0021	,	0x006C	,	0x1E37	,	0x0000),
DEADTRANS(	0x0021	,	0x006D	,	0x1E43	,	0x0000),
DEADTRANS(	0x0021	,	0x006E	,	0x1E47	,	0x0000),
DEADTRANS(	0x0021	,	0x006F	,	0x1ECD	,	0x0000),
DEADTRANS(	0x0021	,	0x0072	,	0x1E5B	,	0x0000),
DEADTRANS(	0x0021	,	0x0073	,	0x1E63	,	0x0000),
DEADTRANS(	0x0021	,	0x0074	,	0x1E6D	,	0x0000),
DEADTRANS(	0x0021	,	0x0075	,	0x1EE5	,	0x0000),
DEADTRANS(	0x0021	,	0x0076	,	0x1E7F	,	0x0000),
DEADTRANS(	0x0021	,	0x0077	,	0x1E89	,	0x0000),
DEADTRANS(	0x0021	,	0x0079	,	0x1EF5	,	0x0000),
DEADTRANS(	0x0021	,	0x007A	,	0x1E93	,	0x0000),
DEADTRANS(	0x0021	,	0x01A0	,	0x1EE2	,	0x0000),
DEADTRANS(	0x0021	,	0x01A1	,	0x1EE3	,	0x0000),
DEADTRANS(	0x0021	,	0x01AF	,	0x1EF0	,	0x0000),
DEADTRANS(	0x0021	,	0x01B0	,	0x1EF1	,	0x0000),
DEADTRANS(	quotedbl	,	apostrophe	,	0x0344	,	0x0000),
DEADTRANS(	quotedbl	,	0x002C	,	doublelowquotemark	,	0x0000),
DEADTRANS(	quotedbl	,	0x002F	,	0x301E	,	0x0000),
DEADTRANS(	quotedbl	,	0x003C	,	leftdoublequotemark	,	0x0000),
DEADTRANS(	quotedbl	,	0x003E	,	rightdoublequotemark	,	0x0000),
DEADTRANS(	quotedbl	,	0x0041	,	Adiaeresis	,	0x0000),
DEADTRANS(	quotedbl	,	0x0045	,	0x00CB	,	0x0000),
DEADTRANS(	quotedbl	,	0x0048	,	0x1E26	,	0x0000),
DEADTRANS(	quotedbl	,	0x0049	,	0x00CF	,	0x0000),
DEADTRANS(	quotedbl	,	0x004F	,	Odiaeresis	,	0x0000),
DEADTRANS(	quotedbl	,	0x0055	,	Udiaeresis	,	0x0000),
DEADTRANS(	quotedbl	,	0x0057	,	0x1E84	,	0x0000),
DEADTRANS(	quotedbl	,	0x0058	,	0x1E8C	,	0x0000),
DEADTRANS(	quotedbl	,	0x0059	,	0x0178	,	0x0000),
DEADTRANS(	quotedbl	,	backslash	,	0x301D	,	0x0000),
DEADTRANS(	quotedbl	,	0x0061	,	adiaeresis	,	0x0000),
DEADTRANS(	quotedbl	,	0x0065	,	0x00EB	,	0x0000),
DEADTRANS(	quotedbl	,	0x0068	,	0x1E27	,	0x0000),
DEADTRANS(	quotedbl	,	0x0069	,	0x00EF	,	0x0000),
DEADTRANS(	quotedbl	,	0x006F	,	0x00F6	,	0x0000),
DEADTRANS(	quotedbl	,	0x0074	,	0x1E97	,	0x0000),
DEADTRANS(	quotedbl	,	0x0075	,	udiaeresis	,	0x0000),
DEADTRANS(	quotedbl	,	0x0077	,	0x1E85	,	0x0000),
DEADTRANS(	quotedbl	,	0x0078	,	0x1E8D	,	0x0000),
DEADTRANS(	quotedbl	,	0x0079	,	0x00FF	,	0x0000),
DEADTRANS(	quotedbl	,	acute	,	0x0344	,	0x0000),
DEADTRANS(	quotedbl	,	acute	,	0x0344	,	0x0000),
DEADTRANS(	quotedbl	,	0x00D5	,	0x1E4E	,	0x0000),
DEADTRANS(	quotedbl	,	0x00F5	,	0x1E4F	,	0x0000),
DEADTRANS(	quotedbl	,	0x016A	,	0x1E7A	,	0x0000),
DEADTRANS(	quotedbl	,	0x016B	,	0x1E7B	,	0x0000),
DEADTRANS(	quotedbl	,	0x0399	,	0x03AA	,	0x0000),
DEADTRANS(	quotedbl	,	0x03A5	,	0x03AB	,	0x0000),
DEADTRANS(	quotedbl	,	Greek_iota	,	0x03CA	,	0x0000),
DEADTRANS(	quotedbl	,	Greek_upsilon	,	0x03CB	,	0x0000),
DEADTRANS(	quotedbl	,	0x03D2	,	0x03D4	,	0x0000),
DEADTRANS(	quotedbl	,	0x0406	,	0x0407	,	0x0000),
DEADTRANS(	quotedbl	,	0x0410	,	0x04D2	,	0x0000),
DEADTRANS(	quotedbl	,	0x0415	,	0x0401	,	0x0000),
DEADTRANS(	quotedbl	,	0x0416	,	0x04DC	,	0x0000),
DEADTRANS(	quotedbl	,	0x0417	,	0x04DE	,	0x0000),
DEADTRANS(	quotedbl	,	0x0418	,	0x04E4	,	0x0000),
DEADTRANS(	quotedbl	,	0x041E	,	0x04E6	,	0x0000),
DEADTRANS(	quotedbl	,	0x0423	,	0x04F0	,	0x0000),
DEADTRANS(	quotedbl	,	0x0427	,	0x04F4	,	0x0000),
DEADTRANS(	quotedbl	,	0x042B	,	0x04F8	,	0x0000),
DEADTRANS(	quotedbl	,	0x042D	,	0x04EC	,	0x0000),
DEADTRANS(	quotedbl	,	0x0430	,	0x04D3	,	0x0000),
DEADTRANS(	quotedbl	,	0x0435	,	0x0451	,	0x0000),
DEADTRANS(	quotedbl	,	0x0436	,	0x04DD	,	0x0000),
DEADTRANS(	quotedbl	,	0x0437	,	0x04DF	,	0x0000),
DEADTRANS(	quotedbl	,	0x0438	,	0x04E5	,	0x0000),
DEADTRANS(	quotedbl	,	0x043E	,	0x04E7	,	0x0000),
DEADTRANS(	quotedbl	,	0x0443	,	0x04F1	,	0x0000),
DEADTRANS(	quotedbl	,	0x0447	,	0x04F5	,	0x0000),
DEADTRANS(	quotedbl	,	0x044B	,	0x04F9	,	0x0000),
DEADTRANS(	quotedbl	,	0x044D	,	0x04ED	,	0x0000),
DEADTRANS(	quotedbl	,	0x0456	,	0x0457	,	0x0000),
DEADTRANS(	quotedbl	,	0x04D8	,	0x04DA	,	0x0000),
DEADTRANS(	quotedbl	,	0x04D9	,	0x04DB	,	0x0000),
DEADTRANS(	quotedbl	,	0x04E8	,	0x04EA	,	0x0000),
DEADTRANS(	quotedbl	,	0x04E9	,	0x04EB	,	0x0000),
DEADTRANS(	0x0023	,	0x0023	,	0x266F	,	0x0000),
DEADTRANS(	0x0023	,	0x0062	,	0x266D	,	0x0000),
DEADTRANS(	0x0023	,	0x0066	,	0x266E	,	0x0000),
DEADTRANS(	percent	,	0x006F	,	0x2030	,	0x0000),
DEADTRANS(	apostrophe	,	0x0020	,	apostrophe	,	0x0000),
DEADTRANS(	apostrophe	,	0x002C	,	singlelowquotemark	,	0x0000),
DEADTRANS(	apostrophe	,	0x003C	,	leftsinglequotemark	,	0x0000),
DEADTRANS(	apostrophe	,	0x003E	,	rightsinglequotemark	,	0x0000),
DEADTRANS(	apostrophe	,	0x0041	,	0x00C1	,	0x0000),
DEADTRANS(	apostrophe	,	0x0043	,	0x0106	,	0x0000),
DEADTRANS(	apostrophe	,	0x0045	,	0x00C9	,	0x0000),
DEADTRANS(	apostrophe	,	0x0047	,	0x01F4	,	0x0000),
DEADTRANS(	apostrophe	,	0x0049	,	0x00CD	,	0x0000),
DEADTRANS(	apostrophe	,	0x004B	,	0x1E30	,	0x0000),
DEADTRANS(	apostrophe	,	0x004C	,	0x0139	,	0x0000),
DEADTRANS(	apostrophe	,	0x004D	,	0x1E3E	,	0x0000),
DEADTRANS(	apostrophe	,	0x004E	,	0x0143	,	0x0000),
DEADTRANS(	apostrophe	,	0x004F	,	0x00D3	,	0x0000),
DEADTRANS(	apostrophe	,	0x0050	,	0x1E54	,	0x0000),
DEADTRANS(	apostrophe	,	0x0052	,	0x0154	,	0x0000),
DEADTRANS(	apostrophe	,	0x0053	,	0x015A	,	0x0000),
DEADTRANS(	apostrophe	,	0x0055	,	0x00DA	,	0x0000),
DEADTRANS(	apostrophe	,	0x0057	,	0x1E82	,	0x0000),
DEADTRANS(	apostrophe	,	0x0059	,	0x00DD	,	0x0000),
DEADTRANS(	apostrophe	,	0x005A	,	0x0179	,	0x0000),
DEADTRANS(	apostrophe	,	0x0061	,	0x00E1	,	0x0000),
DEADTRANS(	apostrophe	,	0x0063	,	0x0107	,	0x0000),
DEADTRANS(	apostrophe	,	0x0065	,	0x00E9	,	0x0000),
DEADTRANS(	apostrophe	,	0x0067	,	0x01F5	,	0x0000),
DEADTRANS(	apostrophe	,	0x0069	,	0x00ED	,	0x0000),
DEADTRANS(	apostrophe	,	0x006B	,	0x1E31	,	0x0000),
DEADTRANS(	apostrophe	,	0x006C	,	0x013A	,	0x0000),
DEADTRANS(	apostrophe	,	0x006D	,	0x1E3F	,	0x0000),
DEADTRANS(	apostrophe	,	0x006E	,	0x0144	,	0x0000),
DEADTRANS(	apostrophe	,	0x006F	,	0x00F3	,	0x0000),
DEADTRANS(	apostrophe	,	0x0070	,	0x1E55	,	0x0000),
DEADTRANS(	apostrophe	,	0x0072	,	0x0155	,	0x0000),
DEADTRANS(	apostrophe	,	0x0073	,	0x015B	,	0x0000),
DEADTRANS(	apostrophe	,	0x0075	,	0x00FA	,	0x0000),
DEADTRANS(	apostrophe	,	0x0077	,	0x1E83	,	0x0000),
DEADTRANS(	apostrophe	,	0x0079	,	0x00FD	,	0x0000),
DEADTRANS(	apostrophe	,	0x007A	,	0x017A	,	0x0000),
DEADTRANS(	apostrophe	,	0x00C2	,	0x1EA4	,	0x0000),
DEADTRANS(	apostrophe	,	0x00C5	,	0x01FA	,	0x0000),
DEADTRANS(	apostrophe	,	0x00C6	,	0x01FC	,	0x0000),
DEADTRANS(	apostrophe	,	0x00C7	,	0x1E08	,	0x0000),
DEADTRANS(	apostrophe	,	0x00CA	,	0x1EBE	,	0x0000),
DEADTRANS(	apostrophe	,	0x00CF	,	0x1E2E	,	0x0000),
DEADTRANS(	apostrophe	,	0x00D4	,	0x1ED0	,	0x0000),
DEADTRANS(	apostrophe	,	0x00D5	,	0x1E4C	,	0x0000),
DEADTRANS(	apostrophe	,	0x00D8	,	0x01FE	,	0x0000),
DEADTRANS(	apostrophe	,	Udiaeresis	,	0x01D7	,	0x0000),
DEADTRANS(	apostrophe	,	0x00E2	,	0x1EA5	,	0x0000),
DEADTRANS(	apostrophe	,	0x00E5	,	0x01FB	,	0x0000),
DEADTRANS(	apostrophe	,	0x00E6	,	0x01FD	,	0x0000),
DEADTRANS(	apostrophe	,	0x00E7	,	0x1E09	,	0x0000),
DEADTRANS(	apostrophe	,	0x00EA	,	0x1EBF	,	0x0000),
DEADTRANS(	apostrophe	,	0x00EF	,	0x1E2F	,	0x0000),
DEADTRANS(	apostrophe	,	0x00F4	,	0x1ED1	,	0x0000),
DEADTRANS(	apostrophe	,	0x00F5	,	0x1E4D	,	0x0000),
DEADTRANS(	apostrophe	,	0x00F8	,	0x01FF	,	0x0000),
DEADTRANS(	apostrophe	,	udiaeresis	,	0x01D8	,	0x0000),
DEADTRANS(	apostrophe	,	0x0102	,	0x1EAE	,	0x0000),
DEADTRANS(	apostrophe	,	0x0103	,	0x1EAF	,	0x0000),
DEADTRANS(	apostrophe	,	0x0112	,	0x1E16	,	0x0000),
DEADTRANS(	apostrophe	,	0x0113	,	0x1E17	,	0x0000),
DEADTRANS(	apostrophe	,	0x014C	,	0x1E52	,	0x0000),
DEADTRANS(	apostrophe	,	0x014D	,	0x1E53	,	0x0000),
DEADTRANS(	apostrophe	,	0x0168	,	0x1E78	,	0x0000),
DEADTRANS(	apostrophe	,	0x0169	,	0x1E79	,	0x0000),
DEADTRANS(	apostrophe	,	0x01A0	,	0x1EDA	,	0x0000),
DEADTRANS(	apostrophe	,	0x01A1	,	0x1EDB	,	0x0000),
DEADTRANS(	apostrophe	,	0x01AF	,	0x1EE8	,	0x0000),
DEADTRANS(	apostrophe	,	0x01B0	,	0x1EE9	,	0x0000),
DEADTRANS(	apostrophe	,	0x0391	,	0x0386	,	0x0000),
DEADTRANS(	apostrophe	,	0x0395	,	0x0388	,	0x0000),
DEADTRANS(	apostrophe	,	0x0397	,	0x0389	,	0x0000),
DEADTRANS(	apostrophe	,	0x0399	,	0x038A	,	0x0000),
DEADTRANS(	apostrophe	,	0x039F	,	0x038C	,	0x0000),
DEADTRANS(	apostrophe	,	0x03A5	,	0x038E	,	0x0000),
DEADTRANS(	apostrophe	,	Greek_OMEGA	,	0x038F	,	0x0000),
DEADTRANS(	apostrophe	,	Greek_alpha	,	0x03AC	,	0x0000),
DEADTRANS(	apostrophe	,	Greek_epsilon	,	0x03AD	,	0x0000),
DEADTRANS(	apostrophe	,	Greek_eta	,	0x03AE	,	0x0000),
DEADTRANS(	apostrophe	,	Greek_iota	,	0x03AF	,	0x0000),
DEADTRANS(	apostrophe	,	Greek_omicron	,	0x03CC	,	0x0000),
DEADTRANS(	apostrophe	,	Greek_upsilon	,	0x03CD	,	0x0000),
DEADTRANS(	apostrophe	,	Greek_omega	,	0x03CE	,	0x0000),
DEADTRANS(	apostrophe	,	0x03CA	,	0x0390	,	0x0000),
DEADTRANS(	apostrophe	,	0x03CB	,	0x03B0	,	0x0000),
DEADTRANS(	apostrophe	,	0x03D2	,	0x03D3	,	0x0000),
DEADTRANS(	apostrophe	,	0x0413	,	0x0403	,	0x0000),
DEADTRANS(	apostrophe	,	0x041A	,	0x040C	,	0x0000),
DEADTRANS(	apostrophe	,	0x0433	,	0x0453	,	0x0000),
DEADTRANS(	apostrophe	,	0x043A	,	0x045C	,	0x0000),
DEADTRANS(	apostrophe	,	0x1F00	,	0x1F04	,	0x0000),
DEADTRANS(	apostrophe	,	0x1F01	,	0x1F05	,	0x0000),
DEADTRANS(	apostrophe	,	0x1F08	,	0x1F0C	,	0x0000),
DEADTRANS(	apostrophe	,	0x1F09	,	0x1F0D	,	0x0000),
DEADTRANS(	apostrophe	,	0x1F10	,	0x1F14	,	0x0000),
DEADTRANS(	apostrophe	,	0x1F11	,	0x1F15	,	0x0000),
DEADTRANS(	apostrophe	,	0x1F18	,	0x1F1C	,	0x0000),
DEADTRANS(	apostrophe	,	0x1F19	,	0x1F1D	,	0x0000),
DEADTRANS(	apostrophe	,	0x1F20	,	0x1F24	,	0x0000),
DEADTRANS(	apostrophe	,	0x1F21	,	0x1F25	,	0x0000),
DEADTRANS(	apostrophe	,	0x1F28	,	0x1F2C	,	0x0000),
DEADTRANS(	apostrophe	,	0x1F29	,	0x1F2D	,	0x0000),
DEADTRANS(	apostrophe	,	0x1F30	,	0x1F34	,	0x0000),
DEADTRANS(	apostrophe	,	0x1F31	,	0x1F35	,	0x0000),
DEADTRANS(	apostrophe	,	0x1F38	,	0x1F3C	,	0x0000),
DEADTRANS(	apostrophe	,	0x1F39	,	0x1F3D	,	0x0000),
DEADTRANS(	apostrophe	,	0x1F40	,	0x1F44	,	0x0000),
DEADTRANS(	apostrophe	,	0x1F41	,	0x1F45	,	0x0000),
DEADTRANS(	apostrophe	,	0x1F48	,	0x1F4C	,	0x0000),
DEADTRANS(	apostrophe	,	0x1F49	,	0x1F4D	,	0x0000),
DEADTRANS(	apostrophe	,	0x1F50	,	0x1F54	,	0x0000),
DEADTRANS(	apostrophe	,	0x1F51	,	0x1F55	,	0x0000),
DEADTRANS(	apostrophe	,	0x1F59	,	0x1F5D	,	0x0000),
DEADTRANS(	apostrophe	,	0x1F60	,	0x1F64	,	0x0000),
DEADTRANS(	apostrophe	,	0x1F61	,	0x1F65	,	0x0000),
DEADTRANS(	apostrophe	,	0x1F68	,	0x1F6C	,	0x0000),
DEADTRANS(	apostrophe	,	0x1F69	,	0x1F6D	,	0x0000),
DEADTRANS(	0x0028	,	0x0028	,	0x005B	,	0x0000),
DEADTRANS(	0x0028	,	0x002D	,	0x007B	,	0x0000),
DEADTRANS(	0x0028	,	0x0391	,	0x1F09	,	0x0000),
DEADTRANS(	0x0028	,	0x0395	,	0x1F19	,	0x0000),
DEADTRANS(	0x0028	,	0x0397	,	0x1F29	,	0x0000),
DEADTRANS(	0x0028	,	0x0399	,	0x1F39	,	0x0000),
DEADTRANS(	0x0028	,	0x039F	,	0x1F49	,	0x0000),
DEADTRANS(	0x0028	,	0x03A1	,	0x1FEC	,	0x0000),
DEADTRANS(	0x0028	,	0x03A5	,	0x1F59	,	0x0000),
DEADTRANS(	0x0028	,	Greek_OMEGA	,	0x1F69	,	0x0000),
DEADTRANS(	0x0028	,	Greek_alpha	,	0x1F01	,	0x0000),
DEADTRANS(	0x0028	,	Greek_epsilon	,	0x1F11	,	0x0000),
DEADTRANS(	0x0028	,	Greek_eta	,	0x1F21	,	0x0000),
DEADTRANS(	0x0028	,	Greek_iota	,	0x1F31	,	0x0000),
DEADTRANS(	0x0028	,	Greek_omicron	,	0x1F41	,	0x0000),
DEADTRANS(	0x0028	,	Greek_rho	,	0x1FE5	,	0x0000),
DEADTRANS(	0x0028	,	Greek_upsilon	,	0x1F51	,	0x0000),
DEADTRANS(	0x0028	,	Greek_omega	,	0x1F61	,	0x0000),
DEADTRANS(	0x0029	,	0x0029	,	0x005D	,	0x0000),
DEADTRANS(	0x0029	,	0x002D	,	0x007D	,	0x0000),
DEADTRANS(	0x0029	,	0x0391	,	0x1F08	,	0x0000),
DEADTRANS(	0x0029	,	0x0395	,	0x1F18	,	0x0000),
DEADTRANS(	0x0029	,	0x0397	,	0x1F28	,	0x0000),
DEADTRANS(	0x0029	,	0x0399	,	0x1F38	,	0x0000),
DEADTRANS(	0x0029	,	0x039F	,	0x1F48	,	0x0000),
DEADTRANS(	0x0029	,	Greek_OMEGA	,	0x1F68	,	0x0000),
DEADTRANS(	0x0029	,	Greek_alpha	,	0x1F00	,	0x0000),
DEADTRANS(	0x0029	,	Greek_epsilon	,	0x1F10	,	0x0000),
DEADTRANS(	0x0029	,	Greek_eta	,	0x1F20	,	0x0000),
DEADTRANS(	0x0029	,	Greek_iota	,	0x1F30	,	0x0000),
DEADTRANS(	0x0029	,	Greek_omicron	,	0x1F40	,	0x0000),
DEADTRANS(	0x0029	,	Greek_rho	,	0x1FE4	,	0x0000),
DEADTRANS(	0x0029	,	Greek_upsilon	,	0x1F50	,	0x0000),
DEADTRANS(	0x0029	,	Greek_omega	,	0x1F60	,	0x0000),
DEADTRANS(	0x002B	,	0x002B	,	0x0023	,	0x0000),
DEADTRANS(	0x002B	,	0x002D	,	plusminus	,	0x0000),
DEADTRANS(	0x002B	,	0x004F	,	0x01A0	,	0x0000),
DEADTRANS(	0x002B	,	0x0055	,	0x01AF	,	0x0000),
DEADTRANS(	0x002B	,	0x006F	,	0x01A1	,	0x0000),
DEADTRANS(	0x002B	,	0x0075	,	0x01B0	,	0x0000),
DEADTRANS(	0x002C	,	0x0020	,	cedilla	,	0x0000),
DEADTRANS(	0x002C	,	quotedbl	,	doublelowquotemark	,	0x0000),
DEADTRANS(	0x002C	,	apostrophe	,	singlelowquotemark	,	0x0000),
DEADTRANS(	0x002C	,	0x002D	,	notsign	,	0x0000),
DEADTRANS(	0x002C	,	0x0043	,	0x00C7	,	0x0000),
DEADTRANS(	0x002C	,	0x0044	,	0x1E10	,	0x0000),
DEADTRANS(	0x002C	,	0x0045	,	0x0228	,	0x0000),
DEADTRANS(	0x002C	,	0x0047	,	0x0122	,	0x0000),
DEADTRANS(	0x002C	,	0x0048	,	0x1E28	,	0x0000),
DEADTRANS(	0x002C	,	0x004B	,	0x0136	,	0x0000),
DEADTRANS(	0x002C	,	0x004C	,	0x013B	,	0x0000),
DEADTRANS(	0x002C	,	0x004E	,	0x0145	,	0x0000),
DEADTRANS(	0x002C	,	0x0052	,	0x0156	,	0x0000),
DEADTRANS(	0x002C	,	0x0053	,	0x015E	,	0x0000),
DEADTRANS(	0x002C	,	0x0054	,	0x0162	,	0x0000),
DEADTRANS(	0x002C	,	0x0063	,	0x00E7	,	0x0000),
DEADTRANS(	0x002C	,	0x0064	,	0x1E11	,	0x0000),
DEADTRANS(	0x002C	,	0x0065	,	0x0229	,	0x0000),
DEADTRANS(	0x002C	,	0x0067	,	0x0123	,	0x0000),
DEADTRANS(	0x002C	,	0x0068	,	0x1E29	,	0x0000),
DEADTRANS(	0x002C	,	0x006B	,	0x0137	,	0x0000),
DEADTRANS(	0x002C	,	0x006C	,	0x013C	,	0x0000),
DEADTRANS(	0x002C	,	0x006E	,	0x0146	,	0x0000),
DEADTRANS(	0x002C	,	0x0072	,	0x0157	,	0x0000),
DEADTRANS(	0x002C	,	0x0073	,	0x015F	,	0x0000),
DEADTRANS(	0x002C	,	0x0074	,	0x0163	,	0x0000),
DEADTRANS(	0x002D	,	0x0020	,	0x007E	,	0x0000),
DEADTRANS(	0x002D	,	0x0028	,	0x007B	,	0x0000),
DEADTRANS(	0x002D	,	0x0029	,	0x007D	,	0x0000),
DEADTRANS(	0x002D	,	0x002C	,	notsign	,	0x0000),
DEADTRANS(	0x002D	,	0x003A	,	division	,	0x0000),
DEADTRANS(	0x002D	,	0x0044	,	0x0110	,	0x0000),
DEADTRANS(	0x002D	,	0x004C	,	sterling	,	0x0000),
DEADTRANS(	0x002D	,	0x0064	,	0x0111	,	0x0000),
DEADTRANS(	0x002E	,	0x002E	,	periodcentered	,	0x0000),
DEADTRANS(	0x002E	,	0x003C	,	guillemotleft2	,	0x0000),
DEADTRANS(	0x002E	,	0x003E	,	guillemotright2	,	0x0000),
DEADTRANS(	0x002E	,	0x0041	,	0x0226	,	0x0000),
DEADTRANS(	0x002E	,	0x0042	,	0x1E02	,	0x0000),
DEADTRANS(	0x002E	,	0x0043	,	0x010A	,	0x0000),
DEADTRANS(	0x002E	,	0x0044	,	0x1E0A	,	0x0000),
DEADTRANS(	0x002E	,	0x0045	,	0x0116	,	0x0000),
DEADTRANS(	0x002E	,	0x0046	,	0x1E1E	,	0x0000),
DEADTRANS(	0x002E	,	0x0047	,	0x0120	,	0x0000),
DEADTRANS(	0x002E	,	0x0048	,	0x1E22	,	0x0000),
DEADTRANS(	0x002E	,	0x0049	,	0x0130	,	0x0000),
DEADTRANS(	0x002E	,	0x004D	,	0x1E40	,	0x0000),
DEADTRANS(	0x002E	,	0x004E	,	0x1E44	,	0x0000),
DEADTRANS(	0x002E	,	0x004F	,	0x022E	,	0x0000),
DEADTRANS(	0x002E	,	0x0050	,	0x1E56	,	0x0000),
DEADTRANS(	0x002E	,	0x0052	,	0x1E58	,	0x0000),
DEADTRANS(	0x002E	,	0x0053	,	0x1E60	,	0x0000),
DEADTRANS(	0x002E	,	0x0054	,	0x1E6A	,	0x0000),
DEADTRANS(	0x002E	,	0x0057	,	0x1E86	,	0x0000),
DEADTRANS(	0x002E	,	0x0058	,	0x1E8A	,	0x0000),
DEADTRANS(	0x002E	,	0x0059	,	0x1E8E	,	0x0000),
DEADTRANS(	0x002E	,	0x005A	,	0x017B	,	0x0000),
DEADTRANS(	0x002E	,	0x0061	,	0x0227	,	0x0000),
DEADTRANS(	0x002E	,	0x0062	,	0x1E03	,	0x0000),
DEADTRANS(	0x002E	,	0x0063	,	0x010B	,	0x0000),
DEADTRANS(	0x002E	,	0x0064	,	0x1E0B	,	0x0000),
DEADTRANS(	0x002E	,	0x0065	,	0x0117	,	0x0000),
DEADTRANS(	0x002E	,	0x0066	,	0x1E1F	,	0x0000),
DEADTRANS(	0x002E	,	0x0067	,	0x0121	,	0x0000),
DEADTRANS(	0x002E	,	0x0068	,	0x1E23	,	0x0000),
DEADTRANS(	0x002E	,	0x006D	,	0x1E41	,	0x0000),
DEADTRANS(	0x002E	,	0x006E	,	0x1E45	,	0x0000),
DEADTRANS(	0x002E	,	0x006F	,	0x022F	,	0x0000),
DEADTRANS(	0x002E	,	0x0070	,	0x1E57	,	0x0000),
DEADTRANS(	0x002E	,	0x0072	,	0x1E59	,	0x0000),
DEADTRANS(	0x002E	,	0x0073	,	0x1E61	,	0x0000),
DEADTRANS(	0x002E	,	0x0074	,	0x1E6B	,	0x0000),
DEADTRANS(	0x002E	,	0x0077	,	0x1E87	,	0x0000),
DEADTRANS(	0x002E	,	0x0078	,	0x1E8B	,	0x0000),
DEADTRANS(	0x002E	,	0x0079	,	0x1E8F	,	0x0000),
DEADTRANS(	0x002E	,	0x007A	,	0x017C	,	0x0000),
DEADTRANS(	0x002E	,	0x015A	,	0x1E64	,	0x0000),
DEADTRANS(	0x002E	,	0x015B	,	0x1E65	,	0x0000),
DEADTRANS(	0x002E	,	0x0160	,	0x1E66	,	0x0000),
DEADTRANS(	0x002E	,	0x0161	,	0x1E67	,	0x0000),
DEADTRANS(	0x002E	,	slong	,	0x1E9B	,	0x0000),
DEADTRANS(	0x002E	,	0x1E62	,	0x1E68	,	0x0000),
DEADTRANS(	0x002E	,	0x1E63	,	0x1E69	,	0x0000),
DEADTRANS(	0x002F	,	0x002F	,	backslash	,	0x0000),
DEADTRANS(	0x002F	,	0x003C	,	backslash	,	0x0000),
DEADTRANS(	0x002F	,	0x0043	,	cent	,	0x0000),
DEADTRANS(	0x002F	,	0x0043	,	0x20A1	,	0x0000),
DEADTRANS(	0x002F	,	0x0044	,	0x0110	,	0x0000),
DEADTRANS(	0x002F	,	0x0047	,	0x01E4	,	0x0000),
DEADTRANS(	0x002F	,	0x0048	,	0x0126	,	0x0000),
DEADTRANS(	0x002F	,	0x0049	,	0x0197	,	0x0000),
DEADTRANS(	0x002F	,	0x004C	,	0x0141	,	0x0000),
DEADTRANS(	0x002F	,	0x004F	,	0x00D8	,	0x0000),
DEADTRANS(	0x002F	,	0x0054	,	0x0166	,	0x0000),
DEADTRANS(	0x002F	,	0x005A	,	0x01B5	,	0x0000),
DEADTRANS(	0x002F	,	asciicircum	,	0x007C	,	0x0000),
DEADTRANS(	0x002F	,	0x0062	,	0x0180	,	0x0000),
DEADTRANS(	0x002F	,	0x0063	,	cent	,	0x0000),
DEADTRANS(	0x002F	,	0x0064	,	0x0111	,	0x0000),
DEADTRANS(	0x002F	,	0x0067	,	0x01E5	,	0x0000),
DEADTRANS(	0x002F	,	0x0068	,	0x0127	,	0x0000),
DEADTRANS(	0x002F	,	0x0069	,	0x0268	,	0x0000),
DEADTRANS(	0x002F	,	0x006C	,	0x0142	,	0x0000),
DEADTRANS(	0x002F	,	0x006D	,	0x20A5	,	0x0000),
DEADTRANS(	0x002F	,	0x006F	,	0x00F8	,	0x0000),
DEADTRANS(	0x002F	,	0x0074	,	0x0167	,	0x0000),
DEADTRANS(	0x002F	,	0x007A	,	0x01B6	,	0x0000),
DEADTRANS(	0x002F	,	0x0294	,	0x02A1	,	0x0000),
DEADTRANS(	0x002F	,	0x0413	,	0x0492	,	0x0000),
DEADTRANS(	0x002F	,	0x041A	,	0x049E	,	0x0000),
DEADTRANS(	0x002F	,	0x0433	,	0x0493	,	0x0000),
DEADTRANS(	0x002F	,	0x043A	,	0x049F	,	0x0000),
DEADTRANS(	0x002F	,	0x04AE	,	0x04B0	,	0x0000),
DEADTRANS(	0x002F	,	0x04AF	,	0x04B1	,	0x0000),
DEADTRANS(	0x002F	,	leftarrow	,	0x219A	,	0x0000),
DEADTRANS(	0x002F	,	rightarrow	,	0x219B	,	0x0000),
DEADTRANS(	0x002F	,	leftrightarrow	,	0x21AE	,	0x0000),
DEADTRANS(	0x0031	,	0x0032	,	0x00BD	,	0x0000),
DEADTRANS(	0x0031	,	0x0034	,	0x00BC	,	0x0000),
DEADTRANS(	0x0033	,	0x0034	,	0x00BE	,	0x0000),
DEADTRANS(	0x003A	,	0x002D	,	division	,	0x0000),
DEADTRANS(	0x003B	,	0x0041	,	0x0104	,	0x0000),
DEADTRANS(	0x003B	,	0x0045	,	0x0118	,	0x0000),
DEADTRANS(	0x003B	,	0x0049	,	0x012E	,	0x0000),
DEADTRANS(	0x003B	,	0x004F	,	0x01EA	,	0x0000),
DEADTRANS(	0x003B	,	0x0055	,	0x0172	,	0x0000),
DEADTRANS(	0x003B	,	0x0061	,	0x0105	,	0x0000),
DEADTRANS(	0x003B	,	0x0065	,	0x0119	,	0x0000),
DEADTRANS(	0x003B	,	0x0069	,	0x012F	,	0x0000),
DEADTRANS(	0x003B	,	0x006F	,	0x01EB	,	0x0000),
DEADTRANS(	0x003B	,	0x0075	,	0x0173	,	0x0000),
DEADTRANS(	0x003C	,	quotedbl	,	leftdoublequotemark	,	0x0000),
DEADTRANS(	0x003C	,	apostrophe	,	leftsinglequotemark	,	0x0000),
DEADTRANS(	0x003C	,	0x002F	,	backslash	,	0x0000),
DEADTRANS(	0x003C	,	0x003C	,	guillemotleft	,	0x0000),
DEADTRANS(	0x003C	,	0x0338	,	0x226E	,	0x0000),
DEADTRANS(	0x003C	,	0x0338	,	0x226E	,	0x0000),
DEADTRANS(	0x003D	,	0x0043	,	EuroSign	,	0x0000),
DEADTRANS(	0x003D	,	0x0045	,	EuroSign	,	0x0000),
DEADTRANS(	0x003D	,	0x004C	,	0x20A4	,	0x0000),
DEADTRANS(	0x003D	,	0x004E	,	0x20A6	,	0x0000),
DEADTRANS(	0x003D	,	0x004F	,	0x0150	,	0x0000),
DEADTRANS(	0x003D	,	0x0055	,	0x0170	,	0x0000),
DEADTRANS(	0x003D	,	0x0057	,	0x20A9	,	0x0000),
DEADTRANS(	0x003D	,	0x0059	,	yen	,	0x0000),
DEADTRANS(	0x003D	,	0x006F	,	0x0151	,	0x0000),
DEADTRANS(	0x003D	,	0x0075	,	0x0171	,	0x0000),
DEADTRANS(	0x003D	,	0x0338	,	0x2260	,	0x0000),
DEADTRANS(	0x003D	,	0x0423	,	0x04F2	,	0x0000),
DEADTRANS(	0x003D	,	0x0443	,	0x04F3	,	0x0000),
DEADTRANS(	0x003E	,	0x0020	,	asciicircum	,	0x0000),
DEADTRANS(	0x003E	,	quotedbl	,	rightdoublequotemark	,	0x0000),
DEADTRANS(	0x003E	,	apostrophe	,	rightsinglequotemark	,	0x0000),
DEADTRANS(	0x003E	,	0x003E	,	guillemotright	,	0x0000),
DEADTRANS(	0x003E	,	0x0338	,	0x226F	,	0x0000),
DEADTRANS(	0x003E	,	0x0338	,	0x226F	,	0x0000),
DEADTRANS(	0x003F	,	0x003F	,	questiondown	,	0x0000),
DEADTRANS(	0x003F	,	0x0041	,	0x1EA2	,	0x0000),
DEADTRANS(	0x003F	,	0x0045	,	0x1EBA	,	0x0000),
DEADTRANS(	0x003F	,	0x0049	,	0x1EC8	,	0x0000),
DEADTRANS(	0x003F	,	0x004F	,	0x1ECE	,	0x0000),
DEADTRANS(	0x003F	,	0x0055	,	0x1EE6	,	0x0000),
DEADTRANS(	0x003F	,	0x0059	,	0x1EF6	,	0x0000),
DEADTRANS(	0x003F	,	0x0061	,	0x1EA3	,	0x0000),
DEADTRANS(	0x003F	,	0x0065	,	0x1EBB	,	0x0000),
DEADTRANS(	0x003F	,	0x0069	,	0x1EC9	,	0x0000),
DEADTRANS(	0x003F	,	0x006F	,	0x1ECF	,	0x0000),
DEADTRANS(	0x003F	,	0x0075	,	0x1EE7	,	0x0000),
DEADTRANS(	0x003F	,	0x0079	,	0x1EF7	,	0x0000),
DEADTRANS(	0x003F	,	0x00C2	,	0x1EA8	,	0x0000),
DEADTRANS(	0x003F	,	0x00CA	,	0x1EC2	,	0x0000),
DEADTRANS(	0x003F	,	0x00D4	,	0x1ED4	,	0x0000),
DEADTRANS(	0x003F	,	0x00E2	,	0x1EA9	,	0x0000),
DEADTRANS(	0x003F	,	0x00EA	,	0x1EC3	,	0x0000),
DEADTRANS(	0x003F	,	0x00F4	,	0x1ED5	,	0x0000),
DEADTRANS(	0x003F	,	0x0102	,	0x1EB2	,	0x0000),
DEADTRANS(	0x003F	,	0x0103	,	0x1EB3	,	0x0000),
DEADTRANS(	0x003F	,	0x01A0	,	0x1EDE	,	0x0000),
DEADTRANS(	0x003F	,	0x01A1	,	0x1EDF	,	0x0000),
DEADTRANS(	0x003F	,	0x01AF	,	0x1EEC	,	0x0000),
DEADTRANS(	0x003F	,	0x01B0	,	0x1EED	,	0x0000),
DEADTRANS(	0x0041	,	0x0045	,	0x00C6	,	0x0000),
DEADTRANS(	0x0041	,	0x0054	,	0x0040	,	0x0000),
DEADTRANS(	0x0043	,	0x002F	,	cent	,	0x0000),
DEADTRANS(	0x0043	,	0x002F	,	0x20A1	,	0x0000),
DEADTRANS(	0x0043	,	0x003D	,	EuroSign	,	0x0000),
DEADTRANS(	0x0043	,	0x0045	,	0x20A0	,	0x0000),
DEADTRANS(	0x0043	,	0x0072	,	0x20A2	,	0x0000),
DEADTRANS(	0x0043	,	0x007C	,	cent	,	0x0000),
DEADTRANS(	0x0044	,	0x0048	,	0x00D0	,	0x0000),
DEADTRANS(	0x0045	,	0x003D	,	EuroSign	,	0x0000),
DEADTRANS(	0x0046	,	0x0072	,	0x20A3	,	0x0000),
DEADTRANS(	0x004C	,	0x002D	,	sterling	,	0x0000),
DEADTRANS(	0x004C	,	0x003D	,	0x20A4	,	0x0000),
DEADTRANS(	0x004C	,	0x0056	,	0x007C	,	0x0000),
DEADTRANS(	0x004E	,	0x003D	,	0x20A6	,	0x0000),
DEADTRANS(	0x004E	,	0x0047	,	0x014A	,	0x0000),
DEADTRANS(	0x004F	,	0x0043	,	0x00A9	,	0x0000),
DEADTRANS(	0x004F	,	0x0045	,	0x0152	,	0x0000),
DEADTRANS(	0x004F	,	0x0052	,	0x00AE	,	0x0000),
DEADTRANS(	0x004F	,	0x0063	,	0x00A9	,	0x0000),
DEADTRANS(	0x004F	,	0x0072	,	0x00AE	,	0x0000),
DEADTRANS(	0x0050	,	0x0021	,	0x00B6	,	0x0000),
DEADTRANS(	0x0050	,	0x0050	,	0x00B6	,	0x0000),
DEADTRANS(	0x0050	,	0x0074	,	0x20A7	,	0x0000),
DEADTRANS(	0x0052	,	0x0073	,	0x20A8	,	0x0000),
DEADTRANS(	0x0054	,	0x0048	,	0x00DE	,	0x0000),
DEADTRANS(	0x0055	,	0x0041	,	0x0102	,	0x0000),
DEADTRANS(	0x0055	,	0x0045	,	0x0114	,	0x0000),
DEADTRANS(	0x0055	,	0x0047	,	0x011E	,	0x0000),
DEADTRANS(	0x0055	,	0x0049	,	0x012C	,	0x0000),
DEADTRANS(	0x0055	,	0x004F	,	0x014E	,	0x0000),
DEADTRANS(	0x0055	,	0x0055	,	0x016C	,	0x0000),
DEADTRANS(	0x0055	,	0x0061	,	0x0103	,	0x0000),
DEADTRANS(	0x0055	,	0x0065	,	0x0115	,	0x0000),
DEADTRANS(	0x0055	,	0x0067	,	0x011F	,	0x0000),
DEADTRANS(	0x0055	,	0x0069	,	0x012D	,	0x0000),
DEADTRANS(	0x0055	,	0x006F	,	0x014F	,	0x0000),
DEADTRANS(	0x0055	,	0x0075	,	0x016D	,	0x0000),
DEADTRANS(	0x0055	,	0x0228	,	0x1E1C	,	0x0000),
DEADTRANS(	0x0055	,	0x0229	,	0x1E1D	,	0x0000),
DEADTRANS(	0x0055	,	0x0391	,	0x1FB8	,	0x0000),
DEADTRANS(	0x0055	,	0x0399	,	0x1FD8	,	0x0000),
DEADTRANS(	0x0055	,	0x03A5	,	0x1FE8	,	0x0000),
DEADTRANS(	0x0055	,	Greek_alpha	,	0x1FB0	,	0x0000),
DEADTRANS(	0x0055	,	Greek_iota	,	0x1FD0	,	0x0000),
DEADTRANS(	0x0055	,	Greek_upsilon	,	0x1FE0	,	0x0000),
DEADTRANS(	0x0055	,	0x0410	,	0x04D0	,	0x0000),
DEADTRANS(	0x0055	,	0x0415	,	0x04D6	,	0x0000),
DEADTRANS(	0x0055	,	0x0416	,	0x04C1	,	0x0000),
DEADTRANS(	0x0055	,	0x0418	,	0x0419	,	0x0000),
DEADTRANS(	0x0055	,	0x0423	,	0x040E	,	0x0000),
DEADTRANS(	0x0055	,	0x0430	,	0x04D1	,	0x0000),
DEADTRANS(	0x0055	,	0x0435	,	0x04D7	,	0x0000),
DEADTRANS(	0x0055	,	0x0436	,	0x04C2	,	0x0000),
DEADTRANS(	0x0055	,	0x0438	,	0x0439	,	0x0000),
DEADTRANS(	0x0055	,	0x0443	,	0x045E	,	0x0000),
DEADTRANS(	0x0055	,	0x1EA0	,	0x1EB6	,	0x0000),
DEADTRANS(	0x0055	,	0x1EA1	,	0x1EB7	,	0x0000),
DEADTRANS(	0x0056	,	0x004C	,	0x007C	,	0x0000),
DEADTRANS(	0x0057	,	0x003D	,	0x20A9	,	0x0000),
DEADTRANS(	0x0059	,	0x003D	,	yen	,	0x0000),
DEADTRANS(	asciicircum	,	0x0020	,	asciicircum	,	0x0000),
DEADTRANS(	asciicircum	,	0x0028	,	0x207D	,	0x0000),
DEADTRANS(	asciicircum	,	0x0029	,	0x207E	,	0x0000),
DEADTRANS(	asciicircum	,	0x002B	,	0x207A	,	0x0000),
DEADTRANS(	asciicircum	,	0x002F	,	0x007C	,	0x0000),
DEADTRANS(	asciicircum	,	0x0030	,	0x2070	,	0x0000),
DEADTRANS(	asciicircum	,	0x0031	,	onesuperior	,	0x0000),
DEADTRANS(	asciicircum	,	0x0032	,	twosuperior	,	0x0000),
DEADTRANS(	asciicircum	,	0x0033	,	threesuperior	,	0x0000),
DEADTRANS(	asciicircum	,	0x0034	,	0x2074	,	0x0000),
DEADTRANS(	asciicircum	,	0x0035	,	0x2075	,	0x0000),
DEADTRANS(	asciicircum	,	0x0036	,	0x2076	,	0x0000),
DEADTRANS(	asciicircum	,	0x0037	,	0x2077	,	0x0000),
DEADTRANS(	asciicircum	,	0x0038	,	0x2078	,	0x0000),
DEADTRANS(	asciicircum	,	0x0039	,	0x2079	,	0x0000),
DEADTRANS(	asciicircum	,	0x003D	,	0x207C	,	0x0000),
DEADTRANS(	asciicircum	,	0x0041	,	0x00C2	,	0x0000),
DEADTRANS(	asciicircum	,	0x0043	,	0x0108	,	0x0000),
DEADTRANS(	asciicircum	,	0x0045	,	0x00CA	,	0x0000),
DEADTRANS(	asciicircum	,	0x0047	,	0x011C	,	0x0000),
DEADTRANS(	asciicircum	,	0x0048	,	0x0124	,	0x0000),
DEADTRANS(	asciicircum	,	0x0049	,	0x00CE	,	0x0000),
DEADTRANS(	asciicircum	,	0x004A	,	0x0134	,	0x0000),
DEADTRANS(	asciicircum	,	0x004F	,	0x00D4	,	0x0000),
DEADTRANS(	asciicircum	,	0x0053	,	0x015C	,	0x0000),
DEADTRANS(	asciicircum	,	0x0055	,	0x00DB	,	0x0000),
DEADTRANS(	asciicircum	,	0x0057	,	0x0174	,	0x0000),
DEADTRANS(	asciicircum	,	0x0059	,	0x0176	,	0x0000),
DEADTRANS(	asciicircum	,	0x005A	,	0x1E90	,	0x0000),
DEADTRANS(	asciicircum	,	0x0061	,	0x00E2	,	0x0000),
DEADTRANS(	asciicircum	,	0x0063	,	0x0109	,	0x0000),
DEADTRANS(	asciicircum	,	0x0065	,	0x00EA	,	0x0000),
DEADTRANS(	asciicircum	,	0x0067	,	0x011D	,	0x0000),
DEADTRANS(	asciicircum	,	0x0068	,	0x0125	,	0x0000),
DEADTRANS(	asciicircum	,	0x0069	,	0x00EE	,	0x0000),
DEADTRANS(	asciicircum	,	0x006A	,	0x0135	,	0x0000),
DEADTRANS(	asciicircum	,	0x006F	,	0x00F4	,	0x0000),
DEADTRANS(	asciicircum	,	0x0073	,	0x015D	,	0x0000),
DEADTRANS(	asciicircum	,	0x0075	,	0x00FB	,	0x0000),
DEADTRANS(	asciicircum	,	0x0077	,	0x0175	,	0x0000),
DEADTRANS(	asciicircum	,	0x0079	,	0x0177	,	0x0000),
DEADTRANS(	asciicircum	,	0x007A	,	0x1E91	,	0x0000),
DEADTRANS(	asciicircum	,	0x1EA0	,	0x1EAC	,	0x0000),
DEADTRANS(	asciicircum	,	0x1EA1	,	0x1EAD	,	0x0000),
DEADTRANS(	asciicircum	,	0x1EB8	,	0x1EC6	,	0x0000),
DEADTRANS(	asciicircum	,	0x1EB9	,	0x1EC7	,	0x0000),
DEADTRANS(	asciicircum	,	0x1ECC	,	0x1ED8	,	0x0000),
DEADTRANS(	asciicircum	,	0x1ECD	,	0x1ED9	,	0x0000),
DEADTRANS(	asciicircum	,	minussign	,	0x207B	,	0x0000),
DEADTRANS(	asciicircum	,	0x4E00	,	0x3192	,	0x0000),
DEADTRANS(	asciicircum	,	0x4E01	,	0x319C	,	0x0000),
DEADTRANS(	asciicircum	,	0x4E09	,	0x3194	,	0x0000),
DEADTRANS(	asciicircum	,	0x4E0A	,	0x3196	,	0x0000),
DEADTRANS(	asciicircum	,	0x4E0B	,	0x3198	,	0x0000),
DEADTRANS(	asciicircum	,	0x4E19	,	0x319B	,	0x0000),
DEADTRANS(	asciicircum	,	0x4E2D	,	0x3197	,	0x0000),
DEADTRANS(	asciicircum	,	0x4E59	,	0x319A	,	0x0000),
DEADTRANS(	asciicircum	,	0x4E8C	,	0x3193	,	0x0000),
DEADTRANS(	asciicircum	,	0x4EBA	,	0x319F	,	0x0000),
DEADTRANS(	asciicircum	,	0x56DB	,	0x3195	,	0x0000),
DEADTRANS(	asciicircum	,	0x5730	,	0x319E	,	0x0000),
DEADTRANS(	asciicircum	,	0x5929	,	0x319D	,	0x0000),
DEADTRANS(	asciicircum	,	0x7532	,	0x3199	,	0x0000),
DEADTRANS(	0x005F	,	0x0028	,	0x208D	,	0x0000),
DEADTRANS(	0x005F	,	0x0028	,	0x208D	,	0x0000),
DEADTRANS(	0x005F	,	0x0029	,	0x208E	,	0x0000),
DEADTRANS(	0x005F	,	0x0029	,	0x208E	,	0x0000),
DEADTRANS(	0x005F	,	0x002B	,	0x208A	,	0x0000),
DEADTRANS(	0x005F	,	0x002B	,	0x208A	,	0x0000),
DEADTRANS(	0x005F	,	0x0030	,	zerosubscript	,	0x0000),
DEADTRANS(	0x005F	,	0x0030	,	zerosubscript	,	0x0000),
DEADTRANS(	0x005F	,	0x0031	,	onesubscript	,	0x0000),
DEADTRANS(	0x005F	,	0x0031	,	onesubscript	,	0x0000),
DEADTRANS(	0x005F	,	0x0032	,	twosubscript	,	0x0000),
DEADTRANS(	0x005F	,	0x0032	,	twosubscript	,	0x0000),
DEADTRANS(	0x005F	,	0x0033	,	threesubscript	,	0x0000),
DEADTRANS(	0x005F	,	0x0033	,	threesubscript	,	0x0000),
DEADTRANS(	0x005F	,	0x0034	,	0x2084	,	0x0000),
DEADTRANS(	0x005F	,	0x0034	,	0x2084	,	0x0000),
DEADTRANS(	0x005F	,	0x0035	,	0x2085	,	0x0000),
DEADTRANS(	0x005F	,	0x0035	,	0x2085	,	0x0000),
DEADTRANS(	0x005F	,	0x0036	,	0x2086	,	0x0000),
DEADTRANS(	0x005F	,	0x0036	,	0x2086	,	0x0000),
DEADTRANS(	0x005F	,	0x0037	,	0x2087	,	0x0000),
DEADTRANS(	0x005F	,	0x0037	,	0x2087	,	0x0000),
DEADTRANS(	0x005F	,	0x0038	,	0x2088	,	0x0000),
DEADTRANS(	0x005F	,	0x0038	,	0x2088	,	0x0000),
DEADTRANS(	0x005F	,	0x0039	,	0x2089	,	0x0000),
DEADTRANS(	0x005F	,	0x0039	,	0x2089	,	0x0000),
DEADTRANS(	0x005F	,	0x003D	,	0x208C	,	0x0000),
DEADTRANS(	0x005F	,	0x003D	,	0x208C	,	0x0000),
DEADTRANS(	0x005F	,	0x0041	,	0x0100	,	0x0000),
DEADTRANS(	0x005F	,	0x0045	,	0x0112	,	0x0000),
DEADTRANS(	0x005F	,	0x0047	,	0x1E20	,	0x0000),
DEADTRANS(	0x005F	,	0x0049	,	0x012A	,	0x0000),
DEADTRANS(	0x005F	,	0x004F	,	0x014C	,	0x0000),
DEADTRANS(	0x005F	,	0x0055	,	0x016A	,	0x0000),
DEADTRANS(	0x005F	,	0x0059	,	0x0232	,	0x0000),
DEADTRANS(	0x005F	,	0x0061	,	0x0101	,	0x0000),
DEADTRANS(	0x005F	,	0x0065	,	0x0113	,	0x0000),
DEADTRANS(	0x005F	,	0x0067	,	0x1E21	,	0x0000),
DEADTRANS(	0x005F	,	0x0069	,	0x012B	,	0x0000),
DEADTRANS(	0x005F	,	0x006F	,	0x014D	,	0x0000),
DEADTRANS(	0x005F	,	0x0075	,	0x016B	,	0x0000),
DEADTRANS(	0x005F	,	0x0079	,	0x0233	,	0x0000),
DEADTRANS(	0x005F	,	Adiaeresis	,	0x01DE	,	0x0000),
DEADTRANS(	0x005F	,	0x00C6	,	0x01E2	,	0x0000),
DEADTRANS(	0x005F	,	0x00D5	,	0x022C	,	0x0000),
DEADTRANS(	0x005F	,	Odiaeresis	,	0x022A	,	0x0000),
DEADTRANS(	0x005F	,	Udiaeresis	,	0x01D5	,	0x0000),
DEADTRANS(	0x005F	,	adiaeresis	,	0x01DF	,	0x0000),
DEADTRANS(	0x005F	,	0x00E6	,	0x01E3	,	0x0000),
DEADTRANS(	0x005F	,	0x00F5	,	0x022D	,	0x0000),
DEADTRANS(	0x005F	,	0x00F6	,	0x022B	,	0x0000),
DEADTRANS(	0x005F	,	udiaeresis	,	0x01D6	,	0x0000),
DEADTRANS(	0x005F	,	0x01EA	,	0x01EC	,	0x0000),
DEADTRANS(	0x005F	,	0x01EB	,	0x01ED	,	0x0000),
DEADTRANS(	0x005F	,	0x0226	,	0x01E0	,	0x0000),
DEADTRANS(	0x005F	,	0x0227	,	0x01E1	,	0x0000),
DEADTRANS(	0x005F	,	0x022E	,	0x0230	,	0x0000),
DEADTRANS(	0x005F	,	0x022F	,	0x0231	,	0x0000),
DEADTRANS(	0x005F	,	0x0391	,	0x1FB9	,	0x0000),
DEADTRANS(	0x005F	,	0x0399	,	0x1FD9	,	0x0000),
DEADTRANS(	0x005F	,	0x03A5	,	0x1FE9	,	0x0000),
DEADTRANS(	0x005F	,	Greek_alpha	,	0x1FB1	,	0x0000),
DEADTRANS(	0x005F	,	Greek_iota	,	0x1FD1	,	0x0000),
DEADTRANS(	0x005F	,	Greek_upsilon	,	0x1FE1	,	0x0000),
DEADTRANS(	0x005F	,	0x0418	,	0x04E2	,	0x0000),
DEADTRANS(	0x005F	,	0x0423	,	0x04EE	,	0x0000),
DEADTRANS(	0x005F	,	0x0438	,	0x04E3	,	0x0000),
DEADTRANS(	0x005F	,	0x0443	,	0x04EF	,	0x0000),
DEADTRANS(	0x005F	,	0x1E36	,	0x1E38	,	0x0000),
DEADTRANS(	0x005F	,	0x1E37	,	0x1E39	,	0x0000),
DEADTRANS(	0x005F	,	0x1E5A	,	0x1E5C	,	0x0000),
DEADTRANS(	0x005F	,	0x1E5B	,	0x1E5D	,	0x0000),
DEADTRANS(	0x005F	,	minussign	,	0x208B	,	0x0000),
DEADTRANS(	0x005F	,	minussign	,	0x208B	,	0x0000),
DEADTRANS(	grave	,	0x0020	,	grave	,	0x0000),
DEADTRANS(	grave	,	0x0041	,	0x00C0	,	0x0000),
DEADTRANS(	grave	,	0x0045	,	0x00C8	,	0x0000),
DEADTRANS(	grave	,	0x0049	,	0x00CC	,	0x0000),
DEADTRANS(	grave	,	0x004E	,	0x01F8	,	0x0000),
DEADTRANS(	grave	,	0x004F	,	0x00D2	,	0x0000),
DEADTRANS(	grave	,	0x0055	,	0x00D9	,	0x0000),
DEADTRANS(	grave	,	0x0057	,	0x1E80	,	0x0000),
DEADTRANS(	grave	,	0x0059	,	0x1EF2	,	0x0000),
DEADTRANS(	grave	,	0x0061	,	0x00E0	,	0x0000),
DEADTRANS(	grave	,	0x0065	,	0x00E8	,	0x0000),
DEADTRANS(	grave	,	0x0069	,	0x00EC	,	0x0000),
DEADTRANS(	grave	,	0x006E	,	0x01F9	,	0x0000),
DEADTRANS(	grave	,	0x006F	,	0x00F2	,	0x0000),
DEADTRANS(	grave	,	0x0075	,	0x00F9	,	0x0000),
DEADTRANS(	grave	,	0x0077	,	0x1E81	,	0x0000),
DEADTRANS(	grave	,	0x0079	,	0x1EF3	,	0x0000),
DEADTRANS(	grave	,	0x00C2	,	0x1EA6	,	0x0000),
DEADTRANS(	grave	,	0x00CA	,	0x1EC0	,	0x0000),
DEADTRANS(	grave	,	0x00D4	,	0x1ED2	,	0x0000),
DEADTRANS(	grave	,	Udiaeresis	,	0x01DB	,	0x0000),
DEADTRANS(	grave	,	0x00E2	,	0x1EA7	,	0x0000),
DEADTRANS(	grave	,	0x00EA	,	0x1EC1	,	0x0000),
DEADTRANS(	grave	,	0x00F4	,	0x1ED3	,	0x0000),
DEADTRANS(	grave	,	udiaeresis	,	0x01DC	,	0x0000),
DEADTRANS(	grave	,	0x0102	,	0x1EB0	,	0x0000),
DEADTRANS(	grave	,	0x0103	,	0x1EB1	,	0x0000),
DEADTRANS(	grave	,	0x0112	,	0x1E14	,	0x0000),
DEADTRANS(	grave	,	0x0113	,	0x1E15	,	0x0000),
DEADTRANS(	grave	,	0x014C	,	0x1E50	,	0x0000),
DEADTRANS(	grave	,	0x014D	,	0x1E51	,	0x0000),
DEADTRANS(	grave	,	0x01A0	,	0x1EDC	,	0x0000),
DEADTRANS(	grave	,	0x01A1	,	0x1EDD	,	0x0000),
DEADTRANS(	grave	,	0x01AF	,	0x1EEA	,	0x0000),
DEADTRANS(	grave	,	0x01B0	,	0x1EEB	,	0x0000),
DEADTRANS(	grave	,	0x0391	,	0x1FBA	,	0x0000),
DEADTRANS(	grave	,	0x0395	,	0x1FC8	,	0x0000),
DEADTRANS(	grave	,	0x0397	,	0x1FCA	,	0x0000),
DEADTRANS(	grave	,	0x0399	,	0x1FDA	,	0x0000),
DEADTRANS(	grave	,	0x039F	,	0x1FF8	,	0x0000),
DEADTRANS(	grave	,	0x03A5	,	0x1FEA	,	0x0000),
DEADTRANS(	grave	,	Greek_OMEGA	,	0x1FFA	,	0x0000),
DEADTRANS(	grave	,	Greek_alpha	,	0x1F70	,	0x0000),
DEADTRANS(	grave	,	Greek_epsilon	,	0x1F72	,	0x0000),
DEADTRANS(	grave	,	Greek_eta	,	0x1F74	,	0x0000),
DEADTRANS(	grave	,	Greek_iota	,	0x1F76	,	0x0000),
DEADTRANS(	grave	,	Greek_omicron	,	0x1F78	,	0x0000),
DEADTRANS(	grave	,	Greek_upsilon	,	0x1F7A	,	0x0000),
DEADTRANS(	grave	,	Greek_omega	,	0x1F7C	,	0x0000),
DEADTRANS(	grave	,	0x03CA	,	0x1FD2	,	0x0000),
DEADTRANS(	grave	,	0x03CB	,	0x1FE2	,	0x0000),
DEADTRANS(	grave	,	0x0415	,	0x0400	,	0x0000),
DEADTRANS(	grave	,	0x0418	,	0x040D	,	0x0000),
DEADTRANS(	grave	,	0x0435	,	0x0450	,	0x0000),
DEADTRANS(	grave	,	0x0438	,	0x045D	,	0x0000),
DEADTRANS(	grave	,	0x1F00	,	0x1F02	,	0x0000),
DEADTRANS(	grave	,	0x1F01	,	0x1F03	,	0x0000),
DEADTRANS(	grave	,	0x1F08	,	0x1F0A	,	0x0000),
DEADTRANS(	grave	,	0x1F09	,	0x1F0B	,	0x0000),
DEADTRANS(	grave	,	0x1F10	,	0x1F12	,	0x0000),
DEADTRANS(	grave	,	0x1F11	,	0x1F13	,	0x0000),
DEADTRANS(	grave	,	0x1F18	,	0x1F1A	,	0x0000),
DEADTRANS(	grave	,	0x1F19	,	0x1F1B	,	0x0000),
DEADTRANS(	grave	,	0x1F20	,	0x1F22	,	0x0000),
DEADTRANS(	grave	,	0x1F21	,	0x1F23	,	0x0000),
DEADTRANS(	grave	,	0x1F28	,	0x1F2A	,	0x0000),
DEADTRANS(	grave	,	0x1F29	,	0x1F2B	,	0x0000),
DEADTRANS(	grave	,	0x1F30	,	0x1F32	,	0x0000),
DEADTRANS(	grave	,	0x1F31	,	0x1F33	,	0x0000),
DEADTRANS(	grave	,	0x1F38	,	0x1F3A	,	0x0000),
DEADTRANS(	grave	,	0x1F39	,	0x1F3B	,	0x0000),
DEADTRANS(	grave	,	0x1F40	,	0x1F42	,	0x0000),
DEADTRANS(	grave	,	0x1F41	,	0x1F43	,	0x0000),
DEADTRANS(	grave	,	0x1F48	,	0x1F4A	,	0x0000),
DEADTRANS(	grave	,	0x1F49	,	0x1F4B	,	0x0000),
DEADTRANS(	grave	,	0x1F50	,	0x1F52	,	0x0000),
DEADTRANS(	grave	,	0x1F51	,	0x1F53	,	0x0000),
DEADTRANS(	grave	,	0x1F59	,	0x1F5B	,	0x0000),
DEADTRANS(	grave	,	0x1F60	,	0x1F62	,	0x0000),
DEADTRANS(	grave	,	0x1F61	,	0x1F63	,	0x0000),
DEADTRANS(	grave	,	0x1F68	,	0x1F6A	,	0x0000),
DEADTRANS(	grave	,	0x1F69	,	0x1F6B	,	0x0000),
DEADTRANS(	0x0061	,	0x0065	,	0x00E6	,	0x0000),
DEADTRANS(	0x0062	,	0x0041	,	0x0102	,	0x0000),
DEADTRANS(	0x0062	,	0x0045	,	0x0114	,	0x0000),
DEADTRANS(	0x0062	,	0x0047	,	0x011E	,	0x0000),
DEADTRANS(	0x0062	,	0x0049	,	0x012C	,	0x0000),
DEADTRANS(	0x0062	,	0x004F	,	0x014E	,	0x0000),
DEADTRANS(	0x0062	,	0x0055	,	0x016C	,	0x0000),
DEADTRANS(	0x0062	,	0x0061	,	0x0103	,	0x0000),
DEADTRANS(	0x0062	,	0x0065	,	0x0115	,	0x0000),
DEADTRANS(	0x0062	,	0x0067	,	0x011F	,	0x0000),
DEADTRANS(	0x0062	,	0x0069	,	0x012D	,	0x0000),
DEADTRANS(	0x0062	,	0x006F	,	0x014F	,	0x0000),
DEADTRANS(	0x0062	,	0x0075	,	0x016D	,	0x0000),
DEADTRANS(	0x0062	,	0x0228	,	0x1E1C	,	0x0000),
DEADTRANS(	0x0062	,	0x0229	,	0x1E1D	,	0x0000),
DEADTRANS(	0x0062	,	0x0391	,	0x1FB8	,	0x0000),
DEADTRANS(	0x0062	,	0x0399	,	0x1FD8	,	0x0000),
DEADTRANS(	0x0062	,	0x03A5	,	0x1FE8	,	0x0000),
DEADTRANS(	0x0062	,	Greek_alpha	,	0x1FB0	,	0x0000),
DEADTRANS(	0x0062	,	Greek_iota	,	0x1FD0	,	0x0000),
DEADTRANS(	0x0062	,	Greek_upsilon	,	0x1FE0	,	0x0000),
DEADTRANS(	0x0062	,	0x0410	,	0x04D0	,	0x0000),
DEADTRANS(	0x0062	,	0x0415	,	0x04D6	,	0x0000),
DEADTRANS(	0x0062	,	0x0416	,	0x04C1	,	0x0000),
DEADTRANS(	0x0062	,	0x0418	,	0x0419	,	0x0000),
DEADTRANS(	0x0062	,	0x0423	,	0x040E	,	0x0000),
DEADTRANS(	0x0062	,	0x0430	,	0x04D1	,	0x0000),
DEADTRANS(	0x0062	,	0x0435	,	0x04D7	,	0x0000),
DEADTRANS(	0x0062	,	0x0436	,	0x04C2	,	0x0000),
DEADTRANS(	0x0062	,	0x0438	,	0x0439	,	0x0000),
DEADTRANS(	0x0062	,	0x0443	,	0x045E	,	0x0000),
DEADTRANS(	0x0062	,	0x1EA0	,	0x1EB6	,	0x0000),
DEADTRANS(	0x0062	,	0x1EA1	,	0x1EB7	,	0x0000),
DEADTRANS(	0x0063	,	0x002F	,	cent	,	0x0000),
DEADTRANS(	0x0063	,	0x0041	,	0x01CD	,	0x0000),
DEADTRANS(	0x0063	,	0x0043	,	0x010C	,	0x0000),
DEADTRANS(	0x0063	,	0x0044	,	0x010E	,	0x0000),
DEADTRANS(	0x0063	,	0x0045	,	0x011A	,	0x0000),
DEADTRANS(	0x0063	,	0x0047	,	0x01E6	,	0x0000),
DEADTRANS(	0x0063	,	0x0048	,	0x021E	,	0x0000),
DEADTRANS(	0x0063	,	0x0049	,	0x01CF	,	0x0000),
DEADTRANS(	0x0063	,	0x004B	,	0x01E8	,	0x0000),
DEADTRANS(	0x0063	,	0x004C	,	0x013D	,	0x0000),
DEADTRANS(	0x0063	,	0x004E	,	0x0147	,	0x0000),
DEADTRANS(	0x0063	,	0x004F	,	0x01D1	,	0x0000),
DEADTRANS(	0x0063	,	0x0052	,	0x0158	,	0x0000),
DEADTRANS(	0x0063	,	0x0053	,	0x0160	,	0x0000),
DEADTRANS(	0x0063	,	0x0054	,	0x0164	,	0x0000),
DEADTRANS(	0x0063	,	0x0055	,	0x01D3	,	0x0000),
DEADTRANS(	0x0063	,	0x005A	,	0x017D	,	0x0000),
DEADTRANS(	0x0063	,	0x0061	,	0x01CE	,	0x0000),
DEADTRANS(	0x0063	,	0x0063	,	0x010D	,	0x0000),
DEADTRANS(	0x0063	,	0x0064	,	0x010F	,	0x0000),
DEADTRANS(	0x0063	,	0x0065	,	0x011B	,	0x0000),
DEADTRANS(	0x0063	,	0x0067	,	0x01E7	,	0x0000),
DEADTRANS(	0x0063	,	0x0068	,	0x021F	,	0x0000),
DEADTRANS(	0x0063	,	0x0069	,	0x01D0	,	0x0000),
DEADTRANS(	0x0063	,	0x006A	,	0x01F0	,	0x0000),
DEADTRANS(	0x0063	,	0x006B	,	0x01E9	,	0x0000),
DEADTRANS(	0x0063	,	0x006C	,	0x013E	,	0x0000),
DEADTRANS(	0x0063	,	0x006E	,	0x0148	,	0x0000),
DEADTRANS(	0x0063	,	0x006F	,	0x01D2	,	0x0000),
DEADTRANS(	0x0063	,	0x0072	,	0x0159	,	0x0000),
DEADTRANS(	0x0063	,	0x0073	,	0x0161	,	0x0000),
DEADTRANS(	0x0063	,	0x0074	,	0x0165	,	0x0000),
DEADTRANS(	0x0063	,	0x0075	,	0x01D4	,	0x0000),
DEADTRANS(	0x0063	,	0x007A	,	0x017E	,	0x0000),
DEADTRANS(	0x0063	,	0x007C	,	cent	,	0x0000),
DEADTRANS(	0x0063	,	Udiaeresis	,	0x01D9	,	0x0000),
DEADTRANS(	0x0063	,	udiaeresis	,	0x01DA	,	0x0000),
DEADTRANS(	0x0063	,	0x01B7	,	0x01EE	,	0x0000),
DEADTRANS(	0x0063	,	0x0292	,	0x01EF	,	0x0000),
DEADTRANS(	0x0064	,	0x002D	,	0x20AB	,	0x0000),
DEADTRANS(	0x0064	,	0x0068	,	0x00F0	,	0x0000),
DEADTRANS(	0x0065	,	0x0065	,	0x0259	,	0x0000),
DEADTRANS(	0x0066	,	0x0053	,	slong	,	0x0000),
DEADTRANS(	0x0066	,	0x0073	,	slong	,	0x0000),
DEADTRANS(	0x0069	,	0x002E	,	0x0131	,	0x0000),
DEADTRANS(	0x006B	,	0x006B	,	0x0138	,	0x0000),
DEADTRANS(	0x006C	,	0x0076	,	0x007C	,	0x0000),
DEADTRANS(	0x006D	,	0x002F	,	0x20A5	,	0x0000),
DEADTRANS(	0x006D	,	0x0075	,	mu	,	0x0000),
DEADTRANS(	0x006E	,	0x0067	,	0x014B	,	0x0000),
DEADTRANS(	0x006F	,	0x0041	,	0x00C5	,	0x0000),
DEADTRANS(	0x006F	,	0x0043	,	0x00A9	,	0x0000),
DEADTRANS(	0x006F	,	0x0052	,	0x00AE	,	0x0000),
DEADTRANS(	0x006F	,	0x0055	,	0x016E	,	0x0000),
DEADTRANS(	0x006F	,	0x0061	,	0x00E5	,	0x0000),
DEADTRANS(	0x006F	,	0x0063	,	0x00A9	,	0x0000),
DEADTRANS(	0x006F	,	0x0065	,	0x0153	,	0x0000),
DEADTRANS(	0x006F	,	0x006F	,	degree	,	0x0000),
DEADTRANS(	0x006F	,	0x0072	,	0x00AE	,	0x0000),
DEADTRANS(	0x006F	,	0x0073	,	section	,	0x0000),
DEADTRANS(	0x006F	,	0x0075	,	0x016F	,	0x0000),
DEADTRANS(	0x006F	,	0x0077	,	0x1E98	,	0x0000),
DEADTRANS(	0x006F	,	0x0078	,	currency	,	0x0000),
DEADTRANS(	0x006F	,	0x0079	,	0x1E99	,	0x0000),
DEADTRANS(	0x0070	,	0x0021	,	0x00B6	,	0x0000),
DEADTRANS(	0x0073	,	0x006F	,	section	,	0x0000),
DEADTRANS(	0x0073	,	0x0073	,	ssharp	,	0x0000),
DEADTRANS(	0x0074	,	0x0068	,	0x00FE	,	0x0000),
DEADTRANS(	0x0076	,	0x006C	,	0x007C	,	0x0000),
DEADTRANS(	0x0078	,	0x006F	,	currency	,	0x0000),
DEADTRANS(	0x0078	,	0x0078	,	multiply	,	0x0000),
DEADTRANS(	0x007C	,	0x0043	,	cent	,	0x0000),
DEADTRANS(	0x007C	,	0x0063	,	cent	,	0x0000),
DEADTRANS(	0x007E	,	0x0020	,	0x007E	,	0x0000),
DEADTRANS(	0x007E	,	0x0041	,	0x00C3	,	0x0000),
DEADTRANS(	0x007E	,	0x0045	,	0x1EBC	,	0x0000),
DEADTRANS(	0x007E	,	0x0049	,	0x0128	,	0x0000),
DEADTRANS(	0x007E	,	0x004E	,	0x00D1	,	0x0000),
DEADTRANS(	0x007E	,	0x004F	,	0x00D5	,	0x0000),
DEADTRANS(	0x007E	,	0x0055	,	0x0168	,	0x0000),
DEADTRANS(	0x007E	,	0x0056	,	0x1E7C	,	0x0000),
DEADTRANS(	0x007E	,	0x0059	,	0x1EF8	,	0x0000),
DEADTRANS(	0x007E	,	0x0061	,	0x00E3	,	0x0000),
DEADTRANS(	0x007E	,	0x0065	,	0x1EBD	,	0x0000),
DEADTRANS(	0x007E	,	0x0069	,	0x0129	,	0x0000),
DEADTRANS(	0x007E	,	0x006E	,	0x00F1	,	0x0000),
DEADTRANS(	0x007E	,	0x006F	,	0x00F5	,	0x0000),
DEADTRANS(	0x007E	,	0x0075	,	0x0169	,	0x0000),
DEADTRANS(	0x007E	,	0x0076	,	0x1E7D	,	0x0000),
DEADTRANS(	0x007E	,	0x0079	,	0x1EF9	,	0x0000),
DEADTRANS(	0x007E	,	0x00C2	,	0x1EAA	,	0x0000),
DEADTRANS(	0x007E	,	0x00CA	,	0x1EC4	,	0x0000),
DEADTRANS(	0x007E	,	0x00D4	,	0x1ED6	,	0x0000),
DEADTRANS(	0x007E	,	0x00E2	,	0x1EAB	,	0x0000),
DEADTRANS(	0x007E	,	0x00EA	,	0x1EC5	,	0x0000),
DEADTRANS(	0x007E	,	0x00F4	,	0x1ED7	,	0x0000),
DEADTRANS(	0x007E	,	0x0102	,	0x1EB4	,	0x0000),
DEADTRANS(	0x007E	,	0x0103	,	0x1EB5	,	0x0000),
DEADTRANS(	0x007E	,	0x01A0	,	0x1EE0	,	0x0000),
DEADTRANS(	0x007E	,	0x01A1	,	0x1EE1	,	0x0000),
DEADTRANS(	0x007E	,	0x01AF	,	0x1EEE	,	0x0000),
DEADTRANS(	0x007E	,	0x01B0	,	0x1EEF	,	0x0000),
DEADTRANS(	0x007E	,	Greek_alpha	,	0x1FB6	,	0x0000),
DEADTRANS(	0x007E	,	Greek_eta	,	0x1FC6	,	0x0000),
DEADTRANS(	0x007E	,	Greek_iota	,	0x1FD6	,	0x0000),
DEADTRANS(	0x007E	,	Greek_upsilon	,	0x1FE6	,	0x0000),
DEADTRANS(	0x007E	,	Greek_omega	,	0x1FF6	,	0x0000),
DEADTRANS(	0x007E	,	0x03CA	,	0x1FD7	,	0x0000),
DEADTRANS(	0x007E	,	0x03CB	,	0x1FE7	,	0x0000),
DEADTRANS(	0x007E	,	0x1F00	,	0x1F06	,	0x0000),
DEADTRANS(	0x007E	,	0x1F01	,	0x1F07	,	0x0000),
DEADTRANS(	0x007E	,	0x1F08	,	0x1F0E	,	0x0000),
DEADTRANS(	0x007E	,	0x1F09	,	0x1F0F	,	0x0000),
DEADTRANS(	0x007E	,	0x1F20	,	0x1F26	,	0x0000),
DEADTRANS(	0x007E	,	0x1F21	,	0x1F27	,	0x0000),
DEADTRANS(	0x007E	,	0x1F28	,	0x1F2E	,	0x0000),
DEADTRANS(	0x007E	,	0x1F29	,	0x1F2F	,	0x0000),
DEADTRANS(	0x007E	,	0x1F30	,	0x1F36	,	0x0000),
DEADTRANS(	0x007E	,	0x1F31	,	0x1F37	,	0x0000),
DEADTRANS(	0x007E	,	0x1F38	,	0x1F3E	,	0x0000),
DEADTRANS(	0x007E	,	0x1F39	,	0x1F3F	,	0x0000),
DEADTRANS(	0x007E	,	0x1F50	,	0x1F56	,	0x0000),
DEADTRANS(	0x007E	,	0x1F51	,	0x1F57	,	0x0000),
DEADTRANS(	0x007E	,	0x1F59	,	0x1F5F	,	0x0000),
DEADTRANS(	0x007E	,	0x1F60	,	0x1F66	,	0x0000),
DEADTRANS(	0x007E	,	0x1F61	,	0x1F67	,	0x0000),
DEADTRANS(	0x007E	,	0x1F68	,	0x1F6E	,	0x0000),
DEADTRANS(	0x007E	,	0x1F69	,	0x1F6F	,	0x0000),
DEADTRANS(	diaeresis	,	apostrophe	,	0x0385	,	0x0000),
DEADTRANS(	diaeresis	,	grave	,	0x1FED	,	0x0000),
DEADTRANS(	diaeresis	,	grave	,	0x1FED	,	0x0000),
DEADTRANS(	diaeresis	,	0x007E	,	0x1FC1	,	0x0000),
DEADTRANS(	diaeresis	,	acute	,	0x0385	,	0x0000),
DEADTRANS(	diaeresis	,	acute	,	0x0385	,	0x0000),
DEADTRANS(	diaeresis	,	0x00f1	,	0x1FC1	,	0x0000),
DEADTRANS(	diaeresis	,	0x0342	,	0x1FC1	,	0x0000),
DEADTRANS(	macron	,	0x0041	,	0x0100	,	0x0000),
DEADTRANS(	macron	,	0x0045	,	0x0112	,	0x0000),
DEADTRANS(	macron	,	0x0047	,	0x1E20	,	0x0000),
DEADTRANS(	macron	,	0x0049	,	0x012A	,	0x0000),
DEADTRANS(	macron	,	0x004F	,	0x014C	,	0x0000),
DEADTRANS(	macron	,	0x0055	,	0x016A	,	0x0000),
DEADTRANS(	macron	,	0x0059	,	0x0232	,	0x0000),
DEADTRANS(	macron	,	0x0061	,	0x0101	,	0x0000),
DEADTRANS(	macron	,	0x0065	,	0x0113	,	0x0000),
DEADTRANS(	macron	,	0x0067	,	0x1E21	,	0x0000),
DEADTRANS(	macron	,	0x0069	,	0x012B	,	0x0000),
DEADTRANS(	macron	,	0x006F	,	0x014D	,	0x0000),
DEADTRANS(	macron	,	0x0075	,	0x016B	,	0x0000),
DEADTRANS(	macron	,	0x0079	,	0x0233	,	0x0000),
DEADTRANS(	macron	,	Adiaeresis	,	0x01DE	,	0x0000),
DEADTRANS(	macron	,	0x00C6	,	0x01E2	,	0x0000),
DEADTRANS(	macron	,	0x00D5	,	0x022C	,	0x0000),
DEADTRANS(	macron	,	Odiaeresis	,	0x022A	,	0x0000),
DEADTRANS(	macron	,	Udiaeresis	,	0x01D5	,	0x0000),
DEADTRANS(	macron	,	adiaeresis	,	0x01DF	,	0x0000),
DEADTRANS(	macron	,	0x00E6	,	0x01E3	,	0x0000),
DEADTRANS(	macron	,	0x00F5	,	0x022D	,	0x0000),
DEADTRANS(	macron	,	0x00F6	,	0x022B	,	0x0000),
DEADTRANS(	macron	,	udiaeresis	,	0x01D6	,	0x0000),
DEADTRANS(	macron	,	0x01EA	,	0x01EC	,	0x0000),
DEADTRANS(	macron	,	0x01EB	,	0x01ED	,	0x0000),
DEADTRANS(	macron	,	0x0226	,	0x01E0	,	0x0000),
DEADTRANS(	macron	,	0x0227	,	0x01E1	,	0x0000),
DEADTRANS(	macron	,	0x022E	,	0x0230	,	0x0000),
DEADTRANS(	macron	,	0x022F	,	0x0231	,	0x0000),
DEADTRANS(	macron	,	0x0391	,	0x1FB9	,	0x0000),
DEADTRANS(	macron	,	0x0399	,	0x1FD9	,	0x0000),
DEADTRANS(	macron	,	0x03A5	,	0x1FE9	,	0x0000),
DEADTRANS(	macron	,	Greek_alpha	,	0x1FB1	,	0x0000),
DEADTRANS(	macron	,	Greek_iota	,	0x1FD1	,	0x0000),
DEADTRANS(	macron	,	Greek_upsilon	,	0x1FE1	,	0x0000),
DEADTRANS(	macron	,	0x0418	,	0x04E2	,	0x0000),
DEADTRANS(	macron	,	0x0423	,	0x04EE	,	0x0000),
DEADTRANS(	macron	,	0x0438	,	0x04E3	,	0x0000),
DEADTRANS(	macron	,	0x0443	,	0x04EF	,	0x0000),
DEADTRANS(	macron	,	0x1E36	,	0x1E38	,	0x0000),
DEADTRANS(	macron	,	0x1E37	,	0x1E39	,	0x0000),
DEADTRANS(	macron	,	0x1E5A	,	0x1E5C	,	0x0000),
DEADTRANS(	macron	,	0x1E5B	,	0x1E5D	,	0x0000),
DEADTRANS(	acute	,	0x0041	,	0x00C1	,	0x0000),
DEADTRANS(	acute	,	0x0043	,	0x0106	,	0x0000),
DEADTRANS(	acute	,	0x0045	,	0x00C9	,	0x0000),
DEADTRANS(	acute	,	0x0047	,	0x01F4	,	0x0000),
DEADTRANS(	acute	,	0x0049	,	0x00CD	,	0x0000),
DEADTRANS(	acute	,	0x004B	,	0x1E30	,	0x0000),
DEADTRANS(	acute	,	0x004C	,	0x0139	,	0x0000),
DEADTRANS(	acute	,	0x004D	,	0x1E3E	,	0x0000),
DEADTRANS(	acute	,	0x004E	,	0x0143	,	0x0000),
DEADTRANS(	acute	,	0x004F	,	0x00D3	,	0x0000),
DEADTRANS(	acute	,	0x0050	,	0x1E54	,	0x0000),
DEADTRANS(	acute	,	0x0052	,	0x0154	,	0x0000),
DEADTRANS(	acute	,	0x0053	,	0x015A	,	0x0000),
DEADTRANS(	acute	,	0x0055	,	0x00DA	,	0x0000),
DEADTRANS(	acute	,	0x0057	,	0x1E82	,	0x0000),
DEADTRANS(	acute	,	0x0059	,	0x00DD	,	0x0000),
DEADTRANS(	acute	,	0x005A	,	0x0179	,	0x0000),
DEADTRANS(	acute	,	0x0061	,	0x00E1	,	0x0000),
DEADTRANS(	acute	,	0x0063	,	0x0107	,	0x0000),
DEADTRANS(	acute	,	0x0065	,	0x00E9	,	0x0000),
DEADTRANS(	acute	,	0x0067	,	0x01F5	,	0x0000),
DEADTRANS(	acute	,	0x0069	,	0x00ED	,	0x0000),
DEADTRANS(	acute	,	0x006B	,	0x1E31	,	0x0000),
DEADTRANS(	acute	,	0x006C	,	0x013A	,	0x0000),
DEADTRANS(	acute	,	0x006D	,	0x1E3F	,	0x0000),
DEADTRANS(	acute	,	0x006E	,	0x0144	,	0x0000),
DEADTRANS(	acute	,	0x006F	,	0x00F3	,	0x0000),
DEADTRANS(	acute	,	0x0070	,	0x1E55	,	0x0000),
DEADTRANS(	acute	,	0x0072	,	0x0155	,	0x0000),
DEADTRANS(	acute	,	0x0073	,	0x015B	,	0x0000),
DEADTRANS(	acute	,	0x0075	,	0x00FA	,	0x0000),
DEADTRANS(	acute	,	0x0077	,	0x1E83	,	0x0000),
DEADTRANS(	acute	,	0x0079	,	0x00FD	,	0x0000),
DEADTRANS(	acute	,	0x007A	,	0x017A	,	0x0000),
DEADTRANS(	acute	,	0x00C2	,	0x1EA4	,	0x0000),
DEADTRANS(	acute	,	0x00C5	,	0x01FA	,	0x0000),
DEADTRANS(	acute	,	0x00C6	,	0x01FC	,	0x0000),
DEADTRANS(	acute	,	0x00C7	,	0x1E08	,	0x0000),
DEADTRANS(	acute	,	0x00CA	,	0x1EBE	,	0x0000),
DEADTRANS(	acute	,	0x00CF	,	0x1E2E	,	0x0000),
DEADTRANS(	acute	,	0x00D4	,	0x1ED0	,	0x0000),
DEADTRANS(	acute	,	0x00D5	,	0x1E4C	,	0x0000),
DEADTRANS(	acute	,	0x00D8	,	0x01FE	,	0x0000),
DEADTRANS(	acute	,	Udiaeresis	,	0x01D7	,	0x0000),
DEADTRANS(	acute	,	0x00E2	,	0x1EA5	,	0x0000),
DEADTRANS(	acute	,	0x00E5	,	0x01FB	,	0x0000),
DEADTRANS(	acute	,	0x00E6	,	0x01FD	,	0x0000),
DEADTRANS(	acute	,	0x00E7	,	0x1E09	,	0x0000),
DEADTRANS(	acute	,	0x00EA	,	0x1EBF	,	0x0000),
DEADTRANS(	acute	,	0x00EF	,	0x1E2F	,	0x0000),
DEADTRANS(	acute	,	0x00F4	,	0x1ED1	,	0x0000),
DEADTRANS(	acute	,	0x00F5	,	0x1E4D	,	0x0000),
DEADTRANS(	acute	,	0x00F8	,	0x01FF	,	0x0000),
DEADTRANS(	acute	,	udiaeresis	,	0x01D8	,	0x0000),
DEADTRANS(	acute	,	0x0102	,	0x1EAE	,	0x0000),
DEADTRANS(	acute	,	0x0103	,	0x1EAF	,	0x0000),
DEADTRANS(	acute	,	0x0112	,	0x1E16	,	0x0000),
DEADTRANS(	acute	,	0x0113	,	0x1E17	,	0x0000),
DEADTRANS(	acute	,	0x014C	,	0x1E52	,	0x0000),
DEADTRANS(	acute	,	0x014D	,	0x1E53	,	0x0000),
DEADTRANS(	acute	,	0x0168	,	0x1E78	,	0x0000),
DEADTRANS(	acute	,	0x0169	,	0x1E79	,	0x0000),
DEADTRANS(	acute	,	0x01A0	,	0x1EDA	,	0x0000),
DEADTRANS(	acute	,	0x01A1	,	0x1EDB	,	0x0000),
DEADTRANS(	acute	,	0x01AF	,	0x1EE8	,	0x0000),
DEADTRANS(	acute	,	0x01B0	,	0x1EE9	,	0x0000),
DEADTRANS(	acute	,	0x0391	,	0x0386	,	0x0000),
DEADTRANS(	acute	,	0x0395	,	0x0388	,	0x0000),
DEADTRANS(	acute	,	0x0397	,	0x0389	,	0x0000),
DEADTRANS(	acute	,	0x0399	,	0x038A	,	0x0000),
DEADTRANS(	acute	,	0x039F	,	0x038C	,	0x0000),
DEADTRANS(	acute	,	0x03A5	,	0x038E	,	0x0000),
DEADTRANS(	acute	,	Greek_OMEGA	,	0x038F	,	0x0000),
DEADTRANS(	acute	,	Greek_alpha	,	0x03AC	,	0x0000),
DEADTRANS(	acute	,	Greek_epsilon	,	0x03AD	,	0x0000),
DEADTRANS(	acute	,	Greek_eta	,	0x03AE	,	0x0000),
DEADTRANS(	acute	,	Greek_iota	,	0x03AF	,	0x0000),
DEADTRANS(	acute	,	Greek_omicron	,	0x03CC	,	0x0000),
DEADTRANS(	acute	,	Greek_upsilon	,	0x03CD	,	0x0000),
DEADTRANS(	acute	,	Greek_omega	,	0x03CE	,	0x0000),
DEADTRANS(	acute	,	0x03CA	,	0x0390	,	0x0000),
DEADTRANS(	acute	,	0x03CB	,	0x03B0	,	0x0000),
DEADTRANS(	acute	,	0x03D2	,	0x03D3	,	0x0000),
DEADTRANS(	acute	,	0x0413	,	0x0403	,	0x0000),
DEADTRANS(	acute	,	0x041A	,	0x040C	,	0x0000),
DEADTRANS(	acute	,	0x0433	,	0x0453	,	0x0000),
DEADTRANS(	acute	,	0x043A	,	0x045C	,	0x0000),
DEADTRANS(	acute	,	0x1F00	,	0x1F04	,	0x0000),
DEADTRANS(	acute	,	0x1F01	,	0x1F05	,	0x0000),
DEADTRANS(	acute	,	0x1F08	,	0x1F0C	,	0x0000),
DEADTRANS(	acute	,	0x1F09	,	0x1F0D	,	0x0000),
DEADTRANS(	acute	,	0x1F10	,	0x1F14	,	0x0000),
DEADTRANS(	acute	,	0x1F11	,	0x1F15	,	0x0000),
DEADTRANS(	acute	,	0x1F18	,	0x1F1C	,	0x0000),
DEADTRANS(	acute	,	0x1F19	,	0x1F1D	,	0x0000),
DEADTRANS(	acute	,	0x1F20	,	0x1F24	,	0x0000),
DEADTRANS(	acute	,	0x1F21	,	0x1F25	,	0x0000),
DEADTRANS(	acute	,	0x1F28	,	0x1F2C	,	0x0000),
DEADTRANS(	acute	,	0x1F29	,	0x1F2D	,	0x0000),
DEADTRANS(	acute	,	0x1F30	,	0x1F34	,	0x0000),
DEADTRANS(	acute	,	0x1F31	,	0x1F35	,	0x0000),
DEADTRANS(	acute	,	0x1F38	,	0x1F3C	,	0x0000),
DEADTRANS(	acute	,	0x1F39	,	0x1F3D	,	0x0000),
DEADTRANS(	acute	,	0x1F40	,	0x1F44	,	0x0000),
DEADTRANS(	acute	,	0x1F41	,	0x1F45	,	0x0000),
DEADTRANS(	acute	,	0x1F48	,	0x1F4C	,	0x0000),
DEADTRANS(	acute	,	0x1F49	,	0x1F4D	,	0x0000),
DEADTRANS(	acute	,	0x1F50	,	0x1F54	,	0x0000),
DEADTRANS(	acute	,	0x1F51	,	0x1F55	,	0x0000),
DEADTRANS(	acute	,	0x1F59	,	0x1F5D	,	0x0000),
DEADTRANS(	acute	,	0x1F60	,	0x1F64	,	0x0000),
DEADTRANS(	acute	,	0x1F61	,	0x1F65	,	0x0000),
DEADTRANS(	acute	,	0x1F68	,	0x1F6C	,	0x0000),
DEADTRANS(	acute	,	0x1F69	,	0x1F6D	,	0x0000),
DEADTRANS(	Greek_iota	,	0x0391	,	0x1FBC	,	0x0000),
DEADTRANS(	Greek_iota	,	0x0397	,	0x1FCC	,	0x0000),
DEADTRANS(	Greek_iota	,	Greek_OMEGA	,	0x1FFC	,	0x0000),
DEADTRANS(	Greek_iota	,	0x03AC	,	0x1FB4	,	0x0000),
DEADTRANS(	Greek_iota	,	0x03AE	,	0x1FC4	,	0x0000),
DEADTRANS(	Greek_iota	,	Greek_alpha	,	0x1FB3	,	0x0000),
DEADTRANS(	Greek_iota	,	Greek_eta	,	0x1FC3	,	0x0000),
DEADTRANS(	Greek_iota	,	Greek_omega	,	0x1FF3	,	0x0000),
DEADTRANS(	Greek_iota	,	0x03CE	,	0x1FF4	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F00	,	0x1F80	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F01	,	0x1F81	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F02	,	0x1F82	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F03	,	0x1F83	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F04	,	0x1F84	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F05	,	0x1F85	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F06	,	0x1F86	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F07	,	0x1F87	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F08	,	0x1F88	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F09	,	0x1F89	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F0A	,	0x1F8A	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F0B	,	0x1F8B	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F0C	,	0x1F8C	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F0D	,	0x1F8D	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F0E	,	0x1F8E	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F0F	,	0x1F8F	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F20	,	0x1F90	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F21	,	0x1F91	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F22	,	0x1F92	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F23	,	0x1F93	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F24	,	0x1F94	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F25	,	0x1F95	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F26	,	0x1F96	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F27	,	0x1F97	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F28	,	0x1F98	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F29	,	0x1F99	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F2A	,	0x1F9A	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F2B	,	0x1F9B	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F2C	,	0x1F9C	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F2D	,	0x1F9D	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F2E	,	0x1F9E	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F2F	,	0x1F9F	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F60	,	0x1FA0	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F61	,	0x1FA1	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F62	,	0x1FA2	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F63	,	0x1FA3	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F64	,	0x1FA4	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F65	,	0x1FA5	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F66	,	0x1FA6	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F67	,	0x1FA7	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F68	,	0x1FA8	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F69	,	0x1FA9	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F6A	,	0x1FAA	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F6B	,	0x1FAB	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F6C	,	0x1FAC	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F6D	,	0x1FAD	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F6E	,	0x1FAE	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F6F	,	0x1FAF	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F70	,	0x1FB2	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F74	,	0x1FC2	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1F7C	,	0x1FF2	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1FB6	,	0x1FB7	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1FC6	,	0x1FC7	,	0x0000),
DEADTRANS(	Greek_iota	,	0x1FF6	,	0x1FF7	,	0x0000),
DEADTRANS(	0x05B4	,	0x05D9	,	0xFB1D	,	0x0000),
DEADTRANS(	0x05B7	,	0x05D0	,	0xFB2E	,	0x0000),
DEADTRANS(	0x05B7	,	0x05F2	,	0xFB1F	,	0x0000),
DEADTRANS(	0x05B8	,	0x05D0	,	0xFB2F	,	0x0000),
DEADTRANS(	0x05B9	,	0x05D5	,	0xFB4B	,	0x0000),
DEADTRANS(	0x05BC	,	0x05D0	,	0xFB30	,	0x0000),
DEADTRANS(	0x05BC	,	0x05D1	,	0xFB31	,	0x0000),
DEADTRANS(	0x05BC	,	0x05D1	,	0xFB31	,	0x0000),
DEADTRANS(	0x05BC	,	0x05D2	,	0xFB32	,	0x0000),
DEADTRANS(	0x05BC	,	0x05D2	,	0xFB32	,	0x0000),
DEADTRANS(	0x05BC	,	0x05D3	,	0xFB33	,	0x0000),
DEADTRANS(	0x05BC	,	0x05D3	,	0xFB33	,	0x0000),
DEADTRANS(	0x05BC	,	0x05D4	,	0xFB34	,	0x0000),
DEADTRANS(	0x05BC	,	0x05D5	,	0xFB35	,	0x0000),
DEADTRANS(	0x05BC	,	0x05D6	,	0xFB36	,	0x0000),
DEADTRANS(	0x05BC	,	0x05D6	,	0xFB36	,	0x0000),
DEADTRANS(	0x05BC	,	0x05D8	,	0xFB38	,	0x0000),
DEADTRANS(	0x05BC	,	0x05D8	,	0xFB38	,	0x0000),
DEADTRANS(	0x05BC	,	0x05D9	,	0xFB39	,	0x0000),
DEADTRANS(	0x05BC	,	0x05DA	,	0xFB3A	,	0x0000),
DEADTRANS(	0x05BC	,	0x05DB	,	0xFB3B	,	0x0000),
DEADTRANS(	0x05BC	,	0x05DC	,	0xFB3C	,	0x0000),
DEADTRANS(	0x05BC	,	0x05DE	,	0xFB3E	,	0x0000),
DEADTRANS(	0x05BC	,	0x05E0	,	0xFB40	,	0x0000),
DEADTRANS(	0x05BC	,	0x05E1	,	0xFB41	,	0x0000),
DEADTRANS(	0x05BC	,	0x05E1	,	0xFB41	,	0x0000),
DEADTRANS(	0x05BC	,	0x05E3	,	0xFB43	,	0x0000),
DEADTRANS(	0x05BC	,	0x05E4	,	0xFB44	,	0x0000),
DEADTRANS(	0x05BC	,	0x05E6	,	0xFB46	,	0x0000),
DEADTRANS(	0x05BC	,	0x05E6	,	0xFB46	,	0x0000),
DEADTRANS(	0x05BC	,	0x05E7	,	0xFB47	,	0x0000),
DEADTRANS(	0x05BC	,	0x05E7	,	0xFB47	,	0x0000),
DEADTRANS(	0x05BC	,	0x05E8	,	0xFB48	,	0x0000),
DEADTRANS(	0x05BC	,	0x05E9	,	0xFB49	,	0x0000),
DEADTRANS(	0x05BC	,	0x05EA	,	0xFB4A	,	0x0000),
DEADTRANS(	0x05BC	,	0x05EA	,	0xFB4A	,	0x0000),
DEADTRANS(	0x05BF	,	0x05D1	,	0xFB4C	,	0x0000),
DEADTRANS(	0x05BF	,	0x05D1	,	0xFB4C	,	0x0000),
DEADTRANS(	0x05BF	,	0x05DB	,	0xFB4D	,	0x0000),
DEADTRANS(	0x05BF	,	0x05E4	,	0xFB4E	,	0x0000),
DEADTRANS(	0x05C1	,	0x05E9	,	0xFB2A	,	0x0000),
DEADTRANS(	0x05C1	,	0xFB49	,	0xFB2C	,	0x0000),
DEADTRANS(	0x05C2	,	0x05E9	,	0xFB2B	,	0x0000),
DEADTRANS(	0x05C2	,	0xFB49	,	0xFB2D	,	0x0000),
DEADTRANS(	0x0653	,	0x0627	,	0x0622	,	0x0000),
DEADTRANS(	0x0654	,	0x0627	,	0x0623	,	0x0000),
DEADTRANS(	0x0654	,	0x0648	,	0x0624	,	0x0000),
DEADTRANS(	0x0654	,	0x064A	,	0x0626	,	0x0000),
DEADTRANS(	0x0654	,	0x06C1	,	0x06C2	,	0x0000),
DEADTRANS(	0x0654	,	0x06D2	,	0x06D3	,	0x0000),
DEADTRANS(	0x0654	,	0x06D5	,	0x06C0	,	0x0000),
DEADTRANS(	0x0655	,	0x0627	,	0x0625	,	0x0000),
DEADTRANS(	0x093C	,	0x0915	,	0x0958	,	0x0000),
DEADTRANS(	0x093C	,	0x0916	,	0x0959	,	0x0000),
DEADTRANS(	0x093C	,	0x0917	,	0x095A	,	0x0000),
DEADTRANS(	0x093C	,	0x091C	,	0x095B	,	0x0000),
DEADTRANS(	0x093C	,	0x0921	,	0x095C	,	0x0000),
DEADTRANS(	0x093C	,	0x0922	,	0x095D	,	0x0000),
DEADTRANS(	0x093C	,	0x0928	,	0x0929	,	0x0000),
DEADTRANS(	0x093C	,	0x092B	,	0x095E	,	0x0000),
DEADTRANS(	0x093C	,	0x092F	,	0x095F	,	0x0000),
DEADTRANS(	0x093C	,	0x0930	,	0x0931	,	0x0000),
DEADTRANS(	0x093C	,	0x0933	,	0x0934	,	0x0000),
DEADTRANS(	0x09BC	,	0x09A1	,	0x09DC	,	0x0000),
DEADTRANS(	0x09BC	,	0x09A2	,	0x09DD	,	0x0000),
DEADTRANS(	0x09BC	,	0x09AF	,	0x09DF	,	0x0000),
DEADTRANS(	0x09C7	,	0x09BE	,	0x09CB	,	0x0000),
DEADTRANS(	0x09C7	,	0x09D7	,	0x09CC	,	0x0000),
DEADTRANS(	0x0A3C	,	0x0A16	,	0x0A59	,	0x0000),
DEADTRANS(	0x0A3C	,	0x0A17	,	0x0A5A	,	0x0000),
DEADTRANS(	0x0A3C	,	0x0A1C	,	0x0A5B	,	0x0000),
DEADTRANS(	0x0A3C	,	0x0A2B	,	0x0A5E	,	0x0000),
DEADTRANS(	0x0A3C	,	0x0A32	,	0x0A33	,	0x0000),
DEADTRANS(	0x0A3C	,	0x0A38	,	0x0A36	,	0x0000),
DEADTRANS(	0x0B3C	,	0x0B21	,	0x0B5C	,	0x0000),
DEADTRANS(	0x0B3C	,	0x0B22	,	0x0B5D	,	0x0000),
DEADTRANS(	0x0B47	,	0x0B3E	,	0x0B4B	,	0x0000),
DEADTRANS(	0x0B47	,	0x0B56	,	0x0B48	,	0x0000),
DEADTRANS(	0x0B47	,	0x0B57	,	0x0B4C	,	0x0000),
DEADTRANS(	0x0BC6	,	0x0BBE	,	0x0BCA	,	0x0000),
DEADTRANS(	0x0BC6	,	0x0BD7	,	0x0BCC	,	0x0000),
DEADTRANS(	0x0BC7	,	0x0BBE	,	0x0BCB	,	0x0000),
DEADTRANS(	0x0BD7	,	0x0B92	,	0x0B94	,	0x0000),
DEADTRANS(	0x0C46	,	0x0C56	,	0x0C48	,	0x0000),
DEADTRANS(	0x0CBF	,	0x0CD5	,	0x0CC0	,	0x0000),
DEADTRANS(	0x0CC6	,	0x0CC2	,	0x0CCA	,	0x0000),
DEADTRANS(	0x0CC6	,	0x0CD5	,	0x0CC7	,	0x0000),
DEADTRANS(	0x0CC6	,	0x0CD6	,	0x0CC8	,	0x0000),
DEADTRANS(	0x0CCA	,	0x0CD5	,	0x0CCB	,	0x0000),
DEADTRANS(	0x0D46	,	0x0D3E	,	0x0D4A	,	0x0000),
DEADTRANS(	0x0D46	,	0x0D57	,	0x0D4C	,	0x0000),
DEADTRANS(	0x0D47	,	0x0D3E	,	0x0D4B	,	0x0000),
DEADTRANS(	0x0DD9	,	0x0DCA	,	0x0DDA	,	0x0000),
DEADTRANS(	0x0DD9	,	0x0DCF	,	0x0DDC	,	0x0000),
DEADTRANS(	0x0DD9	,	0x0DDF	,	0x0DDE	,	0x0000),
DEADTRANS(	0x0DDC	,	0x0DCA	,	0x0DDD	,	0x0000),
DEADTRANS(	0x0F71	,	0x0F72	,	0x0F73	,	0x0000),
DEADTRANS(	0x0F71	,	0x0F74	,	0x0F75	,	0x0000),
DEADTRANS(	0x0F71	,	0x0F80	,	0x0F81	,	0x0000),
DEADTRANS(	0x0F90	,	0x0FB5	,	0x0FB9	,	0x0000),
DEADTRANS(	0x0F92	,	0x0FB7	,	0x0F93	,	0x0000),
DEADTRANS(	0x0F9C	,	0x0FB7	,	0x0F9D	,	0x0000),
DEADTRANS(	0x0FA1	,	0x0FB7	,	0x0FA2	,	0x0000),
DEADTRANS(	0x0FA6	,	0x0FB7	,	0x0FA7	,	0x0000),
DEADTRANS(	0x0FAB	,	0x0FB7	,	0x0FAC	,	0x0000),
DEADTRANS(	0x0FB2	,	0x0F80	,	0x0F76	,	0x0000),
DEADTRANS(	0x0FB3	,	0x0F80	,	0x0F78	,	0x0000),
DEADTRANS(	0x0FB5	,	0x0F40	,	0x0F69	,	0x0000),
DEADTRANS(	0x0FB7	,	0x0F42	,	0x0F43	,	0x0000),
DEADTRANS(	0x0FB7	,	0x0F4C	,	0x0F4D	,	0x0000),
DEADTRANS(	0x0FB7	,	0x0F51	,	0x0F52	,	0x0000),
DEADTRANS(	0x0FB7	,	0x0F56	,	0x0F57	,	0x0000),
DEADTRANS(	0x0FB7	,	0x0F5B	,	0x0F5C	,	0x0000),
DEADTRANS(	0x102E	,	0x1025	,	0x1026	,	0x0000),
DEADTRANS(	0x1100	,	0x1100	,	0x1101	,	0x0000),
DEADTRANS(	0x1102	,	0x1100	,	0x1113	,	0x0000),
DEADTRANS(	0x1102	,	0x1102	,	0x1114	,	0x0000),
DEADTRANS(	0x1102	,	0x1103	,	0x1115	,	0x0000),
DEADTRANS(	0x1102	,	0x1107	,	0x1116	,	0x0000),
DEADTRANS(	0x1103	,	0x1100	,	0x1117	,	0x0000),
DEADTRANS(	0x1103	,	0x1103	,	0x1104	,	0x0000),
DEADTRANS(	0x1105	,	0x1102	,	0x1118	,	0x0000),
DEADTRANS(	0x1105	,	0x1105	,	0x1119	,	0x0000),
DEADTRANS(	0x1105	,	0x110B	,	0x111B	,	0x0000),
DEADTRANS(	0x1105	,	0x1112	,	0x111A	,	0x0000),
DEADTRANS(	0x1106	,	0x1107	,	0x111C	,	0x0000),
DEADTRANS(	0x1106	,	0x110B	,	0x111D	,	0x0000),
DEADTRANS(	0x1107	,	0x1100	,	0x111E	,	0x0000),
DEADTRANS(	0x1107	,	0x1102	,	0x111F	,	0x0000),
DEADTRANS(	0x1107	,	0x1103	,	0x1120	,	0x0000),
DEADTRANS(	0x1107	,	0x1107	,	0x1108	,	0x0000),
DEADTRANS(	0x1107	,	0x1109	,	0x1121	,	0x0000),
DEADTRANS(	0x1107	,	0x110A	,	0x1125	,	0x0000),
DEADTRANS(	0x1107	,	0x110B	,	0x112B	,	0x0000),
DEADTRANS(	0x1107	,	0x110C	,	0x1127	,	0x0000),
DEADTRANS(	0x1107	,	0x110E	,	0x1128	,	0x0000),
DEADTRANS(	0x1107	,	0x1110	,	0x1129	,	0x0000),
DEADTRANS(	0x1107	,	0x1111	,	0x112A	,	0x0000),
DEADTRANS(	0x1107	,	0x112B	,	0x112C	,	0x0000),
DEADTRANS(	0x1107	,	0x112D	,	0x1122	,	0x0000),
DEADTRANS(	0x1107	,	0x112F	,	0x1123	,	0x0000),
DEADTRANS(	0x1107	,	0x1132	,	0x1124	,	0x0000),
DEADTRANS(	0x1107	,	0x1136	,	0x1126	,	0x0000),
DEADTRANS(	0x1108	,	0x110B	,	0x112C	,	0x0000),
DEADTRANS(	0x1109	,	0x1100	,	0x112D	,	0x0000),
DEADTRANS(	0x1109	,	0x1102	,	0x112E	,	0x0000),
DEADTRANS(	0x1109	,	0x1103	,	0x112F	,	0x0000),
DEADTRANS(	0x1109	,	0x1105	,	0x1130	,	0x0000),
DEADTRANS(	0x1109	,	0x1106	,	0x1131	,	0x0000),
DEADTRANS(	0x1109	,	0x1107	,	0x1132	,	0x0000),
DEADTRANS(	0x1109	,	0x1109	,	0x110A	,	0x0000),
DEADTRANS(	0x1109	,	0x110A	,	0x1134	,	0x0000),
DEADTRANS(	0x1109	,	0x110B	,	0x1135	,	0x0000),
DEADTRANS(	0x1109	,	0x110C	,	0x1136	,	0x0000),
DEADTRANS(	0x1109	,	0x110E	,	0x1137	,	0x0000),
DEADTRANS(	0x1109	,	0x110F	,	0x1138	,	0x0000),
DEADTRANS(	0x1109	,	0x1110	,	0x1139	,	0x0000),
DEADTRANS(	0x1109	,	0x1111	,	0x113A	,	0x0000),
DEADTRANS(	0x1109	,	0x1112	,	0x113B	,	0x0000),
DEADTRANS(	0x1109	,	0x111E	,	0x1133	,	0x0000),
DEADTRANS(	0x110A	,	0x1109	,	0x1134	,	0x0000),
DEADTRANS(	0x110B	,	0x1100	,	0x1141	,	0x0000),
DEADTRANS(	0x110B	,	0x1103	,	0x1142	,	0x0000),
DEADTRANS(	0x110B	,	0x1106	,	0x1143	,	0x0000),
DEADTRANS(	0x110B	,	0x1107	,	0x1144	,	0x0000),
DEADTRANS(	0x110B	,	0x1109	,	0x1145	,	0x0000),
DEADTRANS(	0x110B	,	0x110B	,	0x1147	,	0x0000),
DEADTRANS(	0x110B	,	0x110C	,	0x1148	,	0x0000),
DEADTRANS(	0x110B	,	0x110E	,	0x1149	,	0x0000),
DEADTRANS(	0x110B	,	0x1110	,	0x114A	,	0x0000),
DEADTRANS(	0x110B	,	0x1111	,	0x114B	,	0x0000),
DEADTRANS(	0x110B	,	0x1140	,	0x1146	,	0x0000),
DEADTRANS(	0x110C	,	0x110B	,	0x114D	,	0x0000),
DEADTRANS(	0x110C	,	0x110C	,	0x110D	,	0x0000),
DEADTRANS(	0x110E	,	0x110F	,	0x1152	,	0x0000),
DEADTRANS(	0x110E	,	0x1112	,	0x1153	,	0x0000),
DEADTRANS(	0x1111	,	0x1107	,	0x1156	,	0x0000),
DEADTRANS(	0x1111	,	0x110B	,	0x1157	,	0x0000),
DEADTRANS(	0x1112	,	0x1112	,	0x1158	,	0x0000),
DEADTRANS(	0x1121	,	0x1100	,	0x1122	,	0x0000),
DEADTRANS(	0x1121	,	0x1103	,	0x1123	,	0x0000),
DEADTRANS(	0x1121	,	0x1107	,	0x1124	,	0x0000),
DEADTRANS(	0x1121	,	0x1109	,	0x1125	,	0x0000),
DEADTRANS(	0x1121	,	0x110C	,	0x1126	,	0x0000),
DEADTRANS(	0x1132	,	0x1100	,	0x1133	,	0x0000),
DEADTRANS(	0x113C	,	0x113C	,	0x113D	,	0x0000),
DEADTRANS(	0x113E	,	0x113E	,	0x113F	,	0x0000),
DEADTRANS(	0x114E	,	0x114E	,	0x114F	,	0x0000),
DEADTRANS(	0x1150	,	0x1150	,	0x1151	,	0x0000),
DEADTRANS(	0x1161	,	0x1169	,	0x1176	,	0x0000),
DEADTRANS(	0x1161	,	0x116E	,	0x1177	,	0x0000),
DEADTRANS(	0x1161	,	0x1175	,	0x1162	,	0x0000),
DEADTRANS(	0x1163	,	0x1169	,	0x1178	,	0x0000),
DEADTRANS(	0x1163	,	0x116D	,	0x1179	,	0x0000),
DEADTRANS(	0x1163	,	0x1175	,	0x1164	,	0x0000),
DEADTRANS(	0x1165	,	0x1169	,	0x117A	,	0x0000),
DEADTRANS(	0x1165	,	0x116E	,	0x117B	,	0x0000),
DEADTRANS(	0x1165	,	0x1173	,	0x117C	,	0x0000),
DEADTRANS(	0x1165	,	0x1175	,	0x1166	,	0x0000),
DEADTRANS(	0x1167	,	0x1169	,	0x117D	,	0x0000),
DEADTRANS(	0x1167	,	0x116E	,	0x117E	,	0x0000),
DEADTRANS(	0x1167	,	0x1175	,	0x1168	,	0x0000),
DEADTRANS(	0x1169	,	0x1161	,	0x116A	,	0x0000),
DEADTRANS(	0x1169	,	0x1162	,	0x116B	,	0x0000),
DEADTRANS(	0x1169	,	0x1165	,	0x117F	,	0x0000),
DEADTRANS(	0x1169	,	0x1166	,	0x1180	,	0x0000),
DEADTRANS(	0x1169	,	0x1168	,	0x1181	,	0x0000),
DEADTRANS(	0x1169	,	0x1169	,	0x1182	,	0x0000),
DEADTRANS(	0x1169	,	0x116E	,	0x1183	,	0x0000),
DEADTRANS(	0x1169	,	0x1175	,	0x116C	,	0x0000),
DEADTRANS(	0x116A	,	0x1175	,	0x116B	,	0x0000),
DEADTRANS(	0x116D	,	0x1163	,	0x1184	,	0x0000),
DEADTRANS(	0x116D	,	0x1164	,	0x1185	,	0x0000),
DEADTRANS(	0x116D	,	0x1167	,	0x1186	,	0x0000),
DEADTRANS(	0x116D	,	0x1169	,	0x1187	,	0x0000),
DEADTRANS(	0x116D	,	0x1175	,	0x1188	,	0x0000),
DEADTRANS(	0x116E	,	0x1161	,	0x1189	,	0x0000),
DEADTRANS(	0x116E	,	0x1162	,	0x118A	,	0x0000),
DEADTRANS(	0x116E	,	0x1165	,	0x116F	,	0x0000),
DEADTRANS(	0x116E	,	0x1166	,	0x1170	,	0x0000),
DEADTRANS(	0x116E	,	0x1168	,	0x118C	,	0x0000),
DEADTRANS(	0x116E	,	0x116E	,	0x118D	,	0x0000),
DEADTRANS(	0x116E	,	0x1175	,	0x1171	,	0x0000),
DEADTRANS(	0x116E	,	0x117C	,	0x118B	,	0x0000),
DEADTRANS(	0x116F	,	0x1173	,	0x118B	,	0x0000),
DEADTRANS(	0x116F	,	0x1175	,	0x1170	,	0x0000),
DEADTRANS(	0x1172	,	0x1161	,	0x118E	,	0x0000),
DEADTRANS(	0x1172	,	0x1165	,	0x118F	,	0x0000),
DEADTRANS(	0x1172	,	0x1166	,	0x1190	,	0x0000),
DEADTRANS(	0x1172	,	0x1167	,	0x1191	,	0x0000),
DEADTRANS(	0x1172	,	0x1168	,	0x1192	,	0x0000),
DEADTRANS(	0x1172	,	0x116E	,	0x1193	,	0x0000),
DEADTRANS(	0x1172	,	0x1175	,	0x1194	,	0x0000),
DEADTRANS(	0x1173	,	0x116E	,	0x1195	,	0x0000),
DEADTRANS(	0x1173	,	0x1173	,	0x1196	,	0x0000),
DEADTRANS(	0x1173	,	0x1175	,	0x1174	,	0x0000),
DEADTRANS(	0x1174	,	0x116E	,	0x1197	,	0x0000),
DEADTRANS(	0x1175	,	0x1161	,	0x1198	,	0x0000),
DEADTRANS(	0x1175	,	0x1163	,	0x1199	,	0x0000),
DEADTRANS(	0x1175	,	0x1169	,	0x119A	,	0x0000),
DEADTRANS(	0x1175	,	0x116E	,	0x119B	,	0x0000),
DEADTRANS(	0x1175	,	0x1173	,	0x119C	,	0x0000),
DEADTRANS(	0x1175	,	0x119E	,	0x119D	,	0x0000),
DEADTRANS(	0x119E	,	0x1165	,	0x119F	,	0x0000),
DEADTRANS(	0x119E	,	0x116E	,	0x11A0	,	0x0000),
DEADTRANS(	0x119E	,	0x1175	,	0x11A1	,	0x0000),
DEADTRANS(	0x119E	,	0x119E	,	0x11A2	,	0x0000),
DEADTRANS(	0x11A8	,	0x11A8	,	0x11A9	,	0x0000),
DEADTRANS(	0x11A8	,	0x11AF	,	0x11C3	,	0x0000),
DEADTRANS(	0x11A8	,	0x11BA	,	0x11AA	,	0x0000),
DEADTRANS(	0x11A8	,	0x11E7	,	0x11C4	,	0x0000),
DEADTRANS(	0x11AA	,	0x11A8	,	0x11C4	,	0x0000),
DEADTRANS(	0x11AB	,	0x11A8	,	0x11C5	,	0x0000),
DEADTRANS(	0x11AB	,	0x11AE	,	0x11C6	,	0x0000),
DEADTRANS(	0x11AB	,	0x11BA	,	0x11C7	,	0x0000),
DEADTRANS(	0x11AB	,	0x11BD	,	0x11AC	,	0x0000),
DEADTRANS(	0x11AB	,	0x11C0	,	0x11C9	,	0x0000),
DEADTRANS(	0x11AB	,	0x11C2	,	0x11AD	,	0x0000),
DEADTRANS(	0x11AB	,	0x11EB	,	0x11C8	,	0x0000),
DEADTRANS(	0x11AE	,	0x11A8	,	0x11CA	,	0x0000),
DEADTRANS(	0x11AE	,	0x11AF	,	0x11CB	,	0x0000),
DEADTRANS(	0x11AF	,	0x11A8	,	0x11B0	,	0x0000),
DEADTRANS(	0x11AF	,	0x11AA	,	0x11CC	,	0x0000),
DEADTRANS(	0x11AF	,	0x11AB	,	0x11CD	,	0x0000),
DEADTRANS(	0x11AF	,	0x11AE	,	0x11CE	,	0x0000),
DEADTRANS(	0x11AF	,	0x11AF	,	0x11D0	,	0x0000),
DEADTRANS(	0x11AF	,	0x11B7	,	0x11B1	,	0x0000),
DEADTRANS(	0x11AF	,	0x11B8	,	0x11B2	,	0x0000),
DEADTRANS(	0x11AF	,	0x11B9	,	0x11D3	,	0x0000),
DEADTRANS(	0x11AF	,	0x11BA	,	0x11B3	,	0x0000),
DEADTRANS(	0x11AF	,	0x11BB	,	0x11D6	,	0x0000),
DEADTRANS(	0x11AF	,	0x11BF	,	0x11D8	,	0x0000),
DEADTRANS(	0x11AF	,	0x11C0	,	0x11B4	,	0x0000),
DEADTRANS(	0x11AF	,	0x11C1	,	0x11B5	,	0x0000),
DEADTRANS(	0x11AF	,	0x11C2	,	0x11B6	,	0x0000),
DEADTRANS(	0x11AF	,	0x11DA	,	0x11D1	,	0x0000),
DEADTRANS(	0x11AF	,	0x11DD	,	0x11D2	,	0x0000),
DEADTRANS(	0x11AF	,	0x11E5	,	0x11D4	,	0x0000),
DEADTRANS(	0x11AF	,	0x11E6	,	0x11D5	,	0x0000),
DEADTRANS(	0x11AF	,	0x11EB	,	0x11D7	,	0x0000),
DEADTRANS(	0x11AF	,	0x11F9	,	0x11D9	,	0x0000),
DEADTRANS(	0x11B0	,	0x11BA	,	0x11CC	,	0x0000),
DEADTRANS(	0x11B1	,	0x11A8	,	0x11D1	,	0x0000),
DEADTRANS(	0x11B1	,	0x11BA	,	0x11D2	,	0x0000),
DEADTRANS(	0x11B2	,	0x11BA	,	0x11D3	,	0x0000),
DEADTRANS(	0x11B2	,	0x11BC	,	0x11D5	,	0x0000),
DEADTRANS(	0x11B2	,	0x11C2	,	0x11D4	,	0x0000),
DEADTRANS(	0x11B3	,	0x11BA	,	0x11D6	,	0x0000),
DEADTRANS(	0x11B7	,	0x11A8	,	0x11DA	,	0x0000),
DEADTRANS(	0x11B7	,	0x11AF	,	0x11DB	,	0x0000),
DEADTRANS(	0x11B7	,	0x11B8	,	0x11DC	,	0x0000),
DEADTRANS(	0x11B7	,	0x11BA	,	0x11DD	,	0x0000),
DEADTRANS(	0x11B7	,	0x11BB	,	0x11DE	,	0x0000),
DEADTRANS(	0x11B7	,	0x11BC	,	0x11E2	,	0x0000),
DEADTRANS(	0x11B7	,	0x11BE	,	0x11E0	,	0x0000),
DEADTRANS(	0x11B7	,	0x11C2	,	0x11E1	,	0x0000),
DEADTRANS(	0x11B7	,	0x11EB	,	0x11DF	,	0x0000),
DEADTRANS(	0x11B8	,	0x11AF	,	0x11E3	,	0x0000),
DEADTRANS(	0x11B8	,	0x11BA	,	0x11B9	,	0x0000),
DEADTRANS(	0x11B8	,	0x11BC	,	0x11E6	,	0x0000),
DEADTRANS(	0x11B8	,	0x11C1	,	0x11E4	,	0x0000),
DEADTRANS(	0x11B8	,	0x11C2	,	0x11E5	,	0x0000),
DEADTRANS(	0x11BA	,	0x11A8	,	0x11E7	,	0x0000),
DEADTRANS(	0x11BA	,	0x11AE	,	0x11E8	,	0x0000),
DEADTRANS(	0x11BA	,	0x11AF	,	0x11E9	,	0x0000),
DEADTRANS(	0x11BA	,	0x11B8	,	0x11EA	,	0x0000),
DEADTRANS(	0x11BA	,	0x11BA	,	0x11BB	,	0x0000),
DEADTRANS(	0x11BC	,	0x11A8	,	0x11EC	,	0x0000),
DEADTRANS(	0x11BC	,	0x11A9	,	0x11ED	,	0x0000),
DEADTRANS(	0x11BC	,	0x11BC	,	0x11EE	,	0x0000),
DEADTRANS(	0x11BC	,	0x11BF	,	0x11EF	,	0x0000),
DEADTRANS(	0x11C1	,	0x11B8	,	0x11F3	,	0x0000),
DEADTRANS(	0x11C1	,	0x11BC	,	0x11F4	,	0x0000),
DEADTRANS(	0x11C2	,	0x11AB	,	0x11F5	,	0x0000),
DEADTRANS(	0x11C2	,	0x11AF	,	0x11F6	,	0x0000),
DEADTRANS(	0x11C2	,	0x11B7	,	0x11F7	,	0x0000),
DEADTRANS(	0x11C2	,	0x11B8	,	0x11F8	,	0x0000),
DEADTRANS(	0x11CE	,	0x11C2	,	0x11CF	,	0x0000),
DEADTRANS(	0x11DD	,	0x11BA	,	0x11DE	,	0x0000),
DEADTRANS(	0x11EC	,	0x11A8	,	0x11ED	,	0x0000),
DEADTRANS(	0x11F0	,	0x11BA	,	0x11F1	,	0x0000),
DEADTRANS(	0x11F0	,	0x11EB	,	0x11F2	,	0x0000),
DEADTRANS(	greek_psili	,	apostrophe	,	0x1FCE	,	0x0000),
DEADTRANS(	greek_psili	,	grave	,	0x1FCD	,	0x0000),
DEADTRANS(	greek_psili	,	grave	,	0x1FCD	,	0x0000),
DEADTRANS(	greek_psili	,	0x007E	,	0x1FCF	,	0x0000),
DEADTRANS(	greek_psili	,	acute	,	0x1FCE	,	0x0000),
DEADTRANS(	greek_psili	,	acute	,	0x1FCE	,	0x0000),
DEADTRANS(	greek_psili	,	0x00f1	,	0x1FCF	,	0x0000),
DEADTRANS(	greek_psili	,	0x0342	,	0x1FCF	,	0x0000),
DEADTRANS(	greek_dasia	,	apostrophe	,	0x1FDE	,	0x0000),
DEADTRANS(	greek_dasia	,	grave	,	0x1FDD	,	0x0000),
DEADTRANS(	greek_dasia	,	grave	,	0x1FDD	,	0x0000),
DEADTRANS(	greek_dasia	,	0x007E	,	0x1FDF	,	0x0000),
DEADTRANS(	greek_dasia	,	acute	,	0x1FDE	,	0x0000),
DEADTRANS(	greek_dasia	,	acute	,	0x1FDE	,	0x0000),
DEADTRANS(	greek_dasia	,	0x00f1	,	0x1FDF	,	0x0000),
DEADTRANS(	greek_dasia	,	0x0342	,	0x1FDF	,	0x0000),
DEADTRANS(	thereexists	,	0x0338	,	0x2204	,	0x0000),
DEADTRANS(	elementof	,	0x0338	,	0x2209	,	0x0000),
DEADTRANS(	0x220B	,	0x0338	,	0x220C	,	0x0000),
DEADTRANS(	0x2223	,	0x0338	,	0x2224	,	0x0000),
DEADTRANS(	parallelto	,	0x0338	,	0x2226	,	0x0000),
DEADTRANS(	0x223C	,	0x0338	,	0x2241	,	0x0000),
DEADTRANS(	0x223C	,	0x0338	,	0x2247	,	0x0000),
DEADTRANS(	0x2243	,	0x0338	,	0x2244	,	0x0000),
DEADTRANS(	0x2248	,	0x0338	,	0x2249	,	0x0000),
DEADTRANS(	0x224D	,	0x0338	,	0x226D	,	0x0000),
DEADTRANS(	0x2261	,	0x0338	,	0x2262	,	0x0000),
DEADTRANS(	0x2264	,	0x0338	,	0x2270	,	0x0000),
DEADTRANS(	0x2265	,	0x0338	,	0x2271	,	0x0000),
DEADTRANS(	0x2272	,	0x0338	,	0x2274	,	0x0000),
DEADTRANS(	0x2273	,	0x0338	,	0x2275	,	0x0000),
DEADTRANS(	0x2276	,	0x0338	,	0x2278	,	0x0000),
DEADTRANS(	0x2277	,	0x0338	,	0x2279	,	0x0000),
DEADTRANS(	0x227A	,	0x0338	,	0x2280	,	0x0000),
DEADTRANS(	0x227B	,	0x0338	,	0x2281	,	0x0000),
DEADTRANS(	0x227C	,	0x0338	,	0x22E0	,	0x0000),
DEADTRANS(	0x227D	,	0x0338	,	0x22E1	,	0x0000),
DEADTRANS(	includedin	,	0x0338	,	0x2284	,	0x0000),
DEADTRANS(	includedin	,	0x0338	,	0x2284	,	0x0000),
DEADTRANS(	0x2283	,	0x0338	,	0x2285	,	0x0000),
DEADTRANS(	0x2283	,	0x0338	,	0x2285	,	0x0000),
DEADTRANS(	0x2286	,	0x0338	,	0x2288	,	0x0000),
DEADTRANS(	0x2287	,	0x0338	,	0x2289	,	0x0000),
DEADTRANS(	0x2291	,	0x0338	,	0x22E2	,	0x0000),
DEADTRANS(	0x2292	,	0x0338	,	0x22E3	,	0x0000),
DEADTRANS(	0x22A3	,	0x0338	,	0x22AC	,	0x0000),
DEADTRANS(	0x22A8	,	0x0338	,	0x22AD	,	0x0000),
DEADTRANS(	0x22A9	,	0x0338	,	0x22AE	,	0x0000),
DEADTRANS(	0x22AB	,	0x0338	,	0x22AF	,	0x0000),
DEADTRANS(	0x22B2	,	0x0338	,	0x22EA	,	0x0000),
DEADTRANS(	0x22B3	,	0x0338	,	0x22EB	,	0x0000),
DEADTRANS(	0x22B4	,	0x0338	,	0x22EC	,	0x0000),
DEADTRANS(	0x22B5	,	0x0338	,	0x22ED	,	0x0000),
DEADTRANS(	0x2ADD	,	0x0338	,	0x2ADC	,	0x0000),
    0, 0
};
